package com.Geister;

import com.PacketAndStateObjectDLL.Packet;
import com.PacketAndStateObjectDLL.StateObject;
import com.PlayerInfoDLL.PlayerInfo;
import com.RoomDLL.GameRoom;
import com.RoomDLL.RoomState;

import java.io.*;
import java.net.Socket;
import java.nio.charset.StandardCharsets;
import java.text.DateFormatSymbols;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

public class RoomGST extends GameRoom {
    private Object SyncRoot;
    private Timer timer;
    private ArrayList<Player> listPlayer;

    private String gameType;		
    private String hostName;		//開房者名稱
    private String hostPlayerKind;	//0: AI, 1: human
    private String readMidBoard;  //開房者是否斷線覆盤
    private String restartTimes;  //開房者要玩幾盤
    private String takeColor;     //開房者是先手或後手
    private String changeFirst;   //先後手是否互換

    private String setSecond;       //時間長度
    private String plySecondLimit;  //計時方式(0: 回合制 or 大於0: 單手制)(目前沒有實作單手制)
    private String groupPly;        //幾手共用
//    private String maxPositionRepetionCount; //盤面最多重複次數
    private String maxGamePlyCount;       //最多無吃翻次數
    
    // WDLBonusScore
    private double winScore;
    private double drawScore;
    private double loseScore;
    private double[] pieceScore;
    private double bonus;
    private double bonuslimit;
    
    private int gameCount = 0;
    private int whoseTurn;          //下一手是誰 0: player1, 1: player2
    private int capturePieceColor = 0;  //0: none  1:RED  2:BLUE
    
    private RoomGSTMode roomGSTMode;
    private RoomGSTTable roomGSTTable;
    private RoomGSTJudgement roomGSTJudgement;
    private ProcessRecoder processRecoder;
    
    // only LocalServer
    private String undoStepList, undoTimeList, undogameStarted;
    private boolean isUndo = false, undoIsSwapColor;
    private ArrayList<String> arrBoardInfo = new ArrayList<String>();
    private ArrayList<String> arrBoardName = new ArrayList<String>();
    private int boardRepeatCount;
    private boolean isGameFinsh;
    
    public RoomGST(int roomId, RoomState.roomState roomState, String strRoomInfo, PlayerInfo playerInfo)//初始化房間信息
    {
        super(roomId, roomState);
        SyncRoot = new Object();
        timer = new Timer();
        timer.schedule(new TimeTask(), 1000, 1000);
        listPlayer = new ArrayList<Player>();
        this.winScore = 1;
        this.loseScore = 0;
        this.drawScore = 0.5;

        String[] roomInfo = strRoomInfo.split(",");
        this.gameType = roomInfo[0]; //應該要是 "Geister";
        this.hostName = roomInfo[1];
        this.hostPlayerKind = roomInfo[2];

        this.readMidBoard = roomInfo[3];
        this.restartTimes = roomInfo[4];
        this.takeColor = roomInfo[5];
        this.changeFirst = roomInfo[6];

        this.setSecond = roomInfo[7];
        this.plySecondLimit = roomInfo[8];
        this.groupPly = roomInfo[9];
        this.maxGamePlyCount = roomInfo[10];

        
        {
            RoomGST.DEBUG("strRoomInfo = " + strRoomInfo);
            RoomGST.DEBUG("gameType = " + gameType);
            RoomGST.DEBUG("hostName = " + hostName);
            RoomGST.DEBUG("hostPlayerKind = " + hostPlayerKind);
            RoomGST.DEBUG("readMidBoard = " + readMidBoard);
            RoomGST.DEBUG("restartTimes = " + restartTimes);
            RoomGST.DEBUG("takeColor = " + takeColor);
            RoomGST.DEBUG("changeFirst = " + changeFirst);
            RoomGST.DEBUG("setSecond = " + setSecond);
            RoomGST.DEBUG("plySecondLimit = " + plySecondLimit);
            RoomGST.DEBUG("groupPly = " + groupPly);
//            RoomGST.DEBUG("maxPositionRepetionCount = " + maxPositionRepetionCount);
            RoomGST.DEBUG("maxGamePlyCount = " + maxGamePlyCount);
        }

        processRecoder = new ProcessRecoder();
    }

    @Override
    public String getRoomInfo()
    {
        String roomInfo = "";
        roomInfo += String.format("%d,%s,%s,%s,%s,%s,%s,", super.getRoomId(), super.getRoomState(), hostName, hostPlayerKind, readMidBoard, restartTimes, takeColor);
        roomInfo += String.format("%s,%s,%s,%s,%s", changeFirst, setSecond, plySecondLimit, groupPly, maxGamePlyCount);
        return roomInfo;
    }

    @Override
    public String getGameType()
    {
        return gameType;
    }

    @Override
    public boolean roomSetBoard(String boardInfo, StringBuilder setBoardResult)//設定遊戲的棋盤狀態，包括初始化棋盤和判斷物件
    {
    	String result = "Unknown Error";
        
        boardRepeatCount = Integer.parseInt(boardInfo.split(",")[0]);
        // 刪除牌面重複次數 see Client.Game.java line 153
        boardInfo = boardInfo.substring(boardInfo.indexOf(',') + 1); // 刪除後boardInfo="INITIAL,0,......."
        if (readMidBoard.equals("0") || readMidBoard.equals("1")) // client 使用預設盤面  //中盤對下 // client 提供盤面 //中盤對下
        {
            int moveLength, eatLength, timeLength;
        	while(!boardInfo.equals("")) {//boardInfo不在GSTlocal 創建
        		roomGSTMode = new RoomGSTMidBoard();  //用來放盤面資訊
        		moveLength = Integer.parseInt(boardInfo.split(",")[42]);
        		eatLength = Integer.parseInt(boardInfo.split(",")[43 + moveLength]);
        		timeLength = Integer.parseInt(boardInfo.split(",")[44 + moveLength + eatLength]);
//            	timeLength = Integer.parseInt(boardInfo.split(",")[43 + moveLength]);
            	int index = 0;
            	for(int i = 1; i <= 44 + moveLength + eatLength + 1 + timeLength; i++) {
            		index = boardInfo.indexOf(',', index) == -1 ? boardInfo.length() : boardInfo.indexOf(',', index) + 1;
            	}
//            	for(int i = 1; i <= 49 + moveLength + 1 + timeLength; i++) {
//            		index = boardInfo.indexOf(',', index) == -1 ? boardInfo.length() : boardInfo.indexOf(',', index) + 1;
//            	}
            		
        		arrBoardInfo.add(boardInfo.substring(0, index));
        		boardInfo = boardInfo.substring(index, boardInfo.length()); //最後boardInfo應該會等於""，以結束迴圈
        		result = roomGSTMode.setInitialInformation(arrBoardInfo.get(arrBoardInfo.size() - 1));
        		if (!result.equals("Success")) {
        			setBoardResult.append(result);
        			return false;
        		}else {
        			roomGSTTable = new RoomGSTTable();
        	        roomGSTJudgement = new RoomGSTJudgement(String.format("%s,%s,%s,%s", setSecond, plySecondLimit, groupPly, maxGamePlyCount));
        			String initialChessBoard = roomGSTMode.getInitialChessBoard(), initialRemainGhostCount = roomGSTMode.getinitialRemainGhostCount();
        	        String initialStepList = roomGSTMode.getInitialStepList(), initialTimeList = roomGSTMode.getInitialTimeList(), initialgameStarted = roomGSTMode.getInitialgameStarted();
        	        result = initializeTableAndJudgement(roomGSTTable, roomGSTJudgement, initialChessBoard, initialRemainGhostCount, initialStepList, initialTimeList, initialgameStarted);
				}
        		if(!result.equals("Success")) {
        			setBoardResult.append(result);
        			return false;
        		}
        	}
        	roomGSTMode = new RoomGSTMidBoard();
        	result = roomGSTMode.setInitialInformation(arrBoardInfo.get(gameCount / boardRepeatCount % arrBoardInfo.size()));
        }
        else if (readMidBoard.equals("2")) // client 不提供盤面 //斷線中盤 //先後手依照原本盤面的整場先後手
        {
        	// Local Server doesn't provide
        	setBoardResult.append("Local Server doesn't provide");
            return false;
        }
        if (!result.equals("Success"))
        {
        	setBoardResult.append(result);
            return false;
        }

        roomGSTTable = new RoomGSTTable();
        roomGSTJudgement = new RoomGSTJudgement(String.format("%s,%s,%s,%s", setSecond, plySecondLimit, groupPly, maxGamePlyCount));
        
        String initialChessBoard = roomGSTMode.getInitialChessBoard(), initialRemainGhostCount = roomGSTMode.getinitialRemainGhostCount();
        String initialStepList = roomGSTMode.getInitialStepList(), initialTimeList = roomGSTMode.getInitialTimeList(), initialgameStarted = roomGSTMode.getInitialgameStarted();
        result = initializeTableAndJudgement(roomGSTTable, roomGSTJudgement, initialChessBoard, initialRemainGhostCount, initialStepList, initialTimeList, initialgameStarted);
        if (!result.equals("Success"))
        {
        	setBoardResult.append(result);
            return false;
        }
        return true;
    }

    @Override
    protected boolean addPlayer(PlayerInfo playerInfo, String context, StringBuilder addPlayerResult)
    {
    	boolean isSuccess = true;
        //String result = "";

        if (listPlayer.size() == 0)
        {
            listPlayer.add(new Player(playerInfo));
            gotoGameRoomReadData(playerInfo.socket);
            return isSuccess;
        }
        else if (listPlayer.size() == 1)
        {
            if (!listPlayer.get(0).getIsAlive())
            {
                isSuccess = false;
                listPlayer.get(0).closeSocket();
                finishGame();

                return isSuccess;
            }

            listPlayer.add(new Player(playerInfo));
            gotoGameRoomReadData(playerInfo.socket);
            RoomGST.DEBUG(String.format("gameReady = %b", super.gameReady()));
            return isSuccess;
        }
        else
        {
            isSuccess = false;
            addPlayerResult.append("You can not enter the room, because someone has entered the room.");
            return isSuccess;
        }
    }
    //改成個別玩家獲得的格式
    public String InitialInformationForPlayer(String initialInfo, int player) {
        String[] parts = initialInfo.split(","); // 分割為前36個元素和其餘部分
        String[] boardArray = new String[36]; // 前36個元素，即棋盤信息
        System.arraycopy(parts, 0, boardArray, 0, 36);
        StringBuilder processedBoard = new StringBuilder();
        String personalInfo = "";
        // 處理棋盤視角
        for (String piece : boardArray) {
            if (player == 0) {
                // Player 1: 看到自己的是 1 和 2，敵人的是 3
                if (piece.equals("4") || piece.equals("5")) {
                    processedBoard.append("3,");
                } else {
                    processedBoard.append(piece).append(",");
                }
            } else if (player == 1) {
                // Player 2: 看到自己的是 4 和 5，敵人的是 3
                if (piece.equals("1") || piece.equals("2")) {
                    processedBoard.append("3,");
                }
                else if (piece.equals("4")) {
                	processedBoard.append("1,");
                }
                else if (piece.equals("5")) {
                	processedBoard.append("2,");
                }
                else {
                    processedBoard.append(piece).append(",");
                }
            }
        }
        // 去掉最後一個逗號
        if (processedBoard.length() > 0) {
            processedBoard.setLength(processedBoard.length() - 1);
        }
        
        // 反轉棋盤 (僅針對 Player 2)
        if (player == 1) {
            personalInfo += reverseBoard(processedBoard.toString()) + ",";
        }else {
        	personalInfo += processedBoard.toString() + ",";
        }
        
        //加回remain ghost and gameStarted
        for (int i = 36; i <= 40; i++) {
        	personalInfo += parts[i] + ",";
        }
        
        //處理steps訊息
        personalInfo += parts[41] + ",";
        int stepsLen = Integer.parseInt(parts[41]);
    	int stepsIndex = 0;
    	for (int i = 42; i < 42 + stepsLen; i++) {
			if (stepsIndex % 2 != player) {
				personalInfo += parts[i].split(" ")[0].toLowerCase() + " ";
				switch(parts[i].split(" ")[1]) {
					case "N":
		        	case "NORTH":
		        		personalInfo += "SOUTH,";
		        		break;
		        	case "S":
		        	case "SOUTH":
		        		personalInfo += "NORTH,";
		        		break;
		        	case "E":	
		        	case "EAST":
		        		personalInfo += "WEST,";
		        		break;
		        	case "W":
		        	case "WEST":
		        		personalInfo += "EAST,";
		        		break; 
				}
			}else {
				personalInfo += parts[i] + ",";
			}
			stepsIndex++;
		}
    	
    	//eats
    	personalInfo += parts[42 + stepsLen] + ",";
    	int eatsLen = Integer.parseInt(parts[42 + stepsLen]);
    	for (int i = 43 + stepsLen; i < 43 + stepsLen + eatsLen; i++) {
    		personalInfo += parts[i] + ",";
    	}
        
    	//times
    	personalInfo += parts[43 + stepsLen + eatsLen] + ",";
    	int timesLen = Integer.parseInt(parts[43 + stepsLen + eatsLen]);
    	for (int i = 44 + stepsLen + eatsLen; i < 44 + stepsLen + eatsLen + timesLen; i++) {
    		personalInfo += parts[i] + ",";
    	}

        

        // 拼接更新的棋盤信息與原有信息
        return RoomGST.TrimEnd(personalInfo, ',');
    }

    private String reverseBoard(String board) {
        String[] pieces = board.split(",");
        StringBuilder reversedBoard = new StringBuilder();
        
        // 反轉棋盤
        for (int i = pieces.length - 1; i >= 0; i--) {
            reversedBoard.append(pieces[i]).append(",");
        }
        
        // 去掉最後一個逗號
        if (reversedBoard.length() > 0) {
            reversedBoard.setLength(reversedBoard.length() - 1);
        }
        
        return reversedBoard.toString();
    }
    @Override
    protected void roomMain()//遊戲的核心執行邏輯
    {
    	try
        {
            if (takeColor.equals("2"))
            {
                Player tempPlayer = listPlayer.get(0);
                listPlayer.set(0, listPlayer.get(1));
                listPlayer.set(1, tempPlayer);
            }

            gameCount = 1;
            while (gameCount <= Integer.parseInt(restartTimes) && !isGameFinsh)
            {
                roomGSTTable.resetRoomGSTTable();
                // 限於localserver 開局庫
                if(!isUndo) {
                	roomGSTMode = new RoomGSTMidBoard();
                	roomGSTMode.setInitialInformation(arrBoardInfo.get((gameCount - 1) / boardRepeatCount % arrBoardInfo.size()));
                }
                
                //InitialSetting: reset Board、restart search
                synchronized (SyncRoot)
            	{
            		for (int i = 0; i < listPlayer.size(); i++)
                    {
                        if (listPlayer.get(i).getIsAlive())
                        {
                        	listPlayer.get(i).setIsResponse(false);
                            sendData(listPlayer.get(i).getPlayerSocket(), RoomGSTPacketAction.InitialSetting, "");
                        }
                    }
                    if (listPlayer.get(0).getIsAlive() || listPlayer.get(1).getIsAlive())
                    {
                    	SyncRoot.wait(); //等玩家回應
                    }
            	}
                
                //Set initial board info
                String initialRemainGhostCount = roomGSTMode.getinitialRemainGhostCount();
                String initialStepList = roomGSTMode.getInitialStepList(), initialEatList = roomGSTMode.getInitialEatList(), initialTimeList = roomGSTMode.getInitialTimeList(), initialgameStarted = roomGSTMode.getInitialgameStarted();
                // 限於localserver undo
                if(isUndo) {
                	initialStepList = undoStepList;
                	initialTimeList = undoTimeList;
                	initialgameStarted = undogameStarted;
                	isUndo = false;
                }
                
                //Set Red Ghost-----
                if (!readMidBoard.equals("1")) {
                	synchronized (SyncRoot)
	            	{
	            		for (int i = 0; i < listPlayer.size(); i++)
	                    {
	                        if (listPlayer.get(i).getIsAlive())
	                        {
	                        	listPlayer.get(i).setIsResponse(false);
	                            sendData(listPlayer.get(i).getPlayerSocket(), RoomGSTPacketAction.SetRed, String.format("%s,%d", "SET?\r\n", 0)); // <message> <isAck>
	                        }
	                    }
	                    if (listPlayer.get(0).getIsAlive() || listPlayer.get(1).getIsAlive())
	                    {
	                    	SyncRoot.wait(); //等玩家回應
	                    }
	            	}
                	//Set initial chess board (for log after game)
                	roomGSTMode.setInitialChessBoard(roomGSTTable.getStrChessBoard());
                }
                
                
                initializeTableAndJudgement(roomGSTTable, roomGSTJudgement, roomGSTMode.getInitialChessBoard(), initialRemainGhostCount, initialStepList, initialTimeList, initialgameStarted);
                
                String initialInformationOfBoard =  roomGSTMode.getInitialChessBoard() + "," + initialRemainGhostCount + "," + initialgameStarted + ",";
                //String initialInformationOfBoard = roomGSTMode.getInitialHideChessBoard() + "," + initialRemainGhostCount + "," + initialgameStarted + ",";
                initialInformationOfBoard += !initialStepList.equals("") ? RoomGST.TrimEnd(initialStepList, ',').split(",").length + "," + RoomGST.TrimEnd(initialStepList, ',') + "," : "0,";
                initialInformationOfBoard += !initialEatList.equals("") ? RoomGST.TrimEnd(initialEatList, ',').split(",").length + "," + RoomGST.TrimEnd(initialEatList, ',') + "," : "0,";
                initialInformationOfBoard += !initialTimeList.equals("") ? RoomGST.TrimEnd(initialTimeList, ',').split(",").length + "," + RoomGST.TrimEnd(initialTimeList, ',') : "0";
//                initialInformationOfBoard += roomGSTTable.getStrChessBoard() + "," + roomGSTTable.getStrremainAllChessCount(); //重複資訊
                roomGSTMode.setInitialInformationOfBoard(initialInformationOfBoard);
                
                if (roomGSTTable.getJudgeResult().equals("")) 
                {
                	synchronized (SyncRoot)
	                {
	                    for (int i = 0; i < listPlayer.size(); i++)
	                    {
	                        if (listPlayer.get(i).getIsAlive())
	                        {
	                        	//直接改掉前36個棋盤元素
	                        	String updatedInfo = InitialInformationForPlayer(roomGSTMode.getInitialInformationOfBoard(),i);
	                            listPlayer.get(i).setIsResponse(false);
	                            sendData(listPlayer.get(i).getPlayerSocket(), RoomGSTPacketAction.SetRed, String.format("%s,%d", "OK", 1));
	                            sendData(listPlayer.get(i).getPlayerSocket(), RoomGSTPacketAction.InitialBoard, updatedInfo);
	                            //listPlayer.get(i).setIsResponse(false);
	                            //sendData(listPlayer.get(i).getPlayerSocket(), RoomGSTPacketAction.InitialBoard, roomGSTMode.getInitialInformationOfBoard());
	                        }
	                    }
	                    if (listPlayer.get(0).getIsAlive() || listPlayer.get(1).getIsAlive())
	                    {
	                    	SyncRoot.wait(); //等玩家回應
	                    }
	                }
	                synchronized (SyncRoot)
	                {	
	                	
	                    for (int i = 0; i < listPlayer.size(); i++)
	                    {
	                        if (listPlayer.get(i).getIsAlive())
	                        {
	                        	String firstMoveGhostCodeInfo = getGhostCodeInfo(roomGSTTable.getGhostCodeLocation(), roomGSTTable.getGhostCodeColor(), i);
	                            sendData(listPlayer.get(i).getPlayerSocket(), RoomGSTPacketAction.GameStart, String.format("%s,%s,%s,%s,%d,%s", listPlayer.get(0).getPlayerName(), listPlayer.get(0).getPlayerKind(), listPlayer.get(1).getPlayerName(), listPlayer.get(1).getPlayerKind(), i + 1, firstMoveGhostCodeInfo));
	                        }
	                    }
	                
	                    roomGSTTable.setGameStartTimeStamp(String.format("%d", new Date().getTime()));  //毫秒
	                    whoseTurn = roomGSTTable.getStepCount() % 2; //依照步數判斷下一手是誰
	                    listPlayer.get(whoseTurn).setIsResponse(false);
	                    if (roomGSTTable.getJudgeResult().equals(""))
	                    {
	                        SyncRoot.wait(); //等 judgeResult != ""，judgeResult為該局遊戲的勝負結果
	                    }
	                
	                    processRecoder.addProcessLog("==  judgeResult = " + roomGSTTable.getJudgeResult());
	                    RoomGST.DEBUG("==  judgeResult = " + roomGSTTable.getJudgeResult());
	                }
                }
                String judgeResult = "";
                synchronized (SyncRoot)
                {
                    judgeResult = roomGSTTable.getJudgeResult();
                    roomGSTTable.setScore(getScore(judgeResult, roomGSTTable.getgameStarted(), roomGSTTable.getremainGhostCount()));
                    for (int i = 0; i < listPlayer.size(); i++)
                    {
                        if (listPlayer.get(i).getIsAlive())
                        {
                        	String gameResultGhostCodeInfo = getGhostCodeInfo(roomGSTTable.getGhostCodeLocation(), roomGSTTable.getGhostCodeColor(), i);
                            sendData(listPlayer.get(i).getPlayerSocket(), RoomGSTPacketAction.GameResult, String.format("%d,%s,%s,%s", gameCount, judgeResult, roomGSTTable.getScore(), gameResultGhostCodeInfo));
                        }
                    }
                    
                    roomGSTTable.setBoardEndingRecord(saveAndGetLogFile(judgeResult, judgeResult.contains(RoomGSTJudgement.SocketBroken) ? "Socket" : "Normal"));
                    if(gameCount == Integer.parseInt(restartTimes))
                    {
                        isGameFinsh = true;
                    }
                    for (int i = 0; i < listPlayer.size(); i++)
                    {
                        if (listPlayer.get(i).getIsAlive())
                        {
                            listPlayer.get(i).setIsResponse(false);
                            sendData(listPlayer.get(i).getPlayerSocket(), RoomGSTPacketAction.GetBoardEndingRecord, roomGSTTable.getBoardEndingRecord());
                        }
                    }
                    if (listPlayer.get(0).getIsAlive() || listPlayer.get(1).getIsAlive())
                    {
                        SyncRoot.wait(); //等玩家回應
                    }
                }

                processRecoder.saveProcessLog(judgeResult, judgeResult.contains(RoomGSTJudgement.SocketBroken) ? "Socket" : "Normal", listPlayer, roomGSTTable.getGameStartTimeStamp());
                processRecoder.resetProcessRecoder();

                
                if (listPlayer.size() == 2 && (isUndo ? undoIsSwapColor : changeFirst.equals("1")))
                {
                    Player tempPlayer = listPlayer.get(0);
                    listPlayer.set(0, listPlayer.get(1));
                    listPlayer.set(1, tempPlayer);
                }
                if(!isUndo) {
                	gameCount += 1;
                }
            }
            
            broadcastSend(RoomGSTPacketAction.GameFinish, "");
            for (int i = 0; i < listPlayer.size(); i++)
            {
                listPlayer.get(i).closeSocket();
            }
            finishGame();
        }
        catch (Exception exception)
        {
        	StringWriter sw = new StringWriter();
			exception.printStackTrace(new PrintWriter(sw));
			RoomGST.DEBUG(sw.toString());
            ProcessRecoder.saveToErrorLog(String.format("%s", sw.toString()));
            processRecoder.addProcessLog(String.format("%s", sw.toString()));
            processRecoder.saveProcessLog("", "interrupt", listPlayer, roomGSTTable.getGameStartTimeStamp());
            finishGame();
        }
    }

    private void analysisCommand(Socket socket, short actionNum, String context)//解析客戶端發送的命令
    {
        RoomGST.DEBUG("\n------------------RoomGST.analysisCommand------------------");
        RoomGST.DEBUG(">>  actionNum = " + actionNum + ",   " + context);
        RoomGST.DEBUG("------------------RoomGST.analysisCommand------------------\n");
        processRecoder.addProcessLog("\n------------------RoomGST.analysisCommand------------------");
        processRecoder.addProcessLog(">>  actionNum = " + actionNum + ",   " + context);
        processRecoder.addProcessLog("------------------RoomGST.analysisCommand------------------\n");
        
        try
        {
        	if (actionNum == RoomGSTPacketAction.SetRed)
            {
            	int socketOwner = getSocketOwner(socket);
            	synchronized (SyncRoot) {
            		listPlayer.get(socketOwner).setIsResponse(true);
            		Boolean setRedSucess = roomGSTTable.setCustomRedGhost(context, socketOwner);
            		if(!setRedSucess) {
            			roomGSTTable.setJudgeResult(String.format("%s,%s,%d", RoomGSTJudgement.IllegalAction, context, socketOwner + 1));
            		}
                    if ((listPlayer.get(0).getIsResponse() || !listPlayer.get(0).getIsAlive()) && (listPlayer.get(1).getIsResponse() || !listPlayer.get(1).getIsAlive()))
                    {//兩方都回應後才進來
                    	SyncRoot.notify();
                    }
            		
            	}
            }
        	else if (actionNum == RoomGSTPacketAction.InitialSetting)
            {
            	int socketOwner = getSocketOwner(socket);
            	synchronized (SyncRoot) {
            		if (context.equals("1"))
                    {
            			listPlayer.get(socketOwner).setIsResponse(true);
                        if ((listPlayer.get(0).getIsResponse() || !listPlayer.get(0).getIsAlive()) && (listPlayer.get(1).getIsResponse() || !listPlayer.get(1).getIsAlive()))
                        {
                        	SyncRoot.notify();
                        }
                    }
            	}
            }
        	else if (actionNum == RoomGSTPacketAction.InitialBoard)
            {
            	int socketOwner = getSocketOwner(socket);
            	synchronized (SyncRoot) {
            		if (context.equals("1"))
                    {
            			listPlayer.get(socketOwner).setIsResponse(true);
                        if ((listPlayer.get(0).getIsResponse() || !listPlayer.get(0).getIsAlive()) && (listPlayer.get(1).getIsResponse() || !listPlayer.get(1).getIsAlive()))
                        {
                        	SyncRoot.notify();
                        }
                    }
            	}
            }
            else if (actionNum == RoomGSTPacketAction.Move)
            {
            	long Recv_timeStamp = new Date().getTime();
                synchronized (SyncRoot)
                {
                    int socketOwner = getSocketOwner(socket);
                    //listPlayer[socketOwner].setIsWaitingReadData(false);
                    listPlayer.get(socketOwner).setIsResponse(true);
                    if (socketOwner != whoseTurn)       //檢查是否為目前執棋者
                    {
                        return;
                    }
                    if (roomGSTTable.getJudgeResult().equals(""))
                    {
                        int stepCount = -1;
                        String tempJudgeResult = "", nextStep = "", previousTimeStamp = "";
                        long timeStamp = new Date().getTime();  //毫秒
                        long lastTimeStamp;
                        long Diff_timeStamp = timeStamp - Recv_timeStamp;
                        String checkTimeOut = "";
                        long usedMilliSecond;

                        // 檢查遊戲開始且第一手未下
                        // 如果超過第一手出手限制時間，則開始倒數總時間和單手制時間
                        // 超過時間則啟用代打
                        if (roomGSTTable.getIsWaitingFirstPly() && roomGSTTable.getJudgeResult().equals(""))
                        {
                            long gameStartTimeStamp = Long.parseLong(roomGSTTable.getGameStartTimeStamp());
                            usedMilliSecond = timeStamp - gameStartTimeStamp;
                            if (usedMilliSecond >= (GameRoom.firstPly_LimitTime + (Long.parseLong(setSecond) + Long.parseLong(plySecondLimit)) * 1000))
                            {
                                checkTimeOut = String.format("%s,%d", RoomGSTJudgement.TimeOut, whoseTurn + 1);
                            }
                        }

                        if (roomGSTTable.getIsWaitingFirstPly() && readMidBoard.equals("0"))
                        {
                            roomGSTTable.setIsWaitingFirstPly(false);
                            stepCount = roomGSTTable.getStepCount();
                            roomGSTTable.insertTimeArray(timeStamp + "", stepCount);
                        }
                        else if (roomGSTTable.getIsWaitingFirstPly() && !readMidBoard.equals("0"))
                        {
                            roomGSTTable.setIsWaitingFirstPly(false);
                            stepCount = roomGSTTable.getStepCount();
                            lastTimeStamp = Long.parseLong(roomGSTTable.getTimeStamp(stepCount));
                            roomGSTTable.shiftTimeList(timeStamp - lastTimeStamp);
                        }
                        else if (!roomGSTTable.getIsWaitingFirstPly() && Diff_timeStamp > 0)     //收到Move到處理時間大於0ms
                        {
                            roomGSTTable.shiftTimeList(Diff_timeStamp);
                        }
                        
                        capturePieceColor = 0;
                        nextStep = context; //eg: A,North
                        int[] x = { 0, 0 }, y = { 0, 0 };
                        int[] pIndex_src_dst = decodeMoveCommand(nextStep, whoseTurn);
                        int pieceIndex = pIndex_src_dst[0];
                        x[0] = pIndex_src_dst[1];
                        y[0] = pIndex_src_dst[2];
                        x[1] = pIndex_src_dst[3];
                        y[1] = pIndex_src_dst[4];                
                        
                        int[][] chessBoard = roomGSTTable.getChessBoard();
                        String pla = roomGSTTable.getplayer(whoseTurn);
//                        processRecoder.addProcessLog(">>  nextStep = " + nextStep);
                        
                        String actionName = roomGSTJudgement.getActionName(x, y, chessBoard, pla);
                        processRecoder.addProcessLog(">>  actionName = " + actionName);
                        if (actionName.equals(RoomGSTJudgement.MoveChess))
                        {	
                        	if (roomGSTTable.getgameStarted().equals("0"))
                            {
                            	roomGSTTable.setgameStarted("1");
                            }
                            roomGSTTable.increaseStepCount();
                            stepCount = roomGSTTable.getStepCount();
//                            roomGSTTable.increaseMaxPlyCount();
//                            int MaxPlyCount = roomGSTTable.getMaxPlyCount();
                            roomGSTTable.moveChessOnBoard(x, y, pieceIndex);
                            roomGSTTable.insertStepArray(nextStep, stepCount);
                            roomGSTTable.insertEatArray("0", stepCount);
//                            String[] stepArray = roomGSTTable.getStepArray();
                            roomGSTTable.insertTimeArray(timeStamp + "", stepCount);
                            previousTimeStamp = roomGSTTable.getTimeStamp(stepCount - 1);
                            roomGSTTable.updateUsedMilliSecond(whoseTurn, previousTimeStamp, timeStamp + "");
//                            int[] remainAllChessCount = roomGSTTable.getremainAllChessCount();
                            tempJudgeResult = roomGSTJudgement.moveChess(stepCount, whoseTurn);
                        }
                        else if (actionName.equals(RoomGSTJudgement.CaptureChess))
                        {
                        	processRecoder.addProcessLog(">> chessBoard[y[0]][x[0]] = " + chessBoard[y[0]][x[0]]);
                            processRecoder.addProcessLog(">> chessBoard[y[1]][x[1]] = " + chessBoard[y[1]][x[1]]);
                        	capturePieceColor = chessBoard[y[1]][x[1]] % 3;
                            roomGSTTable.increaseStepCount();
                            stepCount = roomGSTTable.getStepCount();
                            roomGSTTable.moveChessOnBoard(x, y, pieceIndex);
                            roomGSTTable.insertStepArray(nextStep, stepCount);
                            roomGSTTable.insertEatArray(capturePieceColor + "", stepCount);
                            roomGSTTable.insertTimeArray(timeStamp + "", stepCount);
                            previousTimeStamp = roomGSTTable.getTimeStamp(stepCount - 1);
                            roomGSTTable.updateUsedMilliSecond(whoseTurn, previousTimeStamp, timeStamp + "");
                            int[] remainAllChessCount = roomGSTTable.getremainAllChessCount();
                            tempJudgeResult = roomGSTJudgement.captureChess(pla, remainAllChessCount, chessBoard, whoseTurn);
                        }
                        else if (actionName.equals(RoomGSTJudgement.ChessEscape)) {
                        	roomGSTTable.increaseStepCount();
                            stepCount = roomGSTTable.getStepCount();
//                            roomGSTTable.setMaxPlyCount(0);
                            roomGSTTable.chessEscapeOnBoard(x[0], y[0]);
                            roomGSTTable.insertStepArray(nextStep, stepCount);
                            roomGSTTable.insertEatArray("0", stepCount);
                            roomGSTTable.insertTimeArray(timeStamp + "", stepCount);
                            previousTimeStamp = roomGSTTable.getTimeStamp(stepCount - 1);
                            roomGSTTable.updateUsedMilliSecond(whoseTurn, previousTimeStamp, timeStamp + "");
                            int escapeChess = roomGSTTable.getEscapeChess();
                            tempJudgeResult = roomGSTJudgement.chessEscape(escapeChess, whoseTurn);
                        }
                        else
                        {
                            tempJudgeResult = String.format("%s,%s,%d", RoomGSTJudgement.IllegalAction, nextStep, whoseTurn + 1);
                        }
                        
                        
                        if (tempJudgeResult.equals(""))
                        {
                            lastTimeStamp = Long.parseLong(roomGSTTable.getTimeStamp(roomGSTTable.getStepCount()));
                            usedMilliSecond = roomGSTTable.getUsedMilliSecond(whoseTurn) + (timeStamp - lastTimeStamp);
                            int groupPlyCount = roomGSTTable.getGroupPlyCount(whoseTurn);
                            Object[] objects = roomGSTJudgement.checkTimeOut(usedMilliSecond, groupPlyCount, whoseTurn);
                            tempJudgeResult = (String) objects[0];
                            usedMilliSecond = (long) objects[1];
                            groupPlyCount = (int) objects[2];
                            checkTimeOut = checkTimeOut == "" ? tempJudgeResult : checkTimeOut;
                            tempJudgeResult = checkTimeOut;
                            roomGSTTable.setUsedMilliSecond(whoseTurn, usedMilliSecond);
                            roomGSTTable.setGroupPlyCount(whoseTurn, groupPlyCount);
                        }
                        processRecoder.addProcessLog("tempJudgeResult = " + tempJudgeResult);
                                            
                        whoseTurn = 1 - whoseTurn;      //換下一位玩家
                        listPlayer.get(whoseTurn).setIsResponse(false);
                        if (tempJudgeResult == "")
                        {
                            broadcastSendMove(actionNum, String.format("%s,%d,%d,%d,%d,%s,%d,%d", nextStep, capturePieceColor, roomGSTTable.getStepCount(), stepCount, whoseTurn + 1, previousTimeStamp, timeStamp, 0), 1 - whoseTurn);
                        }
                        else if (!tempJudgeResult.contains(RoomGSTJudgement.IllegalAction))//遊戲結束
                        {
                            broadcastSendMove(actionNum, String.format("%s,%d,%d,%d,%d,%s,%d,%d", nextStep, capturePieceColor, roomGSTTable.getStepCount(), stepCount, whoseTurn + 1, previousTimeStamp, timeStamp, 1), 1 - whoseTurn);
                            roomGSTTable.setJudgeResult(tempJudgeResult);
                            SyncRoot.notify();
                        }
                        else
                        {
                            roomGSTTable.setJudgeResult(tempJudgeResult);
                            SyncRoot.notify();
                        }
                    }
                    else
                    {
                        if (listPlayer.get(whoseTurn).getIsAlive())
                        {
                            listPlayer.get(whoseTurn).setIsResponse(false);
                        }
                        else
                        {
                            listPlayer.get(whoseTurn).setIsResponse(true);
                        }
                    }
                }
            }
            else if (actionNum == RoomGSTPacketAction.GetBoardEndingRecord)
            {
            	int socketOwner = getSocketOwner(socket);
            	synchronized (SyncRoot) {
            		if (context.equals("1"))
                    {
            			listPlayer.get(socketOwner).setIsResponse(true);
                        if ((listPlayer.get(0).getIsResponse() || !listPlayer.get(0).getIsAlive()) && (listPlayer.get(1).getIsResponse() || !listPlayer.get(1).getIsAlive()))
                        {
                        	SyncRoot.notify();
                        }
                    }
            	}
            }
            else if (actionNum == RoomGSTPacketAction.Resign && listPlayer.get(whoseTurn).getPlayerKind().equals("1"))
            {
            	synchronized (SyncRoot) {
            		roomGSTTable.setJudgeResult(String.format("%s,%d", RoomGSTJudgement.Resign, whoseTurn + 1));
            		SyncRoot.notify();
            	}
            }
            else if (actionNum == RoomGSTPacketAction.DrawOffer)
            {
                // Local server does not provide
            	synchronized (SyncRoot) {
                    sendData(socket, RoomGSTPacketAction.DrawOffer, "0");
                    SyncRoot.notify();
            	}
            }
            else if (actionNum == RoomGSTPacketAction.Abort)
            {
            	synchronized (SyncRoot) {
            		if (listPlayer.get(whoseTurn).getPlayerKind().equals("1") && roomGSTTable.getStepCount() != 0)
                    {
                        if(roomGSTTable.getJudgeResult().equals("")){
                        	roomGSTTable.setJudgeResult(String.format("%s,%d", RoomGSTJudgement.Abort, whoseTurn + 1));
                            if(!socket.isInputShutdown()){
                                sendData(socket, RoomGSTPacketAction.Abort, String.format("%s", "1"));
                            }
                        }
                    }
                    else
                    {
                        if(!socket.isInputShutdown()){
                            sendData(socket, RoomGSTPacketAction.Abort, String.format("%s", "0"));
                        }
                    }
            	}
            }
            else if (actionNum == RoomGSTPacketAction.Undo)
            {
                boolean isUndoSuccess = true;
                
                int undoMoveLength = Integer.parseInt(context.split(",")[0]); //總共有幾步
                if(undoMoveLength <= 0 || undoMoveLength > roomGSTTable.getStepCount()) {
                	isUndoSuccess = false;
                }

                try {
                	if(isUndoSuccess) {
                    	undoStepList = roomGSTTable.getStepList(undoMoveLength);
                    	undoTimeList = roomGSTTable.getTimeList(undoMoveLength);
                    	undoIsSwapColor = context.split(",")[1].equals("1"); //是否交換顏色
                    	undogameStarted = roomGSTTable.getgameStarted();
                        
                    }
				} catch (Exception e) {
					isUndoSuccess = false;
				}
                
                if(isUndoSuccess) {
                	isUndo = true;
                	if(!socket.isInputShutdown()){
                        sendData(socket, RoomGSTPacketAction.Undo, String.format("%s", "1"));
                    }
                    synchronized (SyncRoot) {
                    	SyncRoot.notify();
                    }
                }else{
                    if(!socket.isInputShutdown()){
                        sendData(socket, RoomGSTPacketAction.Undo, String.format("%s", "0"));
                    }
                }
            }
            else
            {
            	synchronized (SyncRoot)
                {
                    processRecoder.addProcessLog(">>  actionNum = " + actionNum + ",   " + context + " (Unknown actionNum)");
                    roomGSTTable.setJudgeResult(String.format("%s,%d", RoomGSTJudgement.Resign, whoseTurn + 1));
                    SyncRoot.notify();
                }
            }
        }
        catch (Exception exception)
        {
        	StringWriter sw = new StringWriter();
			exception.printStackTrace(new PrintWriter(sw));
			RoomGST.DEBUG(sw.toString());
            ProcessRecoder.saveToErrorLog(String.format("%s", sw.toString()));
            processRecoder.addProcessLog(String.format("%s", sw.toString()));
            processRecoder.saveProcessLog("", "interrupt", listPlayer, roomGSTTable.getGameStartTimeStamp());
        }
    }

    private String initializeTableAndJudgement(RoomGSTTable roomGSTTable, RoomGSTJudgement roomGSTJudgement, String initialChessBoard, String initialRemainGhostCount, String initialStepList, String initialTimeList, String gameStarted)
    {
        boolean isSuccess = roomGSTTable.setChessBoardAndRemainGhostCount(initialChessBoard, initialRemainGhostCount);
        // 檢查 有走步卻無初始顏色 或 無剩餘暗子卻仍沒有初始顏色
        if (isSuccess && (!initialStepList.equals("") && gameStarted.equals("0")) || (roomGSTTable.getAllremainAllChessCount() == 0 && gameStarted.equals("0")))
        {
            isSuccess = false;
        }
        if (!isSuccess)
        {
            return "initBoard and initremainAllChessCount and initgameStarted does not match";
        }
        roomGSTTable.setgameStarted(gameStarted);
//        roomGSTTable.setplayer();
        
        if (!initialStepList.equals("") && !initialTimeList.equals(""))
        {
            int stepCount = -1;
            long timeStamp = -1;
            String tempJudgeResult = "", nextStep = "";
            String[] initialStepArray = initialStepList.split(","), initialTimeArray = initialTimeList.split(",");
            stepCount = roomGSTTable.getStepCount();
            roomGSTTable.insertTimeArray(initialTimeArray[0], stepCount);
            for (int i = 0; i < initialStepArray.length; i++)
            {
                nextStep = initialStepArray[i].replace(' ', ',');
                timeStamp = Long.parseLong(initialTimeArray[i + 1]);
                
                int[] x = { 0, 0 }, y = { 0, 0 };
                int[] pIndex_src_dst = decodeMoveCommand(nextStep, i % 2);
                int pieceIndex = pIndex_src_dst[0];
                x[0] = pIndex_src_dst[1];
                y[0] = pIndex_src_dst[2];
                x[1] = pIndex_src_dst[3];
                y[1] = pIndex_src_dst[4];


                int[][] chessBoard = roomGSTTable.getChessBoard();
                String player = roomGSTTable.getplayer(i % 2);
                String actionName = roomGSTJudgement.getActionName(x, y, chessBoard, player);
                if (actionName.equals(RoomGSTJudgement.MoveChess))
                {
                	roomGSTTable.increaseStepCount();
                    stepCount = roomGSTTable.getStepCount();
                    roomGSTTable.moveChessOnBoard(x, y, pieceIndex);
                    roomGSTTable.insertStepArray(nextStep, stepCount);
                    roomGSTTable.insertEatArray("0", stepCount);
                    roomGSTTable.insertTimeArray(timeStamp + "", stepCount);
                    tempJudgeResult = roomGSTJudgement.moveChess(stepCount, i % 2);
                }
                else if (actionName.equals(RoomGSTJudgement.CaptureChess))
                {
                	capturePieceColor = chessBoard[y[1]][x[1]] % 3;
                    roomGSTTable.increaseStepCount();
                    stepCount = roomGSTTable.getStepCount();
                    roomGSTTable.moveChessOnBoard(x, y, pieceIndex);
                    roomGSTTable.insertStepArray(nextStep, stepCount);
                    roomGSTTable.insertEatArray(capturePieceColor + "", stepCount);
                    roomGSTTable.insertTimeArray(timeStamp + "", stepCount);
                    int[] remainAllChessCount = roomGSTTable.getremainAllChessCount();
                    tempJudgeResult = roomGSTJudgement.captureChess(player, remainAllChessCount, chessBoard, i % 2);
                }
                else if (actionName.equals(RoomGSTJudgement.ChessEscape)) {
                	roomGSTTable.increaseStepCount();
                    stepCount = roomGSTTable.getStepCount();
//                    roomGSTTable.setMaxPlyCount(0);
                    roomGSTTable.chessEscapeOnBoard(x[0], y[0]);
                    roomGSTTable.insertStepArray(nextStep, stepCount);
                    roomGSTTable.insertEatArray("0", stepCount);
                    roomGSTTable.insertTimeArray(timeStamp + "", stepCount);
                    int escapeChess = roomGSTTable.getEscapeChess();
                    tempJudgeResult = roomGSTJudgement.chessEscape(escapeChess, i % 2);
                }
                else
                {
                    tempJudgeResult = String.format("%s,%s,%d", RoomGSTJudgement.IllegalAction, nextStep, i % 2 + 1);
                }
                
                if (tempJudgeResult.equals(""))
                {
                    long usedMilliSecond = roomGSTTable.getUsedMilliSecond(i % 2);
                    int groupPlyCount = roomGSTTable.getGroupPlyCount(i % 2);
                    Object[] objects = roomGSTJudgement.checkTimeOut(usedMilliSecond, groupPlyCount, i % 2);
                    tempJudgeResult = (String) objects[0];
                    usedMilliSecond = (long) objects[1];
                    groupPlyCount = (int) objects[2];
                    roomGSTTable.setUsedMilliSecond(i % 2, usedMilliSecond);
                    roomGSTTable.setGroupPlyCount(i % 2, groupPlyCount);
                }
                processRecoder.addProcessLog("initialTempJudgeResult = " + tempJudgeResult);

                if (!tempJudgeResult.equals(""))
                {
                    return tempJudgeResult;
                }
                roomGSTTable.showChessBoard();
            }
        }
        return "Success";
    }
    
    private int[] decodeMoveCommand(String moveCommand, int whoseTurn) {//兩個人傳入都會是大寫+方向(這是要更新上帝視角的盤)
    	int[] PI_srcXY_dstXY = new int[] {0, 6 ,6 ,6 ,6}; //{pieceIndex in GhostCodeLoc, src_x, src_y, dst_x, dst_y} (6,6)是任何已有功能以外的座標
    	String[] commandArray = moveCommand.split(",");
    	int offset = whoseTurn == 0 ? 0 : 8;
    	int pieceIndex = (int)commandArray[0].charAt(0) - (int)('A');
    	if (pieceIndex >= 8) return PI_srcXY_dstXY; // check piece 是否超過H
    	else pieceIndex += offset;
    	PI_srcXY_dstXY[0] = pieceIndex;
    	int src = roomGSTTable.getChostLocation(pieceIndex);
    	PI_srcXY_dstXY[1] = src % 6; // board size = 6*6
    	PI_srcXY_dstXY[2] = src / 6;

    	int dir = whoseTurn == 0 ? 1 : -1; // P2的東南西北是相反的(設定好看己方都在下後)  //這裡不用檢查dst是不是可走的座標，之後的Judgement的getActionName有檢查
    	if (commandArray[1].equals("N") || commandArray[1].equals("NORTH")) {
    		PI_srcXY_dstXY[3] = PI_srcXY_dstXY[1];
    		PI_srcXY_dstXY[4] = PI_srcXY_dstXY[2] - dir;
    	}
    	else if (commandArray[1].equals("E") || commandArray[1].equals("EAST")) {
    		PI_srcXY_dstXY[3] = PI_srcXY_dstXY[1] + dir;
    		PI_srcXY_dstXY[4] = PI_srcXY_dstXY[2];
    	}
    	else if (commandArray[1].equals("W") || commandArray[1].equals("WEST")) {
    		PI_srcXY_dstXY[3] = PI_srcXY_dstXY[1] - dir;
    		PI_srcXY_dstXY[4] = PI_srcXY_dstXY[2];
    	}
    	else if (commandArray[1].equals("S") || commandArray[1].equals("SOUTH")) {
    		PI_srcXY_dstXY[3] = PI_srcXY_dstXY[1];
    		PI_srcXY_dstXY[4] = PI_srcXY_dstXY[2] + dir;
    	}
    	
    	//check is escape move
    	//left corner
    	if (PI_srcXY_dstXY[3] == -1 && (PI_srcXY_dstXY[4] == 0 || PI_srcXY_dstXY[4] == 5)) { //escape dir = WEST
    		PI_srcXY_dstXY[3] = 8;
    		PI_srcXY_dstXY[4] = 8;
    	}
    	if (PI_srcXY_dstXY[3] == 0 && (PI_srcXY_dstXY[4] == -1 || PI_srcXY_dstXY[4] == 6)) { //escape dir = NORTH || SOUTH
    		PI_srcXY_dstXY[3] = 8;
    		PI_srcXY_dstXY[4] = 8;
    	}
    	//right corner
    	if (PI_srcXY_dstXY[3] == 6 && (PI_srcXY_dstXY[4] == 0 || PI_srcXY_dstXY[4] == 5)) { //escape dir = EAST
    		PI_srcXY_dstXY[3] = 8;
    		PI_srcXY_dstXY[4] = 8;
    	}
    	if (PI_srcXY_dstXY[3] == 5 && (PI_srcXY_dstXY[4] == -1 || PI_srcXY_dstXY[4] == 6)) { //escape dir = NORTH || SOUTH
    		PI_srcXY_dstXY[3] = 8;
    		PI_srcXY_dstXY[4] = 8;
    	}
    	return PI_srcXY_dstXY;
    }
    
//    private String setRandomRedGhost(String initialChessBoard) {
//    	int[] randRedLoc = roomGSTTable.computeIniRandRedloc(listPlayer.get(0).getPlayerName(), listPlayer.get(1).getPlayerName());
////    	String temp = "0,3,3,3,3,0,0,3,3,3,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,3,3,3,0,0,3,3,3,3,0";
//    	String[] initialChessBoardArray = initialChessBoard.split(",");
//    	//先把棋子都設為藍鬼：3 -> 1(p1) or 4(p2)
//    	int[] ghostCodeClr = {2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2};
//    	for (int i = 0; i < 36; i++) {
//    		if (i < 12 && initialChessBoardArray[i].equals("3")) {
//    			initialChessBoardArray[i] = "5";
//    		}
//    		else if (i > 24 && initialChessBoardArray[i].equals("3")) {
//    			initialChessBoardArray[i] = "2";
//    		}
//    	}
//    	
////    	int[] randRedLoc = new int[] {0,1,2,3,0,1,2,3};
//    	//設定紅鬼 p1
//    	for (int i = 0; i < 4; i++) {
//    		initialChessBoardArray[roomGSTTable.getChostLocation(randRedLoc[i])] = "1";
//    		ghostCodeClr[randRedLoc[i]] = 1;
//    	}
//    	//p2
//    	for (int i = 4; i < 8; i++) {
//    		initialChessBoardArray[roomGSTTable.getChostLocation(randRedLoc[i]+8)] = "4";
//    		ghostCodeClr[randRedLoc[i]+8] = 1;
//    	}
//    	roomGSTTable.setGhostCodeColor(ghostCodeClr);
//    	String afterSetRedBoard = "";
//    	for (int i = 0; i < 36; i++) {
//    		afterSetRedBoard += initialChessBoardArray[i] + ",";
//    	}
//    	afterSetRedBoard = TrimEnd(afterSetRedBoard, ',');
////    	afterSetRedBoard = "0,4,4,4,4,0,0,5,5,5,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,2,2,2,0,0,1,1,1,1,0";
//    	return afterSetRedBoard;
//    }
    
    String getScore(String judgeResult, String gameStarted, int[] remainGhostCount)
    {
    	if(judgeResult.contains(RoomGSTJudgement.Abort)) {
    		return "0.0,0.0";
    	}
        double[] playerTotal = new double[] { 0, 0 };
//        double[] pieceTotal = new double[] { 0, 0 };
        String[] judgeResultArray = judgeResult.split(",");
        int whoIsWinner = Integer.parseInt(judgeResultArray[judgeResultArray.length - 1]) == 1 ? 2 : 1;
        if (judgeResult.contains("Draw"))
        {
            playerTotal[0] += drawScore;
            playerTotal[1] += drawScore;
        }
        else
        {
            playerTotal[0] += whoIsWinner == 1 ? winScore : loseScore;
            playerTotal[1] += whoIsWinner == 1 ? loseScore : winScore;
        }

        /*if (gameStarted.equals("0"))
        {
            for (int i = 0; i < 8; i++)
            {
                pieceTotal[0] += remainGhostCount[i] * pieceScore[i];
                pieceTotal[1] += remainGhostCount[8 + i] * pieceScore[8 + i];
            }
//            for (int i = 0; i < 7; i++)
//            {
//                pieceTotal[0] += remainGhostCount[i] * pieceScore[i];
//                pieceTotal[1] += remainGhostCount[7 + i] * pieceScore[7 + i];
//            }
        }
        else if (gameStarted.equals("1"))
        {
            for (int i = 0; i < 8; i++)
            {
                pieceTotal[1] += remainGhostCount[i] * pieceScore[i];
                pieceTotal[0] += remainGhostCount[8 + i] * pieceScore[8 + i];
            }
//            for (int i = 0; i < 7; i++)
//              {
//                  pieceTotal[1] += remainGhostCount[i] * pieceScore[i];
//                  pieceTotal[0] += remainGhostCount[7 + i] * pieceScore[7 + i];
//              }
        }
        if (pieceTotal[0] > pieceTotal[1] && (playerTotal[0] == winScore || playerTotal[0] == drawScore))
        {
            if (pieceTotal[0] - pieceTotal[1] >= bonuslimit)
            {
                playerTotal[0] += bonus;
            }
            else
            {
                playerTotal[0] += bonus / bonuslimit * (pieceTotal[0] - pieceTotal[1]);
            }
        }
        if (pieceTotal[1] > pieceTotal[0] && (playerTotal[1] == winScore || playerTotal[1] == drawScore))
        {
            if (pieceTotal[1] - pieceTotal[0] >= bonuslimit)
            {
                playerTotal[1] += bonus;
            }
            else
            {
                playerTotal[1] += bonus / bonuslimit * (pieceTotal[1] - pieceTotal[0]);
            }
        }*/
        return String.format("%f,%f", playerTotal[0], playerTotal[1]);
    }

    private void finishGame()
    {
    	for (Player player : listPlayer)
        {
            try {
            	if(player.getPlayerSocket() != null) {
            		if(!player.getPlayerSocket().isOutputShutdown()) {
            			player.getPlayerSocket().shutdownOutput();
            		}
            		if(!player.getPlayerSocket().isInputShutdown()) {
            			player.getPlayerSocket().shutdownInput();
            		}
                	player.getPlayerSocket().close();
            	}
            }catch (IOException ioException){
                ioException.printStackTrace();
            }
        }
        timer.cancel();
        super.closeRoom();
    }

    public class readData extends Thread{//處理從客戶端接收數據的任務
    	int receiveLength, packetLength;
    	byte[] fullData;
        short actionNum;
        String context;
        Socket socket;
        Packet packet;
        StateObject stateObject;
        BufferedInputStream bufferedInputStream;
    	public readData(Socket socket) {
    		this.socket = socket;
		}
		@Override
		public void run() {
			try{
				while(!isGameFinsh) {
					RoomGST.DEBUG("------------------RoomGST.readData------------------\n");
					RoomGST.DEBUG(">>  " + listPlayer.get(getSocketOwner(socket)).getPlayerName());
					RoomGST.DEBUG("------------------RoomGST.readData------------------\n");
					
					packet = new Packet(4);
	                stateObject = new StateObject(this.socket, packet, StateObject.ReceivingDataLength);
	                bufferedInputStream = new BufferedInputStream(this.socket.getInputStream());
	                
	                receiveLength = bufferedInputStream.read(stateObject.packet.data, 0, 4);
	                // RoomGST.DEBUG("receiveLength = " + receiveLength);
	                stateObject.doneLength += receiveLength;
	                while(stateObject.doneLength < stateObject.packet.dataLength && receiveLength > 0){
	                    receiveLength = bufferedInputStream.read(stateObject.packet.data, stateObject.doneLength, stateObject.packet.dataLength - stateObject.doneLength);
	                    // RoomGST.DEBUG("receiveLength = " + receiveLength);
	                    stateObject.doneLength += receiveLength;
	                }
	                packetLength = receiveLength > 0 ? byteArrayToInt(stateObject.packet.data, 0) : 0;
	                // RoomGST.DEBUG("packetLength = " + packetLength);
	                packet = new Packet(packetLength);
	                stateObject = new StateObject(socket, packet, StateObject.ReceivingData);
	                receiveLength = bufferedInputStream.read(stateObject.packet.data, 0, packetLength);
	                // RoomGST.DEBUG("receiveLength = " + receiveLength);
	                stateObject.doneLength += receiveLength;
	                while(stateObject.doneLength < stateObject.packet.dataLength  && receiveLength > 0){
	                    receiveLength = bufferedInputStream.read(stateObject.packet.data, stateObject.doneLength, stateObject.packet.dataLength - stateObject.doneLength);
	                    stateObject.doneLength += receiveLength;
	                }
	                if(receiveLength > 0){
	                    fullData = stateObject.packet.data;
	                    actionNum = byteArrayToShort(fullData, 0);
	                    context = new String(fullData, 2, fullData.length - 2, StandardCharsets.UTF_16LE);
	                    analysisCommand(socket, actionNum, context);
	                }else{
	                	synchronized (SyncRoot) {
	                		RoomGST.DEBUG(String.format("roomId = %d, roomState = %s, playerCount = %d, playerIP = %s, 關閉連線", getRoomId(), getRoomState(), listPlayer.size(), socket.getRemoteSocketAddress()));
	                		socket.shutdownInput();
                        }
	                }
                }
            }catch (IOException exception){
            	synchronized (SyncRoot) {
            		RoomGST.DEBUG(String.format("roomId = %d, roomState = %s, playerCount = %d, playerIP = %s, 中斷連線", getRoomId(), getRoomState(), listPlayer.size(), socket.getRemoteSocketAddress()));
            		try {
						socket.shutdownInput();
					} catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
                }
            	StringWriter sw = new StringWriter();
    			exception.printStackTrace(new PrintWriter(sw));
    			RoomGST.DEBUG(sw.toString());
                ProcessRecoder.saveToErrorLog(String.format("%s", sw.toString()));
                processRecoder.addProcessLog(String.format("%s", sw.toString()));
                processRecoder.saveProcessLog("", "interrupt", listPlayer, roomGSTTable.getGameStartTimeStamp());
            }
			
		}
    	
    }
    
    private void gotoGameRoomReadData(Socket socket) {
    	readData reaDataThread = new readData(socket);
    	RoomGST.DEBUG(String.format("%s readData in gameRoom", listPlayer.get(getSocketOwner(socket)).getPlayerName()));
    	reaDataThread.start();
    }

    private void sendData(Socket socket, Short actionNum, String context)
    {
        RoomGST.DEBUG("\n------------------RoomGST.SendData------------------");
        RoomGST.DEBUG("<<  actionNum = " + actionNum + ",   " + context);
        RoomGST.DEBUG("------------------RoomGST.SendData------------------\n");

        byte[] byteActionNum = shortToByteArray(actionNum); //action 是 short 所以 byteAction.Length = 2 (byte)
        byte[] byteContext =  context.getBytes(StandardCharsets.UTF_16LE); //data 用 unicode 編碼
        byte[] dataLength = intToByteArray(byteActionNum.length + byteContext.length); //action 是 short 所以 byteAction.Length = 2
        byte[] fullData = new byte[dataLength.length + byteActionNum.length + byteContext.length]; //newByteData = packetLength + byteAction + byteData

        //先copy封包長度至buffer(newpacket)中(4byte)封包長度包含動作
        System.arraycopy(dataLength, 0, fullData, 0, dataLength.length);
        //copy動作(2byte)
        System.arraycopy(byteActionNum, 0, fullData, dataLength.length, byteActionNum.length);
        //copy資料至buffer
        System.arraycopy(byteContext, 0, fullData, dataLength.length + byteActionNum.length, byteContext.length);

        try {
            // Begin sending the data to the remote device.
            BufferedOutputStream bufferedOutputStream = new BufferedOutputStream(socket.getOutputStream());
            bufferedOutputStream.write(fullData);
            bufferedOutputStream.flush();
        }catch(IOException ioException){
            RoomGST.DEBUG(String.format("roomId = %d, roomState = %s, playerCount = %d, playerIP = %s, 中斷連線", super.getRoomId(), super.getRoomState(), listPlayer.size(), socket.getRemoteSocketAddress()));
            ioException.printStackTrace();
        }
    }
    
    private void broadcastSend(short actionNum, String context)
    {
    	RoomGST.DEBUG("\n------------------RoomMJ.broadcastSend------------------");
    	RoomGST.DEBUG("<<  actionNum = " + actionNum + ",   " + context);
    	RoomGST.DEBUG("------------------RoomMJ.broadcastSend------------------\n");
        
        for (Player playerInfo : listPlayer)
        {
            if(playerInfo.getPlayerSocket() != null && playerInfo.getIsAlive()) {
                sendData(playerInfo.getPlayerSocket(), actionNum, context);
            }
        }
    }
    
    //廣播目前走的
    private void broadcastSendMove(short actionNum, String context, int whoseTurn)
    {
    	RoomGST.DEBUG("\n------------------RoomMJ.broadcastSend------------------");
    	RoomGST.DEBUG("<<  actionNum = " + actionNum + ",   " + context);
    	RoomGST.DEBUG("------------------RoomMJ.broadcastSend------------------\n");
    	int player = 0;
    	for (Player playerInfo : listPlayer) {
            if (playerInfo.getPlayerSocket() != null && playerInfo.getIsAlive()) {
            	String GhostCodeInfo = getGhostCodeInfo(roomGSTTable.getGhostCodeLocation(), roomGSTTable.getGhostCodeColor(), player);
                String personalizedContext = changeContext(context, whoseTurn, player, GhostCodeInfo);
                sendData(playerInfo.getPlayerSocket(), actionNum, personalizedContext);
            }
            player++;
        }
    }
    private String changeContext(String context, int whoseTurn, int player, String GhostCodeInfo) {
        if (whoseTurn == player) {
            return context + "," + GhostCodeInfo;
        } else {
            return adjustContextForOpponent(context) + "," + GhostCodeInfo;
        }
    }

    private String adjustContextForOpponent(String context) {
        // 方向相反
        String[] command = context.split(",");
        //piece大小寫相反
        StringBuilder sb = new StringBuilder();
        sb.append(Character.toLowerCase(context.charAt(0)) + ",");
        
        switch(command[1]) {
        	case "N":
        	case "NORTH":
        		sb.append("SOUTH");
        		break;
        	case "S":
        	case "SOUTH":
        		sb.append("NORTH");
        		break;
        	case "E":	
        	case "EAST":
        		sb.append("WEST");
        		break;
        	case "W":
        	case "WEST":
        		sb.append("EAST");
        		break;
        }
        
        for (int i = 2; i < command.length; i++) {
        	sb.append("," + command[i]);
        }
        
        return sb.toString();
    }
    
    private int getSocketOwner(Socket socket)
    {
    	// 0: player1, 1: player2
        if (socket == listPlayer.get(0).getPlayerSocket())
        {
            return 0;
        }
        else if (socket == listPlayer.get(1).getPlayerSocket())
        {
            return 1;
        }
//        else if (socket == listPlayer.get(2).getPlayerSocket())
//        {
//            return 2;
//        }
//        else if (socket == listPlayer.get(3).getPlayerSocket())
//        {
//            return 3;
//        }
        return -1;
    }
    
    private String getGhostCodeInfo(int[] GhostCodeLocation, int[] GhostCodeColor, int whoseTurn) {
    	StringBuilder sb = new StringBuilder();
    	int x, y;
    	String color;
    	if (whoseTurn == 0) {//P1
    		for (int i = 0; i < 8; i++) {
    			if (GhostCodeLocation[i] == 36) {
    				x = 9;
    				y = 9;
    				color = GhostCodeColor[i] == 1 ? "r" : "b";
    			}else {
    				x = GhostCodeLocation[i] % 6;
    				y = GhostCodeLocation[i] / 6;
    				color = GhostCodeColor[i] == 1 ? "R" : "B";
    			}	
    			sb.append("" + x + y + color);
    		}
    		for (int i = 8; i < GhostCodeLocation.length; i++) {
    			if (GhostCodeLocation[i] == 36) {
    				x = 9;
    				y = 9;
    				color = GhostCodeColor[i] == 1 ? "r" : "b";
    			}else {
    				x = GhostCodeLocation[i] % 6;
    				y = GhostCodeLocation[i] / 6;
    				color = "u";
    			}
    			sb.append("" + x + y + color);
    		}
    	}
    	else {//P2
    		for (int i = 8; i < GhostCodeLocation.length; i++) {
    			if (GhostCodeLocation[i] == 36) {
    				x = 9;
    				y = 9;
    				color = GhostCodeColor[i] == 1 ? "r" : "b";
    			}else {
    				x = 5 - GhostCodeLocation[i] % 6;
    				y = 5 - GhostCodeLocation[i] / 6;
    				color = GhostCodeColor[i] == 1 ? "R" : "B";
    			}
    			sb.append("" + x + y + color);
    		}
    		for (int i = 0; i < 8; i++) {
    			if (GhostCodeLocation[i] == 36) {
    				x = 9;
    				y = 9;
    				color = GhostCodeColor[i] == 1 ? "r" : "b";
    			}else {
    				x = 5 - GhostCodeLocation[i] % 6;
    				y = 5 - GhostCodeLocation[i] / 6;
    				color = "u";
    			}
    			sb.append("" + x + y + color);
    		}
    	}
    	return sb.toString();
    }

    private class TimeTask extends TimerTask {//看有沒有超時
        @Override
        public void run() {
        	try{
            	synchronized (SyncRoot) {
            		// 檢查遊戲開始且第一手未下
                    // 如果超過第一手出手限制時間，則開始倒數總時間和單手制時間
                    // 超過時間則判負
                    if (!roomGSTTable.getGameStartTimeStamp().equals("0") && roomGSTTable.getIsWaitingFirstPly() && roomGSTTable.getJudgeResult().equals(""))
                    {
                        long timeStamp = new Date().getTime(); //毫秒
                        long gameStartTimeStamp = Long.parseLong(roomGSTTable.getGameStartTimeStamp());
                        long usedMilliSecond = timeStamp - gameStartTimeStamp;
                        if(usedMilliSecond >= (GameRoom.firstPly_LimitTime + (Long.parseLong(setSecond) + Long.parseLong(plySecondLimit)) * 1000))
                        {
                        	roomGSTTable.setJudgeResult(String.format("%s,%d", RoomGSTJudgement.TimeOut, whoseTurn + 1));
                            SyncRoot.notify();
                        }
                        return;
                    }
                    
            		if (roomGSTTable.getJudgeResult().equals("") && !roomGSTTable.getIsWaitingFirstPly() && roomGSTTable.getStepCount() > 0) {
            			long timeStamp = new Date().getTime(); //毫秒
                        long lastTimeStamp = Long.parseLong(roomGSTTable.getTimeStamp(roomGSTTable.getStepCount()));
                        
                        long usedMilliSecond = roomGSTTable.getUsedMilliSecond(whoseTurn) + (timeStamp - lastTimeStamp);    
                        int groupPlyCount = roomGSTTable.getGroupPlyCount(whoseTurn);
                        Object[] objects = roomGSTJudgement.checkTimeOut(usedMilliSecond, groupPlyCount, whoseTurn);
                        String tempJudgeResult = (String) objects[0]; 
                        if (tempJudgeResult.contains(RoomGSTJudgement.TimeOut))
                        {
                        	roomGSTTable.setJudgeResult(tempJudgeResult);
                            SyncRoot.notify();
                        }
                        return;
            		}
            	}
            }catch (Exception exception) {
            	StringWriter sw = new StringWriter();
				exception.printStackTrace(new PrintWriter(sw));
				RoomGST.DEBUG(sw.toString());
	            ProcessRecoder.saveToErrorLog(String.format("%s", sw.toString()));
	            processRecoder.addProcessLog(String.format("%s", sw.toString()));
			}   
        }
    }

    private String saveAndGetLogFile(String judgeResult, String saveSituation)//儲存對局日誌
    {
    	Date date = new Date();
        String folderName = String.format("JudgementGSTLog" + File.separator + new SimpleDateFormat("yyyy" + File.separator + "MM" + File.separator + "dd").format(date));
        
        if (!saveSituation.equals("Normal"))
        {
            folderName += "\\" + saveSituation;
        }
        String[] judgeResultArray = judgeResult.split(",");
        judgeResult = judgeResultArray[0];
        for (int i = 1; i < judgeResultArray.length - 1; i++)
        {
            judgeResult += "," + judgeResultArray[i];
        }
        String boardEndingRecordFileName;
        int whoIsWinner = Integer.parseInt(judgeResultArray[judgeResultArray.length - 1]) == 1 ? 2 : 1;
        if (judgeResult.contains("Draw"))
        {
            boardEndingRecordFileName = String.format("%s VS %s (%s)-(%s(Draw))_%s_%s.txt", listPlayer.get(0).getPlayerName(), listPlayer.get(1).getPlayerName(), new SimpleDateFormat("yyyy_MM_dd_HH_mm_ss").format(date), listPlayer.get(0).getPlayerName(), judgeResultArray[0], saveSituation);
        }
        else
        {
            boardEndingRecordFileName = String.format("%s VS %s (%s)-(%s(Win))_%s_%s.txt", listPlayer.get(0).getPlayerName(), listPlayer.get(1).getPlayerName(), new SimpleDateFormat("yyyy_MM_dd_HH_mm_ss").format(date), listPlayer.get(whoIsWinner - 1).getPlayerName(), judgeResultArray[0], saveSituation);
        }

        File file = new File(folderName);
        if (!file.exists()){ //檢查是否資料夾存在,如果存在就不開新的資料夾
            file.mkdirs();     //資料夾不存在,開新資料夾
        }
        String boardEndingRecord = "";
        OutputStream outputStream;
        try {
        	outputStream = new FileOutputStream(folderName + File.separator + boardEndingRecordFileName);
            String rowOfBoardEndingRecord;
            
            SimpleDateFormat format = new SimpleDateFormat("K:ma MMM d, y", Locale.ENGLISH);
    		DateFormatSymbols symbols = new DateFormatSymbols(Locale.ENGLISH);
            // override symbols
            symbols.setAmPmStrings(new String[] { "am", "pm" });
            format.setDateFormatSymbols(symbols);
            String start_time = format.format(new Date());
            
            
            rowOfBoardEndingRecord = String.format("%s\n", roomGSTMode.getBoardName());
            outputStream.write(rowOfBoardEndingRecord.getBytes(StandardCharsets.UTF_8));
            boardEndingRecord += rowOfBoardEndingRecord;
            
            rowOfBoardEndingRecord = String.format("* %s\n", start_time);
            outputStream.write(rowOfBoardEndingRecord.getBytes(StandardCharsets.UTF_8));
            boardEndingRecord += rowOfBoardEndingRecord;
            
//            String strgameStarted = roomGSTTable.getgameStarted() == "0" ? "Red" : "Black", player1Color = roomGSTTable.getplayer(0);
            rowOfBoardEndingRecord = String.format("* %s vs %s\n", listPlayer.get(0).getPlayerName(), listPlayer.get(1).getPlayerName());
            outputStream.write(rowOfBoardEndingRecord.getBytes(StandardCharsets.UTF_8));
            boardEndingRecord += rowOfBoardEndingRecord;

            rowOfBoardEndingRecord = String.format("* %s\n", roomGSTMode.getinitialRemainGhostCount().replace(',', ' '));
            outputStream.write(rowOfBoardEndingRecord.getBytes(StandardCharsets.UTF_8));
            boardEndingRecord += rowOfBoardEndingRecord;

            String[] initialChessBoardArray = roomGSTMode.getInitialChessBoard().split(",");
            for (int i = 0; i < 36; i += 6)  //盤面大小為 32
            {
                initialChessBoardArray[i] = RoomGSTJudgement.chessName[Integer.parseInt(initialChessBoardArray[i])] + "";
                initialChessBoardArray[i + 1] = RoomGSTJudgement.chessName[Integer.parseInt(initialChessBoardArray[i + 1])] + "";
                initialChessBoardArray[i + 2] = RoomGSTJudgement.chessName[Integer.parseInt(initialChessBoardArray[i + 2])] + "";
                initialChessBoardArray[i + 3] = RoomGSTJudgement.chessName[Integer.parseInt(initialChessBoardArray[i + 3])] + "";
                initialChessBoardArray[i + 4] = RoomGSTJudgement.chessName[Integer.parseInt(initialChessBoardArray[i + 4])] + "";
                initialChessBoardArray[i + 5] = RoomGSTJudgement.chessName[Integer.parseInt(initialChessBoardArray[i + 5])] + "";
                rowOfBoardEndingRecord = String.format("* %s %s %s %s %s %s\n", initialChessBoardArray[i], initialChessBoardArray[i + 1], initialChessBoardArray[i + 2], initialChessBoardArray[i + 3],initialChessBoardArray[i + 4],initialChessBoardArray[i + 5]);
                outputStream.write(rowOfBoardEndingRecord.getBytes(StandardCharsets.UTF_8));
                boardEndingRecord += rowOfBoardEndingRecord;
            }
//            for (int i = 0; i < 32; i += 4)  //盤面大小為 32
//            {
//                initialChessBoardArray[i] = RoomGSTJudgement.chessName[Integer.parseInt(initialChessBoardArray[i])] + "";
//                initialChessBoardArray[i + 1] = RoomGSTJudgement.chessName[Integer.parseInt(initialChessBoardArray[i + 1])] + "";
//                initialChessBoardArray[i + 2] = RoomGSTJudgement.chessName[Integer.parseInt(initialChessBoardArray[i + 2])] + "";
//                initialChessBoardArray[i + 3] = RoomGSTJudgement.chessName[Integer.parseInt(initialChessBoardArray[i + 3])] + "";
//                rowOfBoardEndingRecord = String.format("* %s %s %s %s\n", initialChessBoardArray[i], initialChessBoardArray[i + 1], initialChessBoardArray[i + 2], initialChessBoardArray[i + 3]);
//                outputStream.write(rowOfBoardEndingRecord.getBytes(StandardCharsets.UTF_8));
//                boardEndingRecord += rowOfBoardEndingRecord;
//            }

//            rowOfBoardEndingRecord = String.format("* first %s\n", roomGSTTable.getgameStarted());
//            outputStream.write(rowOfBoardEndingRecord.getBytes(StandardCharsets.UTF_8));
//            boardEndingRecord += rowOfBoardEndingRecord;
            
            rowOfBoardEndingRecord = String.format("* time %d\n", Integer.parseInt(setSecond));
            outputStream.write(rowOfBoardEndingRecord.getBytes(StandardCharsets.UTF_8));
            boardEndingRecord += rowOfBoardEndingRecord;

//            String[] stepArray = RoomGST.TrimEnd(roomGSTTable.getStepList(), ',').split(",");
            String[] stepArray = roomGSTTable.getStepList().split(" ");
            for (int i = 0, j = 1; i < stepArray.length; i += 2, j++)
            {
                if (i + 1 == stepArray.length)
                {
                    rowOfBoardEndingRecord = String.format("* %d. %s\n", j, stepArray[i]);
                    outputStream.write(rowOfBoardEndingRecord.getBytes(StandardCharsets.UTF_8));
                    boardEndingRecord += rowOfBoardEndingRecord;
                }
                else
                {
                    rowOfBoardEndingRecord = String.format("* %d. %s %s\n", j, stepArray[i], stepArray[i + 1]);
                    outputStream.write(rowOfBoardEndingRecord.getBytes(StandardCharsets.UTF_8));
                    boardEndingRecord += rowOfBoardEndingRecord;
                }
            }

            if (judgeResult.contains("Draw"))
            {
                rowOfBoardEndingRecord = String.format("* %s %s Draw %s\n", judgeResult, listPlayer.get(0).getPlayerName(), roomGSTTable.getScore());
            }
            else
            {
                rowOfBoardEndingRecord = String.format("* %s %s wins %s\n", judgeResult, listPlayer.get(whoIsWinner - 1).getPlayerName(), roomGSTTable.getScore());
            }
            outputStream.write(rowOfBoardEndingRecord.getBytes(StandardCharsets.UTF_8));
            boardEndingRecord += rowOfBoardEndingRecord;

            rowOfBoardEndingRecord = "* Comment 0 0\n";
            outputStream.write(rowOfBoardEndingRecord.getBytes(StandardCharsets.UTF_8));
            boardEndingRecord += rowOfBoardEndingRecord;

            String[] timeArray = RoomGST.TrimEnd(roomGSTTable.getTimeList(), ',').split(",");
            for (int i = 0; i < timeArray.length; i++)
            {
                rowOfBoardEndingRecord = String.format("%d. %s\n", i, timeArray[i]);
                outputStream.write(rowOfBoardEndingRecord.getBytes(StandardCharsets.UTF_8));
                boardEndingRecord += rowOfBoardEndingRecord;
            }
            
            String[] eatArray = RoomGST.TrimEnd(roomGSTTable.getEatList(), ',').split(",");
            for (int i = 0; i < eatArray.length; i++)
            {
                rowOfBoardEndingRecord = String.format("%d. %s\n", i, eatArray[i]);
                outputStream.write(rowOfBoardEndingRecord.getBytes(StandardCharsets.UTF_8));
                boardEndingRecord += rowOfBoardEndingRecord;
            }

            outputStream.flush();
            outputStream.close();
            
		} catch (Exception exception) {
			exception.printStackTrace();
		}
        return boardEndingRecord;
    }    

//	private String getScore(String judgeResult, int whoIsWinner, String gameStarted, int[] remainGhostCount) {
//    	double[] playerTotal = new double[] { 0, 0 };
//        double[] pieceTotal = new double[] { 0, 0 };
//        judgeResult.split(",");
//        if (judgeResult.contains("Draw"))
//        {
//            playerTotal[0] += drawScore;
//            playerTotal[1] += drawScore;
//        }
//        else
//        {
//            playerTotal[0] += whoIsWinner == 1 ? winScore : loseScore;
//            playerTotal[1] += whoIsWinner == 1 ? loseScore : winScore;
//        }
//        if (gameStarted.equals("0"))
//        {
//            for (int i = 0; i < 8; i++)
//            {
//                pieceTotal[0] += remainGhostCount[i] * pieceScore[i];
//                pieceTotal[1] += remainGhostCount[8 + i] * pieceScore[8 + i];
//            }
////            for (int i = 0; i < 7; i++)
////            {
////                pieceTotal[0] += remainGhostCount[i] * pieceScore[i];
////                pieceTotal[1] += remainGhostCount[7 + i] * pieceScore[7 + i];
////            }
//        }
//        else if (gameStarted.equals("1"))
//        {
//            for (int i = 0; i < 8; i++)
//            {
//                pieceTotal[1] += remainGhostCount[i] * pieceScore[i];
//                pieceTotal[0] += remainGhostCount[8 + i] * pieceScore[8 + i];
//            }
////            for (int i = 0; i < 7; i++)
////            {
////                pieceTotal[1] += remainGhostCount[i] * pieceScore[i];
////                pieceTotal[0] += remainGhostCount[7 + i] * pieceScore[7 + i];
////            }
//        }
//        if (pieceTotal[0] > pieceTotal[1] && (playerTotal[0] == winScore || playerTotal[0] == drawScore))
//        {
//            if (pieceTotal[0] - pieceTotal[1] >= bonuslimit)
//            {
//                playerTotal[0] += bonus;
//            }
//            else
//            {
//                playerTotal[0] += bonus / bonuslimit * (pieceTotal[0] - pieceTotal[1]);
//            }
//        }
//        if (pieceTotal[1] > pieceTotal[0] && (playerTotal[1] == winScore || playerTotal[1] == drawScore))
//        {
//            if (pieceTotal[1] - pieceTotal[0] >= bonuslimit)
//            {
//                playerTotal[1] += bonus;
//            }
//            else
//            {
//                playerTotal[1] += bonus / bonuslimit *  (pieceTotal[1] - pieceTotal[0]);
//            }
//        }
//        return String.format("%f,%f", playerTotal[0], playerTotal[1]);
//    }

    private byte[] shortToByteArray(short shortNum){
        return new byte[]{(byte)shortNum, (byte)(shortNum >> 8)};
    }

    private  byte[] intToByteArray(int intNum){
        return new byte[]{(byte)intNum, (byte)(intNum >> 8), (byte)(intNum >> 16), (byte)(intNum >> 24)};
    }

    private short byteArrayToShort(byte[] byteArray, int offset){
        return (short) ((byteArray[offset + 1] << 8) & 0xff00 | byteArray[offset] & 0x00ff);
    }

    private int byteArrayToInt(byte[] byteArray, int offset){
        return ((byteArray[offset + 3] << 24) & 0xff000000 | (byteArray[offset + 2] << 16) & 0x00ff0000 | (byteArray[offset + 1] << 8) & 0x0000ff00 | byteArray[offset] & 0x000000ff);
    }
    
    public static String TrimEnd(String String, char characters) {
    	return String.replaceAll("[" + characters + "]+$", "");
	}
	
	public static void DEBUG(String message)
    {
        System.out.println(message);
    }
}
