package gst;

import java.io.File;
import client.BoardWebSocketServer;
import client.Game;
import client.Message;
import client.ProgramPath;
import client.Room;
import client.Room_SettingFile.BoardDisconnectAutoRestart;
import client.Room_SettingFile.BoardOpenType;
import client.SettingFile.STARTMODE;
import client.Warning;

/**
 * 
 * 繼承至 {@link client.BoardWebSocketServer}，管理暗棋任何與 Web 盤面顯示有關的 class。
 * 
 * @author ldslds449
 * @see client.BoardWebSocketServer
 *
 */
class GST_Board extends BoardWebSocketServer{
	
	private STARTMODE startMode;
	private BoardDisconnectAutoRestart restart;
	private GST_Room room;
	private GST_Game game;
	
	/**
	 * 
	 * GST_Board 的 constructor。
	 * 
	 * @param type 盤面開啟類型，主要用來區別是遠端還是本地盤面，{@link client.Room_SettingFile.BoardOpenType}。
	 * @param startMode 進房類別，開房還是加房，初始的 port 不同，{@link client.SettingFile.STARTMODE}。
	 * @param room 給 {@link client.BoardWebSocketServer} 來顯示連接的 port 在 windows title。
	 * @param isCLI 是否為 CLI mode。
	 * @throws Warning 發生錯誤。
	 */
	GST_Board(BoardOpenType type, STARTMODE startMode, BoardDisconnectAutoRestart restart, GST_Room room, GST_Game game, boolean isCLI, boolean isPlay, boolean needCloseWeb) throws Warning{
		super(type, startMode, (Room)room, game, "GST", isCLI, isPlay, needCloseWeb);
		this.startMode = startMode;
		this.restart = restart;
		this.room = room;
		this.game = game;
	}
	
	/**
	 * 
	 * 傳送初始盤面訊息給 Web。此為初始盤面時開啟盤面所呼叫的 function
	 * 
	 * @param data 初始盤面訊息，包含走步、時間、最終盤面和初始盤面。
	 * @param step_time 每個走步所花費的時間。
	 * @param total_spent_time 從遊戲開始到每個走步，各玩家所花費的時間累積。
	 * @param times 各玩家所剩餘的時間。
	 * @throws Warning 發生錯誤。
	 */
	void sendInitBoard(String data, Integer[] step_time, Integer[] total_spent_time, int[] times, int playerNum) throws Warning{
		String temp[] = data.split(",");
		StringBuilder sb = new StringBuilder();
		
        // board + remain chess
        for(int i = 0; i < 40; ++i){
            sb.append(temp[i]);
            sb.append(',');
        }
//        // first color
//        sb.append(temp[40]);
//        sb.append(',');
		
		// steps num
		int step_num = Integer.parseInt(temp[41]);
		sb.append(step_num);
		sb.append(',');
		
		//playerNum = 0 in initial
		sb.append("" + playerNum);
		
		// steps
		for(int i = 0; i < step_num; i++){
			sb.append(',');
			sb.append(temp[42 + i]);
			sb.append(',');
			sb.append(temp[43 + step_num + i]); //吃子顏色
			sb.append(',');
			sb.append((i + 1) + ""); //step num
			sb.append(',');
			sb.append(step_time[i]);
			sb.append(',');
			sb.append(total_spent_time[i]);
		}
		
		// first ply player time
		sb.append(',');
		sb.append(times[0]);
		// second ply player time
		sb.append(',');
		sb.append(times[1]);
		
		send(sb.toString());
	}
//	void sendInitBoard(String data, Integer[] step_time, Integer[] total_spent_time, int[] times) throws Warning{
//		String temp[] = data.split(",");
//		StringBuilder sb = new StringBuilder();
//		
//        // board + remain chess
//        for(int i = 0; i < 46; ++i){
//            sb.append(temp[i]);
//            sb.append(',');
//        }
//        // first color
//        sb.append(temp[46]);
//        sb.append(',');
//		
//		// steps num
//		int step_num = Integer.parseInt(temp[47]);
//		sb.append(step_num);
//		// steps
//		for(int i = 0; i < step_num; i++){
//			sb.append(',');
//			sb.append(temp[48 + i]);
//			sb.append(',');
//			sb.append(step_time[i]);
//			sb.append(',');
//			sb.append(total_spent_time[i]);
//		}
//		// noEatFlip (noEatFlip Number always start from zero)
//		sb.append(',');
//		sb.append(0);
//		// first ply player time
//		sb.append(',');
//		sb.append(times[0]);
//		// second ply player time
//		sb.append(',');
//		sb.append(times[1]);
//		
//		send(sb.toString());
//	}
	
	/**
	 * 
	 * 傳送初始盤面訊息給 Web。 此為遊戲進行到中途開啟盤面所呼叫的 function。
	 * 
	 * @param data 初始盤面訊息，包含走步、時間、最終盤面和初始盤面(只用到初始盤面訊息)。
	 * @param steps 遊戲進行到當下所有走步。
	 * @param step_time 每個走步所花費的時間。
	 * @param total_spent_time 從遊戲開始到每個走步，各玩家所花費的時間累積。
	 * @param times 各玩家所剩餘的時間。
	 * @throws Warning 發生錯誤。
	 */
	void sendInitBoard(String data, String[] steps, String[] eats, Integer[] step_time, Integer[] total_spent_time, int[] times, int playerNum) throws Warning{
		String temp[] = data.split(",");
		StringBuilder sb = new StringBuilder();
		
        // board + remain chess
        for(int i = 0; i < 40; ++i){
            sb.append(temp[i]);
            sb.append(',');
        }
        // first color
//        sb.append(temp[40]);
//        sb.append(',');
		
		// steps num
		int step_num = steps.length;
		sb.append(step_num);
		sb.append(',');
		
		// playerNum (用於判斷是否輪到自己)
		sb.append("" + playerNum);
		
		// steps
		for(int i = 0; i < step_num; i++){
			sb.append(',');
			sb.append(steps[i].split(",")[0] + " " + steps[i].split(",")[1]); //A,NORTH -> A NORTH
			sb.append(',');
			sb.append(eats[i]);  //吃子顏色 0:沒吃
			sb.append(',');
			sb.append((i + 1) + ""); //step num
			sb.append(',');
			sb.append(step_time[i]);
			sb.append(',');
			sb.append(total_spent_time[i]);
		}
		// player 1 time
		sb.append(',');
		sb.append(times[0]);
		
		// player 1 time
		sb.append(',');
		sb.append(times[1]);
		
		send(sb.toString());
	}
//	void sendInitBoard(String data, String[] steps, Integer[] step_time, Integer[] total_spent_time, int[] times) throws Warning{
//		String temp[] = data.split(",");
//		StringBuilder sb = new StringBuilder();
//		
//        // board + remain chess
//        for(int i = 0; i < 46; ++i){
//            sb.append(temp[i]);
//            sb.append(',');
//        }
//        // first color
//        sb.append(temp[46]);
//        sb.append(',');
//		
//		// steps num
//		int step_num = steps.length;
//		sb.append(step_num);
//		// steps
//		for(int i = 0; i < step_num; i++){
//			sb.append(',');
//			sb.append(steps[i]);
//			sb.append(',');
//			sb.append(step_time[i]);
//			sb.append(',');
//			sb.append(total_spent_time[i]);
//		}
//		// noEatFlip (noEatFlip Number always start from zero)
//		sb.append(',');
//		sb.append(0);
//		// first move player time
//		sb.append(',');
//		sb.append(times[0]);
//		// second move player time
//		sb.append(',');
//		sb.append(times[1]);
//		
//		send(sb.toString());
//	}
//	
	
	/**
	 * 傳送玩家名稱給 Web。
	 * @param first_move_name 先手玩家名稱。
	 * @param second_move_name 後手玩家名稱。
	 */
	void sendPlayerName(String first_move_name, String second_move_name){
		send(String.format("players %s %s", first_move_name, second_move_name));
	}
	
	/**
	 * 
	 * 傳送走步。
	 * 
	 * @param step 走步訊息，翻棋或移子皆可。
	 * @param noEatFlip 當前無吃翻累計數字。
	 * @param step_time 當前走步所花費的時間。
	 * @param total_spent_time 當前走步的玩家到當前走步所花費的累積時間。
	 * @param time 當前走步的玩家所剩餘的時間。 
	 * @throws Warning 發生錯誤。
	 */
	void sendMove(String step, int capturePieceColor, int noEatFlip, int step_time, int total_spent_time, int time) throws Warning{
		StringBuilder sb = new StringBuilder();
		// step
		sb.append("move ");
		sb.append(step);
		//capturePieceColor
		sb.append(' ');
		sb.append(capturePieceColor);
		// noEatFlip
		sb.append(' ');
		sb.append(noEatFlip);
		// this step take how much time
		sb.append(' ');
		sb.append(step_time);
		// until this step, the player spent how much time
		sb.append(' ');
		sb.append(total_spent_time);
		// player remain time
		sb.append(' ');
		sb.append(time);
		
		send(sb.toString());
	}
//	void sendMove(String step, int noEatFlip, int step_time, int total_spent_time, int time) throws Warning{
//		StringBuilder sb = new StringBuilder();
//		// step
//		if(step.charAt(2) == '('){
//			sb.append("flip ");
//		}else{
//			sb.append("move ");
//		}
//		sb.append(step);
//		// noEatFlip
//		sb.append(' ');
//		sb.append(noEatFlip);
//		// this step take how much time
//		sb.append(' ');
//		sb.append(step_time);
//		// until this step, the player spent how much time
//		sb.append(' ');
//		sb.append(total_spent_time);
//		// player remain time
//		sb.append(' ');
//		sb.append(time);
//		
//		send(sb.toString());
//	}
	
	/**
	 * 
	 * 傳送重置盤面指令給 Web。
	 * 
	 * @throws Warning 發生錯誤。
	 */
	void restartBoard() throws Warning{
		send("reset_board");
	}
	
	/**
	 * 
	 * 傳送遊戲結果給 Web。
	 * 
	 * @param result 遊戲結果。
	 * @throws Warning 發生錯誤。
	 */
	void sendResult(String result) throws Warning {
		send(String.format("result %s", result));
	}
	
	/**
	 * 
	 * {@inheritDoc}
	 * 用來取得此遊戲的 HTML 檔案路徑位置及名稱。
	 * 
	 * @return 盤面顯示檔案 (HTML) 的檔案路徑位置及名稱。
	 */
	@Override
	protected String getHtmlFileName() {
		if(this.startMode == STARTMODE.OPEN){
			return String.format("%s%sWebServer%s%s%sopen.html", 
					ProgramPath.getProgramPath(),
					File.separator,
					File.separator,
					GameInfo.getGameName(),
					File.separator);
		}else{
			return String.format("%s%sWebServer%s%s%senter.html", 
					ProgramPath.getProgramPath(),
					File.separator,
					File.separator,
					GameInfo.getGameName(),
					File.separator);
		}
	}
	
	@Override
	protected void boardCloseErrorListener(){
		if(restart == BoardDisconnectAutoRestart.YES){
			try {
				game.openBoard(true);
			} catch (Warning e) {
				Message.printError("Auto Restart ERROR");
				e.printStackTrace(Message.errorStream);
			}
		}
	}
}
