package gst;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import gst.GST_SettingFile.CONNECTMODE;
import client.Contest;
import client.HintTextField;
import client.Message;
import client.ProgramPath;
import client.Room;
import client.Room_SettingFile;

import java.awt.GridBagLayout;
import javax.swing.JComboBox;

/**
 * 
 * 繼承至 {@link client.Contest}，處理比賽模式介面的 class。
 * 
 * @author ldslds449
 * @see client.Contest
 *
 */

public class GST_Contest extends Contest{
	
	private JTextField Search_Field;
	private JTextField Search_Arg_Field;
	private JButton Search_Button;
	private JPanel Protocol_Panel;
	private JLabel Protocol_Label;
	private JComboBox<CONNECTMODE> Protocol_ComboBox;
	
	
	/**
	 * 
	 * constructor，負責初始化比賽選單介面。
	 * 
	 * @param setting 遊戲設定，初始化遊戲設定。
	 * @param room 房間變數，用來鎖定房間頁面和置中。
	 * @param roomSetting 房間設定，初始化父類別。
	 */
	GST_Contest(GST_SettingFile setting, Room room, Room_SettingFile roomSetting) {
		super(room, roomSetting);
		GridBagLayout gridBagLayout = (GridBagLayout) getContentPane().getLayout();
		gridBagLayout.columnWidths = new int[]{};
		gridBagLayout.columnWeights = new double[]{1.0};
		gridBagLayout.rowWeights = new double[]{1.0, 0.0, 0.0, 0.0, 0.0, 0.0};
		gridBagLayout.rowHeights = new int[]{0, 40, 40, 10, 28, 10};
		
		Protocol_Panel = new JPanel();
		Protocol_Panel.add(Box.createRigidArea(new Dimension(10, 0)));
		GridBagConstraints gbc_Protocol_Panel = new GridBagConstraints();
		gbc_Protocol_Panel.insets = new Insets(0, 0, 5, 0);
		gbc_Protocol_Panel.fill = GridBagConstraints.BOTH;
		gbc_Protocol_Panel.gridx = 0;
		gbc_Protocol_Panel.gridy = 1;
		getContentPane().add(Protocol_Panel, gbc_Protocol_Panel);
		Protocol_Panel.setLayout(new BoxLayout(Protocol_Panel, BoxLayout.X_AXIS));
		
		Protocol_Label = new JLabel("Protocol");
		Protocol_Label.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		Protocol_Panel.add(Protocol_Label);
		Protocol_Panel.add(Box.createRigidArea(new Dimension(10, 0)));
		
		Protocol_ComboBox = new JComboBox<CONNECTMODE>();
		Protocol_ComboBox.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		for(CONNECTMODE p : CONNECTMODE.values()){
			Protocol_ComboBox.addItem(p);
		}
		Protocol_ComboBox.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				setting.connectMode = (CONNECTMODE)Protocol_ComboBox.getSelectedItem();
			}
		});
		Protocol_Panel.add(Protocol_ComboBox);
		Protocol_Panel.add(Box.createRigidArea(new Dimension(10, 0)));
		
		JPanel Search_Panel = new JPanel();
		Search_Panel.add(Box.createRigidArea(new Dimension(10, 0)));
		GridBagConstraints gbc_Search_Panel = new GridBagConstraints();
		gbc_Search_Panel.insets = new Insets(0, 0, 5, 0);
		gbc_Search_Panel.fill = GridBagConstraints.BOTH;
		gbc_Search_Panel.gridx = 0;
		gbc_Search_Panel.gridy = 2;
		getContentPane().add(Search_Panel, gbc_Search_Panel);
		Search_Panel.setLayout(new BoxLayout(Search_Panel, BoxLayout.X_AXIS));
		
		JLabel AI_Path_Label = new JLabel("AI Path   ");
		AI_Path_Label.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		Search_Panel.add(AI_Path_Label);
		
		Search_Panel.add(Box.createRigidArea(new Dimension(10, 0)));
		
		Search_Field = new JTextField();
		Search_Field.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		Search_Field.getDocument().addDocumentListener(new DocumentListener() {
			public void changedUpdate(DocumentEvent e) {
				setting.searchPath = Search_Field.getText();
			}

			@Override
			public void insertUpdate(DocumentEvent arg0) {
				setting.searchPath = Search_Field.getText();
			}

			@Override
			public void removeUpdate(DocumentEvent arg0) {
				setting.searchPath = Search_Field.getText();
			}
		});
		Search_Panel.add(Search_Field);
		Search_Panel.add(Box.createRigidArea(new Dimension(10, 0)));
		
		Search_Button = new JButton("\u9078\u64C7");
		Search_Button.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		Search_Button.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				JFileChooser jfc = new JFileChooser();
				File f = new File(Search_Field.getText());
				
				if(f.isFile()){
					jfc.setCurrentDirectory(f.getAbsoluteFile().getParentFile());
				}else{
					jfc.setCurrentDirectory(new File(String.format("%s%sSearch", ProgramPath.getProgramPath(), File.separator)));
				}
				jfc.resetChoosableFileFilters();
				int selectValue = jfc.showDialog(room, "\u9078\u64c7");
				
				if(selectValue == JFileChooser.APPROVE_OPTION){
					File fl = jfc.getSelectedFile();
					Search_Field.setText(fl.getAbsolutePath());
					// store to setting
					setting.searchPath = fl.getAbsolutePath();
				}
			}
		});
		Search_Panel.add(Search_Button);
		Search_Panel.add(Box.createRigidArea(new Dimension(10, 0)));
		
		JPanel Search_Arg_Panel = new JPanel();
		Search_Arg_Panel.add(Box.createRigidArea(new Dimension(10, 0)));
		GridBagConstraints gbc_Search_Arg_Panel = new GridBagConstraints();
		gbc_Search_Arg_Panel.insets = new Insets(0, 0, 5, 0);
		gbc_Search_Arg_Panel.fill = GridBagConstraints.BOTH;
		gbc_Search_Arg_Panel.gridx = 0;
		gbc_Search_Arg_Panel.gridy = 3;
		getContentPane().add(Search_Arg_Panel, gbc_Search_Arg_Panel);
		Search_Arg_Panel.setLayout(new BoxLayout(Search_Arg_Panel, BoxLayout.X_AXIS));
		
		JLabel AI_Arg_Label = new JLabel("AI Arg    ");
		AI_Arg_Label.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		Search_Arg_Panel.add(AI_Arg_Label);
		
		Search_Arg_Panel.add(Box.createRigidArea(new Dimension(10, 0)));
		
		Search_Arg_Field = new HintTextField("( Empty )");
		Search_Arg_Field.setText(setting.arg);
		Search_Arg_Field.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		Search_Arg_Field.getDocument().addDocumentListener(new DocumentListener() {
			public void changedUpdate(DocumentEvent e) {
				setting.arg = Search_Arg_Field.getText();
			}

			@Override
			public void insertUpdate(DocumentEvent arg0) {
				setting.arg = Search_Arg_Field.getText();
			}

			@Override
			public void removeUpdate(DocumentEvent arg0) {
				setting.arg = Search_Arg_Field.getText();
			}
		});
		Search_Arg_Panel.add(Search_Arg_Field);
		Search_Arg_Panel.add(Box.createRigidArea(new Dimension(10, 0)));
		
		GridBagConstraints gbc_glue_1 = new GridBagConstraints();
		gbc_glue_1.insets = new Insets(0, 0, 5, 0);
		gbc_glue_1.gridx = 0;
		gbc_glue_1.gridy = 3;
		getContentPane().add(Box.createGlue(), gbc_glue_1);
		
		GridBagConstraints gbc_glue = new GridBagConstraints();
		gbc_glue.gridx = 0;
		gbc_glue.gridy = 5;
		getContentPane().add(Box.createGlue(), gbc_glue);
		
		// set initial value
		Search_Field.setText(setting.searchPath);
//		Search_Arg_Field.setText(setting.arg);
		Protocol_ComboBox.setSelectedItem(setting.connectMode);
	}
	
	/**
	 * 
	 * {@inheritDoc}
	 * 用來開啟/鎖定設定選項按鈕的 function。
	 * 
	 * @param state 開啟 or 鎖定。
	 */
	@Override
	protected void enableComponent(boolean state) {
		Search_Field.setEnabled(state);
		Search_Button.setEnabled(state);
		Search_Arg_Field.setEnabled(state);
		Protocol_ComboBox.setEnabled(state);
	}

}
