package gst;

import java.awt.Dialog.ModalityType;
import java.awt.Frame;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Scanner;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import org.apache.commons.io.comparator.NameFileComparator;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.exception.ExceptionUtils;

import client.BoardWebSocketServer;
import client.Contest;
import client.Game;
import client.GetRoomList;
import client.Message;
import client.Packet;
import client.Pair;
import client.ProgramPath;
import client.Room;
import client.RoomList;
import client.Room_SettingFile;
import client.Room_SettingFile.BoardFileOutput;
import client.Room_SettingFile.CommandLogOutput;
import client.Room_SettingFile.RoomStatisticOutput;
import client.Server;
import client.SettingFile;
//import client.SettingFile.OPENINGMODE;
import client.SettingFile.ROOMTYPE;
import client.Warning;
import client.Warning.Type;
import gst.GST_SettingFile.CHANGEFIRSTMOVE;
import gst.GST_SettingFile.FIRSTMOVE;
import gst.GST_SettingFile.MIDBOARD;

/**
 * 與遊戲流程有關的管理和控制。
 * @author ldslds449
 * @see client.Game
 */
public class GST_Game extends Game{
	
	private SearchConnect connect = null;
	
	private int Turn;
	private int gameStarted; // gameStarted != 0 代表中盤  
	private int setRedSuccess = 0;
	private int TakeTime[] = new int[2]; // initial board remain time (ms) [0]: first player [1]: second player
	private int PlayerNum = -1;
	private String opponentName;
	private boolean opponentisHuman;
	
	private int Time;
	private int OppoTime;
	private long searchTime = 0;
	private int errorTime = 0;
	
	private int Win = 0; // accumulative Win times
	private int Draw = 0; // accumulative Draw times
	private int Lose = 0; // accumulative Lose times
	private int Abort = 0; // consecutive Abort times
	private int[] LastResult; // use for getFinishInfo()
	
	private String BoardRecordDirName = null;
	private String GameResult;
	private double FirstScore; // first hand score
	private double SecondScore; // second hand score
	private String InitialBoard;
	private boolean MidBoardFirstStep = false;
	private boolean needToCleanDebugInfo = false;
	
	private Thread DOReadStepThread;
	private BooleanHolder DOReadStepRunning = new BooleanHolder();
	private Thread DOReadResponseThread;
	private BooleanHolder DOReadResponseRunning = new BooleanHolder();
	private Runnable AskMoveRunnable;
//	private Runnable AskSetRedRunnable;
	
	private String stdout_stamp = null;
	private String stderr_stamp = null;
	
	private ArrayList<String> steps;
	private ArrayList<String> eats;
	private ArrayList<Integer> step_time;
	private ArrayList<Integer> total_spent_time;
	
	private String Search_Name = "";
	private String Search_Version = "";
	
	private String initBoardData = "";
	
	private Lock lock = new ReentrantLock();
	
	private static final long SECTOMILLISEC = 1000;
	
	/**
	 * constructor。
	 * @param server 與 Server 之間的 Socket 連線。
	 * @param setting 遊戲參數。
	 * @param room 房間頁面的 instance
	 * @param roomSetting 房間參數。
	 * @param isCLI 當前是否為 CLI mode。
	 */
	public GST_Game(Server server, GST_SettingFile setting, GST_Room room, Room_SettingFile roomSetting, boolean isCLI, boolean isPlay, Integer specific_roomID) {
		super(server, setting, room, roomSetting, isCLI, isPlay, specific_roomID);
	}
	
	/**
	 * {@inheritDoc}
	 * 針對 Server 發送來的指令做出相對應的動作。
	 * @param p 伺服器發過來的封包。
	 */
	@Override
	protected void analysisAction(Packet p) throws Warning{
		String[] temp = p.getData().split(",");
		
		// stop thread
		DOReadResponseRunning.setFalse();
		
		switch(p.getAction()){
		case GST_Action.InitialSetting:
			// <"">
			// clean debug info
			if(needToCleanDebugInfo){
				if(!this.isCLI){room.cleanDebugText();}
				
				// send board reset info
				if(((GST_SettingFile)Setting).connectMode == GST_SettingFile.CONNECTMODE.HUMAN){
					((Human) connect).restartBoard();
				}else if(board != null){
					((GST_Board)board).restartBoard();
				}
				
				if(!(connect instanceof Human)){
					// start search.exe
					startSearch();
				}
			}
			isFinishAGame = false;
			if(connect instanceof Human){
				((Human)connect).setisFinish(new BooleanHolder(false));
			}
			
			
			SendData(new Packet(GST_Action.InitialSetting, "1"));
			break;
		case GST_Action.SetRed:
			// <message> <isAck>
			if(temp[1].equals("0")) {
				Message.printInfo("Waiting for set red ghost");
				// initial
				lock.lock(); // lock steps, step_time, total_spent_time
				steps = new ArrayList<String>();
				eats = new ArrayList<String>();
				step_time = new ArrayList<Integer>();
				total_spent_time = new ArrayList<Integer>();
				lock.unlock();
				setRedSuccess = 0;
				if(connect instanceof Human){
					((Human)connect).setisFinish(new BooleanHolder(false));
				}
				// record search time
				long startSetTime = System.currentTimeMillis();
				// ask move
				AskMoveRunnable = new Runnable(){
					@Override
					public void run() {
						try{
							Message.printInfo("Ask for set red ghost");
							Packet tmp = connect.askSetRed();
							searchTime = System.currentTimeMillis() - startSetTime;
							// send move
							if(!isFinishAGame){
								SendData(tmp);
							}
							// to get the human step
//							if(connect instanceof Human && tmp.getAction() == GST_Action.DrawOffer){
//								DOReadStepRunning.setTure();
//								DOReadStepThread = new Thread(new Runnable(){
//									@Override
//									public void run() {
//										try {
//											Packet p = ((Human)connect).listenDrawOfferStep(DOReadStepRunning);
//											if(p != null && DOReadStepRunning.getValue()){
//												SendData(p);
//											}
//										} catch (Warning e) {
//											Message.printError(ExceptionUtils.getStackTrace(e));
//											if(!isFinishAGame){
//												ErrorOccur(e);
//											}
//										}
//									}
//								});
//								DOReadStepThread.start();
//							}
						}catch(Warning w){ 
							Message.printError(w.toString()); 
							if(!isFinishAGame){
								ErrorOccur(w); 
							}
						}
					}
				};
				if(!isFinishAGame){
					new Thread(AskMoveRunnable).start();
				}
			}else {
				setRedSuccess = 1;
				if(!(connect instanceof Human)) {
				connect.sendMessage(temp[0], 0, true);
				}
			}
			
			break;
		case GST_Action.InitialBoard://初始化棋盤的動作
			// <board> <RemainGhostCount> <gameStarted> <stepListLength> <stepList> <eatListLength> <eatList> <timeListLength> <timeList>
			// 0-35    36-39              40            
						
			InitialBoard = p.getData();
			//gameStarted != 0 代表中盤
			gameStarted = Integer.parseInt(temp[40]);

			if(!this.isCLI){
				((GST_Room)room).clearInteractiveLog();
			}
			
			
			
			// start record temp file (local board record)
			processRecord = new GST_ProcessRecord(Rounds + 1, InitialBoard, getSearchInfo(), Setting);
			
			MidBoardFirstStep = ((GST_SettingFile)Setting).midBoard == GST_SettingFile.MIDBOARD.YES ? true : false;
			
			
			// count initial time (ms)
			// [0]: game first player [1]: game second player
			TakeTime = new int[2];
			
			if(gameStarted != 0){
				lock.lock(); // lock steps, step_time, total_spent_time
				steps = new ArrayList<String>();
				eats = new ArrayList<String>();
				step_time = new ArrayList<Integer>();
				total_spent_time = new ArrayList<Integer>();
				// step
				int init_step_len = Integer.parseInt(temp[41]);
				for(int i = 0; i < init_step_len; i++){
//					steps.add(temp[42 + i].split(" ")[0] + "," + temp[42 + i].split(" ")[1]);
					steps.add(temp[42 + i].replace(' ', ','));
				}
				// eats
				int init_eat_len = Integer.parseInt(temp[42 + init_step_len]);
				for(int i = 0; i < init_eat_len; i++){
					eats.add(temp[43 + init_step_len + i]);
				}
				// time
				int init_time_len = Integer.parseInt(temp[43 + init_step_len + init_eat_len]);
				TakeTime[0] = (int)(((GST_SettingFile)Setting).timeLimit * SECTOMILLISEC);
				TakeTime[1] = (int)(((GST_SettingFile)Setting).timeLimit * SECTOMILLISEC); 
				
				Long pre_time = Long.parseLong(temp[44 + init_step_len + init_eat_len]);
				
				for(int i = 1; i < init_time_len; i++){
					Long now_time = Long.parseLong(temp[44 + init_step_len + init_eat_len + i]);
					TakeTime[(i - 1) % 2] -= (int)(now_time - pre_time);
					
					// record (i starts one)
					step_time.add((int)(now_time - pre_time));
					total_spent_time.add((int)(((GST_SettingFile)Setting).timeLimit * SECTOMILLISEC) - TakeTime[(i - 1) % 2]);
					
					pre_time = now_time;
				}
				setRedSuccess = 1;
				lock.unlock();
			}else{
				TakeTime[0] = (int)(((GST_SettingFile)Setting).timeLimit * SECTOMILLISEC);
				TakeTime[1] = (int)(((GST_SettingFile)Setting).timeLimit * SECTOMILLISEC);
			}
			
			// send initial board to webServer (Human mode send in line 304)
			if(board != null){
				lock.lock(); // lock steps, step_time, total_spent_time
				((GST_Board)board).sendInitBoard(p.getData(), step_time.toArray(new Integer[0]), total_spent_time.toArray(new Integer[0]), TakeTime, 0);
				lock.unlock();
			}

			
			initBoardData = p.getData();
			
			SendData(new Packet(GST_Action.InitialBoard, "1"));
			break;
		case GST_Action.GameStart://開始遊戲動作
			// <first move player> <first move player kind> <second move player> <second move player kind> <playerNum> <firstMoveGhostCodeInfo>
			
			isFinishAGame = false;
			// <playerNum> (1: game first move 2: game second move)
			PlayerNum = Integer.parseInt(temp[4]);
			
			// opponent name
			opponentName = (PlayerNum == 1 ? temp[2] : temp[0]);
			opponentisHuman = (PlayerNum == 1 ? (temp[3].equals("1") ? true : false) : (temp[1].equals("1") ? true : false));
			
			// send players name to board
			if(((GST_SettingFile)Setting).connectMode == GST_SettingFile.CONNECTMODE.HUMAN){
				// Do Nothing
			}else if(board != null){
				((GST_Board)board).sendPlayerName(temp[0], temp[2]);
			}
			
			// set opponent if is the Human
			if(connect instanceof Human){
				((Human)connect).setOppoisHuman(opponentisHuman);
				((Human)connect).setisFinish(new BooleanHolder(false));
				((Human)connect).setPlayerName(temp[0], temp[2]);
			}
			
			// readfile and human need to set player name
//			if(connect instanceof ReadFile){
//				((ReadFile) connect).setPlayerName(temp[0], temp[2]);
//			}
			if(connect instanceof Human) {
				lock.lock(); // lock steps, step_time, total_spent_time
				if(((GST_SettingFile)Setting).timeLimit == 0 &&
						((GST_SettingFile)Setting).onePlyTime == 0 &&
						((GST_SettingFile)Setting).plyShare == 0){
					int all_zero[] = {0, 0};
					connect.setGameInfo(Setting, 
							InitialBoard, 
							0,
							all_zero,
							step_time.toArray(new Integer[0]),
							total_spent_time.toArray(new Integer[0]));
				}else{
					connect.setGameInfo(Setting, 
							InitialBoard, 
							PlayerNum == 1 ? TakeTime[0] : TakeTime[1],
							TakeTime,
							step_time.toArray(new Integer[0]),
							total_spent_time.toArray(new Integer[0]));
				}
				lock.unlock();
			}
			
			
			// Create Board Name
			if(BoardRecordDirName == null){
				DateFormat dateFormat = new SimpleDateFormat("yyyy_MM_dd_HH_mm_ss");
				Date date = new Date();
				
				File directory_board = new File(String.format("%s%sBoard", ProgramPath.getProgramPath(), File.separator));
			    if (!directory_board.exists()){
			    	directory_board.mkdir();
			    }
			    
			    File directory_gst = new File(String.format("%s%s%s", directory_board.getAbsolutePath(), File.separator, GameInfo.getGameName()));
			    if (!directory_gst.exists()){
			    	directory_gst.mkdir();
			    }
			    
			    BoardRecordDirName = String.format("%s%sBoard%s%s%s%s_vs_%s_%s", ProgramPath.getProgramPath(), File.separator, File.separator, GameInfo.getGameName(), File.separator, Setting.account, opponentName, dateFormat.format(date));
			    File BoardRecordDir = new File(BoardRecordDirName);
			    if(roomSetting.boardFileOutput == BoardFileOutput.YES ||
		    		roomSetting.commandLogOutput == CommandLogOutput.YES ||
		    		roomSetting.roomStatisticOutput == RoomStatisticOutput.YES){
			    	if(!BoardRecordDir.exists()){
						BoardRecordDir.mkdir();
					}
			    }
			    ((GST_ProcessRecord)processRecord).setDir(BoardRecordDirName);
			}
			
			// set record temp file (local board record) Info
			((GST_ProcessRecord)processRecord).setPlayerName(temp[0], temp[2]);
			if(roomSetting.boardFileOutput == BoardFileOutput.YES){
				((GST_ProcessRecord)processRecord).writeTempFile(connect.getStdoutStamps(), connect.getStderrStamps(), connect.getConnectInfo(), connect.getErrorInfo());
			}
			
			// set the initial Board color
			int initial_board_color;
//			Message.printInfo("gameStarted: %d", gameStarted);
			Message.printInfo("PlayerNum: %d", PlayerNum);
			if(PlayerNum == 1) {
				initial_board_color = 0;
			}else {
				initial_board_color = 1;
			}
			
			Message.printInfo("initial_board_color: %d", initial_board_color);
			
			// set gui color
			if(!this.isCLI){
				((GST_Room)room).setPlayerColor(initial_board_color);
			}
			// set record temp file color
			((GST_ProcessRecord)processRecord).setColor(initial_board_color);
			
			
			// set color
			if(connect instanceof Ponder){
				((Ponder) connect).setColor(initial_board_color);
			}else if(connect instanceof MGTP){
				((MGTP) connect).setColor(initial_board_color);
			}else if(connect instanceof Human){
				((Human) connect).setColor(initial_board_color);
			}
			
			
			// set Room display
			if(((GST_SettingFile)Setting).midBoard == GST_SettingFile.MIDBOARD.YES){ // use disconnect midboard
				if(Rounds + 1 == 1){ // first round
					if(Setting.startMode == SettingFile.STARTMODE.OPEN){ // only open room need to change
						if(PlayerNum == 1){ // first move
							((GST_SettingFile)Setting).firstMove = GST_SettingFile.FIRSTMOVE.YES;
						}else{ // second move
							((GST_SettingFile)Setting).firstMove = GST_SettingFile.FIRSTMOVE.NO;
						}
					}
				}
			}
			if(!this.isCLI){
				((GST_Room)room).setRoomInfo(Setting, opponentName, opponentisHuman);
				if(((GST_SettingFile)Setting).connectMode == GST_SettingFile.CONNECTMODE.HUMAN){
					((GST_Room)room).changeTitle("Human");
				}else{
					((GST_Room)room).changeTitle(((GST_SettingFile)Setting).searchPath);
				}
			}
			
			// initial Time
			if(PlayerNum == 1){
				Time = TakeTime[0];
				OppoTime = TakeTime[1];
			}else{
				Time = TakeTime[1];
				OppoTime = TakeTime[0];
			}
			if(!this.isCLI){
				((GST_Room)room).setGameInfo(Time, OppoTime, 0, Win, Draw, Lose);
			}
			
			boolean first_move = (PlayerNum == 1 && steps.size() % 2 == 0) || (PlayerNum == 2 && steps.size() % 2 == 1);
			connect.gameStart(PlayerNum, first_move);
			
			if(first_move){
				
				Message.printInfo("Waiting for Player %d", PlayerNum);
				// record search time
				long startSearchTime = System.currentTimeMillis();
				// ask move
				AskMoveRunnable = new Runnable(){
					@Override
					public void run() {
						try{
							Packet tmp = connect.askMove(temp[5]);
							searchTime = System.currentTimeMillis() - startSearchTime;
							// send move
							if(!isFinishAGame){
								SendData(tmp);
							}
							// to get the human step
							if(connect instanceof Human && tmp.getAction() == GST_Action.DrawOffer){
								DOReadStepRunning.setTure();
								DOReadStepThread = new Thread(new Runnable(){
									@Override
									public void run() {
										try {
											Packet p = ((Human)connect).listenDrawOfferStep(DOReadStepRunning);
											if(p != null && DOReadStepRunning.getValue()){
												SendData(p);
											}
										} catch (Warning e) {
											Message.printError(ExceptionUtils.getStackTrace(e));
											if(!isFinishAGame){
												ErrorOccur(e);
											}
										}
									}
								});
								DOReadStepThread.start();
							}
						}catch(Warning w){ 
							Message.printError(w.toString()); 
							if(!isFinishAGame){
								ErrorOccur(w); 
							}
						}
					}
				};
				if(!isFinishAGame){
					new Thread(AskMoveRunnable).start();
				}
			}
			break;
		
		case GST_Action.Move://移動棋子動作 沒有處理行動邏輯
			// <piece code> <direction> <capturePieceColor> <MaxPlyNum> <turn> <nextPlayer> <last move time stamp> <this move time stamp> <isLast> <GhostCodeInfo>
			int capturePieceColor = Integer.parseInt(temp[2]);
			int MaxPlyNum = Integer.parseInt(temp[3]);
			Turn = Integer.parseInt(temp[4]);
			int nextPlayer = Integer.parseInt(temp[5]);
			// calculate this time stamp
			long lastTimeStamp = Long.parseLong(temp[6]);
			long thisTimeStamp = Long.parseLong(temp[7]);
			
			boolean isLast = (Integer.parseInt(temp[8]) == 1);
			
			lock.lock(); // lock steps, step_time, total_spent_time
			steps.add(temp[0]+","+temp[1]);
			eats.add("" + capturePieceColor);
			// minus the time
			step_time.add((int)(thisTimeStamp - lastTimeStamp));
			if(nextPlayer != PlayerNum){ // mine move
				Time -= (thisTimeStamp - lastTimeStamp);
				
				// oppo turn
				if(!isLast && !this.isCLI){
					((GST_Room)room).setWhichTurn(1);
				}
				
				total_spent_time.add((int)(((GST_SettingFile)Setting).timeLimit * SECTOMILLISEC) - Time);
				
				// error time divided by two in order to get single delay form server to client
				errorTime = (int)((thisTimeStamp - lastTimeStamp) - searchTime) / 2;
				if(!this.isCLI){
					((GST_Room)room).setGameInfo(Time, OppoTime, errorTime, Win, Draw, Lose);
				}
				
				if (Time < 0) Time = 0;
				
			}else{ // oppo
				OppoTime -= (thisTimeStamp - lastTimeStamp);
				
				total_spent_time.add((int)(((GST_SettingFile)Setting).timeLimit * SECTOMILLISEC) - OppoTime);
				
				// my turn
				if(!isLast && !this.isCLI){
					((GST_Room)room).setWhichTurn(0);
				}
				if(!this.isCLI){
					((GST_Room)room).setGameInfo(Time, OppoTime, errorTime, Win, Draw, Lose);
				}
				
				if (OppoTime < 0) OppoTime = 0;
			}
			lock.unlock();
			
			
			// send move to board and AI (AI mode)
			if(board != null){
				lock.lock(); // lock steps, step_time, total_spent_time
				Message.printInfo("PlayerNum: %d, NextPlayer: %d", PlayerNum, nextPlayer);
				((GST_Board)board).sendMove(temp[0] + " " + temp[1], 
						capturePieceColor,
						MaxPlyNum, 
						step_time.get(Turn - 1), 
						total_spent_time.get(Turn - 1),
						(nextPlayer != PlayerNum ? Time : OppoTime));
				lock.unlock();
				
				if( PlayerNum != nextPlayer ) {
					connect.sendMessage("OK", capturePieceColor, true);
				}
			}else {// send move to board(Human mode)
				lock.lock(); // lock steps, step_time, total_spent_time
				if(((GST_SettingFile)Setting).timeLimit == 0 &&
						((GST_SettingFile)Setting).onePlyTime == 0 &&
						((GST_SettingFile)Setting).plyShare == 0){
					connect.sendMove(p.getData(), 
							capturePieceColor,
							MaxPlyNum, 
							step_time.get(Turn - 1), 
							total_spent_time.get(Turn - 1), 
							0, 
							0, 
							isLast);
				}else{
					connect.sendMove(p.getData(), 
							capturePieceColor,
							MaxPlyNum, 
							step_time.get(Turn - 1), 
							total_spent_time.get(Turn - 1), 
							Time, 
							OppoTime, 
							isLast);
				}
				lock.unlock();
			}
			
			if(MidBoardFirstStep){
				((GST_ProcessRecord)processRecord).modifyTimeStamp(lastTimeStamp);
			}else if(Turn != 1) {
				((GST_ProcessRecord)processRecord).shiftTimeStamp(lastTimeStamp);
			}
			
			// set record temp file (local board record) Info
			((GST_ProcessRecord)processRecord).addData(p.getData(), Turn == 1 || MidBoardFirstStep);
			if(roomSetting.boardFileOutput == BoardFileOutput.YES){
				((GST_ProcessRecord)processRecord).writeTempFile(connect.getStdoutStamps(), connect.getStderrStamps(), connect.getConnectInfo(), connect.getErrorInfo());
			}
			MidBoardFirstStep = false;
			
			Message.printInfo("My: %d Oppo: %d\n", Time, OppoTime);
//			lock.lock(); // lock steps, step_time, total_spent_time
//			if(((GST_SettingFile)Setting).timeLimit == 0 &&
//					((GST_SettingFile)Setting).onePlyTime == 0 &&
//					((GST_SettingFile)Setting).plyShare == 0){
//				connect.sendMove(p.getData(), 
//						capturePieceColor,
//						MaxPlyNum, 
//						step_time.get(Turn - 1), 
//						total_spent_time.get(Turn - 1), 
//						0, 
//						0, 
//						isLast);
//			}else{
//				connect.sendMove(p.getData(), 
//						capturePieceColor,
//						MaxPlyNum, 
//						step_time.get(Turn - 1), 
//						total_spent_time.get(Turn - 1), 
//						Time, 
//						OppoTime, 
//						isLast);
//			}
//			lock.unlock();
			
			if(nextPlayer == PlayerNum && !isLast){
				Message.printInfo("Waiting for Player %d", PlayerNum);
				// record search time
				long startSearchTime = System.currentTimeMillis();
				// ask move
				AskMoveRunnable = new Runnable(){
					@Override
					public void run() {
						try{
							Packet tmp = connect.askMove(temp[9]);
							searchTime = System.currentTimeMillis() - startSearchTime;
							// send move
							if(!isFinishAGame){
								SendData(tmp);
							}
							// to get the human step
							if(connect instanceof Human && tmp.getAction() == GST_Action.DrawOffer){
								DOReadStepRunning.setTure();
								DOReadStepThread = new Thread(new Runnable(){
									@Override
									public void run() {
										try {
											Packet p = ((Human)connect).listenDrawOfferStep(DOReadStepRunning);
											if(p != null && DOReadStepRunning.getValue()){
												SendData(p);
											}
										} catch (Warning e) {
											Message.printError(ExceptionUtils.getStackTrace(e));
											if(!isFinishAGame){
												ErrorOccur(e);
											}
										}
									}
								});
								DOReadStepThread.start();
							}
						}catch(Warning w){  
							Message.printError(w.toString()); 
							if(!isFinishAGame){
								ErrorOccur(w); 
							}
						}
					}
				};
				if(!isFinishAGame){
					new Thread(AskMoveRunnable).start();
				}
			}
			break;
		case GST_Action.DrawOffer://求和動作
			// <kind> 0:disagree 1:agree X:ask
			if(connect instanceof Human){
				if(p.getData().equals("")){
					DOReadResponseRunning.setTure();
					DOReadResponseThread = new Thread(new Runnable(){
						@Override
						public void run() {
							try {
								Packet p = ((Human)connect).askDrawOffer(DOReadResponseRunning);
								if(p != null && DOReadResponseRunning.getValue()){
									SendData(p);
								}
							} catch (Warning e) {
								Message.printError(ExceptionUtils.getStackTrace(e));
								if(!isFinishAGame){
									ErrorOccur(e);
								}
							}
						}
					});
					DOReadResponseThread.start();
				}else if(p.getData().equals("0")){
					// stop the thread of reading step when waiting opponent response
					if(DOReadStepThread != null && DOReadStepThread.isAlive()){
						DOReadStepThread.interrupt();
						DOReadStepRunning.setFalse();
					}
					((Human)connect).sendCommandResult("drawOffer", 0);
					// ask move
					AskMoveRunnable = new Runnable(){
						@Override
						public void run() {
							try{
								Packet tmp = connect.askMove("drawOffer");
								// send move
								if(!isFinishAGame){
									SendData(tmp);
								}
								// to get the human step
								if(connect instanceof Human && tmp.getAction() == GST_Action.DrawOffer){
									DOReadStepRunning.setTure();
									DOReadStepThread = new Thread(new Runnable(){
										@Override
										public void run() {
											try {
												Packet p = ((Human)connect).listenDrawOfferStep(DOReadStepRunning);
												if(p != null && DOReadStepRunning.getValue()){
													SendData(p);
												}
											} catch (Warning e) {
												Message.printError(ExceptionUtils.getStackTrace(e));
												if(!isFinishAGame){
													ErrorOccur(e);
												}
											}
										}
									});
									DOReadStepThread.start();
								}
							}catch(Warning w){ 
								Message.printError(w.toString()); 
								if(!isFinishAGame){
									ErrorOccur(w); 
								}
							}
						}
					};
					if(!isFinishAGame){
						new Thread(AskMoveRunnable).start();
					}
				}else if(p.getData().equals("1")){
					// server will not send this, instead sending the game result
				}
			}
			break;
		case GST_Action.Abort://收到中斷
			// <AbortResult> 0: fail  1: success 
			if(connect instanceof Human){
				int result = Integer.parseInt(p.getData());
				// stop timer
				if(result == 1 && !this.isCLI){
					((GST_Room)room).stopTimer();
				}
				// send abort result
				((Human)connect).sendCommandResult("abort", result);
				// ask for undo info
				SendData(((Human)connect).askUndo());
			}
			break;
		case GST_Action.Undo://撤銷
			// <UndoResult> 0: fail  1: success
			if(connect instanceof Human){
				int result = Integer.parseInt(p.getData());
				// send undo result
				((Human)connect).sendCommandResult("undo", result);
				// check undo result
				if(result == 0){
					// ask for undo info
					SendData(((Human)connect).askUndo());
				}
			}
			break;
		case GST_Action.GameResult://遊戲結果
			// <gameCount> <judgeResult>  <FirstPlayerScore> <SecondPlayerScore> <gameResultGhostCodeInfo>
			//            <result><loser>
			
			needToCleanDebugInfo = true;
			// stop timer
			if(!this.isCLI){
				((GST_Room)room).stopTimer();
			}
			
			// stop search 
			Message.printInfo("Close Search.exe");
			isFinishAGame = true;
			
			stdout_stamp = connect.getStdoutStamps();
			stderr_stamp = connect.getStderrStamps();
			
			
			Rounds = Integer.parseInt(temp[0]);
			switch(temp[1]){
			case "PositionRepetitionDraw":
				// <RepeitionCount> <RepetitionInPly> <PlayerNum>
				GameResult = String.format("%s %s(%s-%s)", "Draw", temp[1], temp[2], temp[3]);
				Draw++;
				FirstScore = Double.parseDouble(temp[5]);
				SecondScore = Double.parseDouble(temp[6]);
				break;
			case "GamePlyLimitDraw":
				
				GameResult = String.format("%s %s", "Draw", temp[1]);
				Draw++;
				FirstScore = Double.parseDouble(temp[3]);
				SecondScore = Double.parseDouble(temp[4]);
				break;
			case "TimeOut":
			case "NoMove":
			case "NoBlueGhost":
			case "NoRedGhost":
			case "ChessEscape":
			case "SocketBroken":
			case "Resign":
				// <PlayerNum>
				GameResult = String.format("%s_wins %s", (Integer.parseInt(temp[2]) == PlayerNum ? opponentName : Setting.account), temp[1]);
				if(Integer.parseInt(temp[2]) == PlayerNum){
					Lose++;
					if(!(connect instanceof Human)) {
						connect.sendMessage(String.format("LST:%s", temp[5]), 0, false);
					}
				}else{
					Win++;
					if(!(connect instanceof Human)) {
						connect.sendMessage(String.format("WON:%s", temp[5]), 0, false);
					}
				}
				FirstScore = Double.parseDouble(temp[3]);
				SecondScore = Double.parseDouble(temp[4]);
				break;
			case "IllegalAction":
				// <pieceCode> <direction> <PlayerNum>
				GameResult = String.format("%s_wins %s(%s_%s)", (Integer.parseInt(temp[4]) == PlayerNum ? opponentName : Setting.account), temp[1], temp[2], temp[3]);
				if(Integer.parseInt(temp[4]) == PlayerNum){
					Lose++;
					if(!(connect instanceof Human)) {
						connect.sendMessage("NG", 0, false);
						connect.sendMessage(String.format("LST:%s", temp[7]), 0, false);
					}
				}else{
					Win++;
					if(!(connect instanceof Human)) {
						connect.sendMessage("NG", 0, false);
						connect.sendMessage(String.format("WON:%s", temp[7]), 0, false);
					}
				}
				FirstScore = Double.parseDouble(temp[5]);
				SecondScore = Double.parseDouble(temp[6]);
				break;
			//case "NoEatFlipDraw":
			case "DrawOffer":
				// <PlayerNum>
				GameResult = String.format("%s %s", "Draw", temp[1]);
				Draw++;
				FirstScore = Double.parseDouble(temp[3]);
				SecondScore = Double.parseDouble(temp[4]);
				break;
			case "Abort":
				// <PlayerNum>
				GameResult = String.format("%s %s", "Abort", temp[1]);
				Abort++;
				FirstScore = Double.parseDouble(temp[3]);
				SecondScore = Double.parseDouble(temp[4]);
				break;
			}
			
			if(!(connect instanceof Human)){
				// stop search
				connect.stopSearch();
				
				System.gc();
			}else{
				// stop thread
				DOReadStepRunning.setFalse();
				DOReadResponseRunning.setFalse();
				// set finish
				((Human)connect).setisFinish(new BooleanHolder(true));
			}
			
			// consecutive Abort times
			if(!temp[1].equals("Abort")){
				Abort = 0;
			}
			
			if(!this.isCLI){
				((GST_Room)room).setGameResult(Win, Draw, Lose);
			}
			
			// send result to board
			if(((GST_SettingFile)Setting).connectMode == GST_SettingFile.CONNECTMODE.HUMAN){
				((Human)connect).sendGameResult(GameResult);
			}else if(board != null){
				((GST_Board)board).sendResult(GameResult);
			}
			
			break;
		case GST_Action.GetBoardEndingRecord://獲取棋盤結束紀錄並重新遊戲
			// <board.txt>
			
			// delay
			showCountDownDialog();
			
			writeBoard(GameResult, FirstScore, SecondScore, p.getData(), stdout_stamp, stderr_stamp, connect.getConnectInfo(), connect.getErrorInfo());
			
			// delete temp file, if finish game in normal way
			if(p.getData().length() > 0){
				processRecord.deleteTempFile();
			}
			
			// wait for timer stop
			waitTimer();
			
			Message.printInfo("Restart Game");
			
			SendData(new Packet(GST_Action.GetBoardEndingRecord, "1"));
			break;
		case GST_Action.GameFinish://遊戲結束並log
			isFinishRounds = true;
			BoardRecordDirName = null;
			
			LastResult = new int[]{Win, Draw, Lose};
			// reset in contest mode
			Win = 0; // accumulative Win times
			Draw = 0; // accumulative Draw times
			Lose = 0; // accumulative Lose times
			Abort = 0; // consecutive Abort times
			Rounds = 0;
			
			// clean last debug info
			if(roomSetting.cleanLastDebug == Room_SettingFile.CleanLastDebug.YES){
				needToCleanDebugInfo = false;
				if(!this.isCLI){room.cleanDebugText();}
				
				// send board reset info
				if(((GST_SettingFile)Setting).connectMode == GST_SettingFile.CONNECTMODE.HUMAN){
					((Human) connect).restartBoard();
				}else if(board != null){
					((GST_Board)board).restartBoard();
				}
			}
			break;
		default:
			Message.printError("#ERROR: ANALYSIS ACTION");
			Message.printError("#%s", p.toString());
			
			//throw new Warning(Type.SERVER_ACTION_ERROR, "UnCaught Action");
		}
	}
	
	/**
	 * {@inheritDoc}
	 * 啟動 Search 程式。
	 */
	@Override
	protected void startSearch() throws Warning{
		Message.printInfo(">> start calling Search EXE");
		
		// change relative path to absolute path
		String AI_Path = ProgramPath.changePathToAbsolute(Setting.searchPath);
		Message.printInfo(">> Search EXE Path: %s", AI_Path);
		Message.printCLI("Path: %s", AI_Path);
		
		// connect to search
		switch(((GST_SettingFile)Setting).connectMode){
		case TCP:
			connect = new Ponder(AI_Path, Setting.arg, (GST_Room)room);
			break;
		case STDIO:
			connect = new MGTP(AI_Path, Setting.arg, (GST_Room)room); 
//			Search_Name = ((MGTP)connect).getSearchName();
//			Search_Version = ((MGTP)connect).getSearchVersion();
			break;
		case HUMAN:
			connect = new Human(
					roomSetting.boardOpenType, 
					Setting.startMode, 
					roomSetting.boardDisconnectAutoRestart, 
					(GST_Room)room, 
					this, 
					((GST_SettingFile)Setting).localServer, 
					isCLI, 
					isPlay,
					roomSetting.finishGameCloseHTML == Room_SettingFile.FinishGameCloseHTML.YES);
		}
		Message.printInfo(">> finish opening Search EXE");
	}
	
	/**
	 * {@inheritDoc}
	 * 開啟 Web 盤面。
	 */
	@Override
	protected void startBoard(boolean isMid, boolean auto) throws Warning{
		GST_SettingFile s = ((GST_SettingFile)Setting);
		
		if(!isMid){
			if(roomSetting.boardOpenType == Room_SettingFile.BoardOpenType.CLOSE ||
					s.connectMode == GST_SettingFile.CONNECTMODE.HUMAN){
				board = null;
			}else{
				board = new GST_Board(
						roomSetting.boardOpenType, 
						Setting.startMode, 
						roomSetting.boardDisconnectAutoRestart, 
						(GST_Room)room, 
						this, 
						this.isCLI, 
						this.isPlay,
						roomSetting.finishGameCloseHTML == Room_SettingFile.FinishGameCloseHTML.YES);
				board.start_and_wait_Connect();
			}
		}else if(isMid && !this.isCLI){
			// close previous board
			if(board != null){
				board.stopWebServer();
			}
			// connect to board
			Room_SettingFile.BoardOpenType type;
			if(auto){
				type = roomSetting.boardOpenType;
			}else{
				type = Room_SettingFile.BoardOpenType.BROWSER;
			}
			BoardWebSocketServer board_tmp = new GST_Board(
					type, 
					Setting.startMode, 
					roomSetting.boardDisconnectAutoRestart, 
					(GST_Room)room, 
					this, 
					this.isCLI, 
					this.isPlay, 
					roomSetting.finishGameCloseHTML == Room_SettingFile.FinishGameCloseHTML.YES);
			board_tmp.start_and_wait_Connect();
			
			// initial
			if(isMid){
				lock.lock();
				// send players name
				if(PlayerNum != -1){
					if(PlayerNum == 1){
						((GST_Board)board_tmp).sendPlayerName(Setting.account, opponentName);
					}else{
						((GST_Board)board_tmp).sendPlayerName(opponentName, Setting.account);
					}
				}
				// send initial board
				if(steps != null && step_time != null && total_spent_time != null){
					((GST_Board)board_tmp).sendInitBoard(initBoardData, steps.toArray(new String[0]), eats.toArray(new String[0]), step_time.toArray(new Integer[0]), total_spent_time.toArray(new Integer[0]), TakeTime, PlayerNum);
				}
				
				// if finish, send result
				if(isFinishAGame){
					((GST_Board)board_tmp).sendResult(GameResult);
				}
				// set board
				if(((GST_SettingFile)Setting).connectMode != GST_SettingFile.CONNECTMODE.HUMAN){
					board = board_tmp;
				}else{
					board = null;
					((Human)connect).resetBoard(((GST_Board)board_tmp));
				}
				// send setRed message if game didn't start in Human mode
				if(((GST_SettingFile)Setting).connectMode == GST_SettingFile.CONNECTMODE.HUMAN && setRedSuccess != 1) {
					Message.printInfo("Continue Set Red");
					((Human)connect).sendMessage("set", 0, false);
				}
				// if not finish, send generate move when using human mode
				else if(!isFinishAGame && ((GST_SettingFile)Setting).connectMode == GST_SettingFile.CONNECTMODE.HUMAN){
					Message.printInfo("Continue game");
					if(Turn + PlayerNum % 2 == 1){ // mine turn
						new Thread(AskMoveRunnable).start();
					}
				}
				lock.unlock();
			}
		}
	}
	
	/**
	 * {@inheritDoc}
	 * 開房方需要取得使用者設定的參數，傳送給伺服器開房。
	 * @return 使用者設定的遊戲參數
	 */
	@Override
	protected String genOpenRoomInfo() {
		String roomParam = String.format("%s,%s,%d,%d,%d,%d,%d,%d,%d,%d,%d",
				Setting.gametype, // gametype
				Setting.account, // account 
				(((GST_SettingFile)Setting).connectMode == GST_SettingFile.CONNECTMODE.HUMAN ? 1 : 0),
				(((GST_SettingFile)Setting).midBoard == MIDBOARD.NO ? 0 : (((GST_SettingFile)Setting).midBoard == MIDBOARD.CUSTOM) ? 1 : 2), // NO: 0 CUSTOM: 1 YES: 2
				Setting.repeatTime, // rounds 
				(((GST_SettingFile)Setting).firstMove == FIRSTMOVE.YES ? 1 : 2), // first move 
				(((GST_SettingFile)Setting).changeFirstMove == CHANGEFIRSTMOVE.YES ? 1 : 0), // canlook 
				((GST_SettingFile)Setting).timeLimit, // setSecond
				((GST_SettingFile)Setting).onePlyTime, // one ply time
				((GST_SettingFile)Setting).plyShare, // ply share 
				((GST_SettingFile)Setting).MaxPly// Max ply to draw
				);
		return roomParam;
	}
	
	/**
	 * 輸出伺服器傳送過來的 log 檔案成 txt 檔案。
	 * @param result 遊戲結果
	 * @param firstscore 先手分數
	 * @param secondscore 後手分數
	 * @param board 伺服器傳送過來的 log 檔案
	 * @param stdout_stamp 標準輸出的郵戳(用來指示走步大概的範圍)
	 * @param stderr_stamp 標準錯誤輸出的郵戳(用來指示走步大概的範圍)
	 * @param stdout 標準輸出字串
	 * @param stderr 標準錯誤字串
	 * @throws Warning 發生錯誤。
	 */
	private void writeBoard(String result, double firstscore, double secondscore, String board, String stdout_stamp, String stderr_stamp, String stdout, String stderr) throws Warning{
	    // check board length, which need to large than zero
		if(board.length() <= 0){
			// game did not start yet, so do not to store the board.txt
			return;
		}
		// change '/r/n' to correspond new line of system 
	    board = board.replace(String.format("%c", '\n'), System.getProperty("line.separator"));
	    
	    String AllRecord = "";
		try {
			// output board stdout stderr
			AllRecord += board + 
					System.getProperty("line.separator") + 
					stdout_stamp + 
					System.getProperty("line.separator") + 
					stdout + 
					System.getProperty("line.separator") + 
					stderr_stamp + 
					System.getProperty("line.separator") + 
					stderr + 
					System.getProperty("line.separator") + 
					"###" + 
					System.getProperty("line.separator") +
					getSearchInfo() +
					"###" +
					System.getProperty("line.separator");
			
			
		    // output board
			String boardFilePath_Name = null;
			String info = null;
		    if(Abort > 0){
		    	boardFilePath_Name = String.format("%s%s%d_%d %s", BoardRecordDirName, File.separator, Rounds, Abort, result);
		    	info = String.format("%d_%d %s %s %f / %f %s %s%s", Rounds, Abort, 
		    			board.split(System.getProperty("line.separator"))[2].substring(2), 
		    			result, 
		    			firstscore, secondscore,
		    			board.split(System.getProperty("line.separator"))[1].substring(2), 
		    			board.split(System.getProperty("line.separator"))[0],
		    			System.getProperty("line.separator"));
		    }else{
		    	boardFilePath_Name = String.format("%s%s%d %s", BoardRecordDirName, File.separator, Rounds, result);
		    	info = String.format("%d %s %s %f / %f %s %s%s", Rounds, 
		    			board.split(System.getProperty("line.separator"))[2].substring(2), 
		    			result, 
		    			firstscore, secondscore,
		    			board.split(System.getProperty("line.separator"))[1].substring(2), 
		    			board.split(System.getProperty("line.separator"))[0],
		    			System.getProperty("line.separator"));
		    }
		    Message.printInfo("BoardFilePath: %s", boardFilePath_Name + ".txt");
		    Message.printInfo("info: %s", info);
		    Message.printInfo("BoardFileOutput: %s", roomSetting.boardFileOutput.name());
		    if(roomSetting.boardFileOutput == BoardFileOutput.YES){
		    	Files.write(Paths.get(boardFilePath_Name + ".txt"), AllRecord.getBytes(), StandardOpenOption.CREATE);
		    }
		    
		    // output game info
		    Message.printInfo("RoomStatisticOutput: %s", roomSetting.roomStatisticOutput.name());
		    if(roomSetting.roomStatisticOutput == RoomStatisticOutput.YES){
		    	File dir = new File(BoardRecordDirName);
				String[] dir_children = dir.list();
				boolean isFound = false;
				if (dir_children != null) {
				    for(int i = 0; i < dir_children.length; i++){
				    	if(dir_children[i].startsWith("Game") && dir_children[i].endsWith(".info")){
				    		isFound = true;
				    		// get target file
				    		File gameInfoFile = new File(String.format("%s%s%s", BoardRecordDirName, File.separator, dir_children[i]));
				    		Files.write(gameInfoFile.toPath(), info.getBytes(), StandardOpenOption.APPEND);
					    	// rename the game info file
						    Files.move(gameInfoFile.toPath(), new File(String.format("%s%sGame_%dW%dD%dL.info", BoardRecordDirName, File.separator, Win, Draw, Lose)).toPath());
						    // delete the old game info file
						    if(gameInfoFile.exists()){
						    	Files.delete(gameInfoFile.toPath());
						    }
						    Message.printInfo("GameInfoFile: %s", gameInfoFile.getAbsolutePath());
						    Message.printInfo("New_GameInfoFile: %s", new File(String.format("%s%sGame_%dW%dD%dL.info", BoardRecordDirName, File.separator, Win, Draw, Lose)).toPath().toString());
						    break;
				    	}
				    }
				    if(!isFound){
				    	// not found (create a new one)
					    Files.write(Paths.get(String.format("%s%sGame_%dW%dD%dL.info", BoardRecordDirName, File.separator, Win, Draw, Lose)), info.getBytes(), StandardOpenOption.CREATE);
				    }
				}
		    }
			
			// output command file
		    Message.printInfo("CommandLogOutput: %s", roomSetting.commandLogOutput.name());
		    if(roomSetting.commandLogOutput == CommandLogOutput.YES){
		    	String search_command = connect.getInteractiveLog();
				if(search_command != ""){
					Files.write(Paths.get(boardFilePath_Name + ".command"), 
							search_command.getBytes(), StandardOpenOption.CREATE);
				}
		    }
		    
		}catch (IOException e) {
			Message.printError("#ERROR: RECORD GAME BOARD RESULT");
			Message.printError("#BoardDirName:%s", BoardRecordDirName);
			Message.printError("#Board:%s", AllRecord);
			Message.printError(ExceptionUtils.getStackTrace(e));
			
			throw new Warning(Type.BOARD_FORMAT_ERROR, "RECORD GAME BOARD RESULT ERROR");
		}
	}
	
	private String readMidBoard(String filename) throws Exception{
		StringBuilder board = new StringBuilder();
		BufferedReader BR = new BufferedReader(new FileReader(Paths.get(filename).toFile()));
		
		String[] remain;

		// ignore result and time
		while(!BR.readLine().startsWith("* ")){}
		
		// ignore players
		BR.readLine();
		
		// store remain chess
		String remain_chess = BR.readLine();
		
		// set initial board
		final char kind[] = { '-', 'R', 'B', 'X', 'r', 'b' };
		for(int i = 0; i < 6; i++){
			remain = BR.readLine().split(" ");
			for(int j = 1; j < 7; j++){
				boolean found = false;
				for(int k = 0; k < kind.length; k++){
					if(kind[k] == remain[j].charAt(0)){
						board.append(k);
						found = true;
						break;
					}
				}
				if(found == false){
					BR.close();
					throw new Exception("BOARD CHESS CODE ERROR");
				}
				board.append(',');
			}
		}
		
		// set remain chess
		remain = remain_chess.split(" ");
		for(int i = 1; i < 5; i++){
			board.append(Integer.parseInt(remain[i]));
			board.append(',');
		}
//		final char kind[] = {'-', 'K', 'G', 'M', 'R', 'N', 'C', 'P', 'X', 'k', 'g', 'm', 'r', 'n', 'c', 'p'};
//		for(int i = 0; i < 8; i++){
//			remain = BR.readLine().split(" ");
//			for(int j = 1; j < 5; j++){
//				boolean found = false;
//				for(int k = 0; k < kind.length; k++){
//					if(kind[k] == remain[j].charAt(0)){
//						board.append(k);
//						found = true;
//						break;
//					}
//				}
//				if(found == false){
//					BR.close();
//					throw new Exception("BOARD CHESS CODE ERROR");
//				}
//				board.append(',');
//			}
//		}
//		
//		// set remain chess
//		remain = remain_chess.split(" ");
//		for(int i = 1; i < 15; i++){
//			board.append(Integer.parseInt(remain[i]));
//			board.append(',');
//		}

		
		// set gameStarted = 1
		board.append("1");
		board.append(',');
		
		// ignore time
		BR.readLine();
		
		// read steps
		// \\d+ : any length of number
		// \\. : '.'
		ArrayList<String> steps = new ArrayList<String>();
		while((remain = BR.readLine().split(" "))[1].matches("\\d+\\.")){
			for(int i = 2; i < remain.length; i++){
				steps.add(remain[i].replace(',', ' '));
			}
		}
		if(!remain[1].equals("Comment")){
			if(BR.readLine().split(" ")[1].equals("Comment")){
				BR.close();
				throw new Exception("THIS GAME HAD BEEN FINISHED");
			}else{
				BR.close();
				throw new Exception("FORMAT ERROR");
			}
		}
		// set steps length
		board.append(steps.size());
		board.append(',');
		// set steps
		for(String s : steps){
			board.append(s);
			board.append(',');
		}
		
		// read times
		// \\d+ : any length of number
		// \\. : '.'
		ArrayList<Long> times = new ArrayList<Long>();
		String tmp;
		while((tmp = BR.readLine()) != null){
			remain = tmp.split(" ");
			if(!remain[0].matches("\\d+\\.")) break;
			times.add(Long.parseLong(remain[1]));
			if(times.size() == steps.size() + 1) break;
		}
		Message.printInfo("times:%d steps:%d", times.size(), steps.size());
		if(times.size() != steps.size() + 1){
			BR.close();
			throw new Exception("NUMBER OF STEPS AND TIMES ARE ILLEGAL");
		}
		
		// read eats
		// \\d+ : any length of number
		// \\. : '.'
		ArrayList<Integer> eats = new ArrayList<Integer>();
		while((tmp = BR.readLine()) != null){
			remain = tmp.split(" ");
			if(!remain[0].matches("\\d+\\.")) break;
			eats.add(Integer.parseInt(remain[1]));
		}
		if(eats.size() != steps.size()){
			BR.close();
			throw new Exception("NUMBER OF STEPS AND EATS ARE ILLEGAL");
		}
		
		// set eats length
		board.append(eats.size());
		board.append(',');
		// set eats
		for(Integer e : eats) {
			board.append(e);
			board.append(',');
		}
		
		// set times length
		board.append(times.size());
		board.append(',');
		// set times
		for(Long t : times){
			board.append(t);
			board.append(',');
		}
		
		BR.close();
		
		return board.toString();
	}
	
	/**
	 * {@inheritDoc}
	 * 開房方依照使用者設定，取得初始盤面。
	 * @return 初始盤面。
	 */
	@Override
	protected String getInitBoard() throws Warning{
		StringBuilder board = new StringBuilder();
		
		try{
			if(Setting.roomType != ROOMTYPE.OPENING){
				if(((GST_SettingFile)Setting).midBoard == GST_SettingFile.MIDBOARD.NO){
					board.append("INITIAL,");
					// initial board
					for(int i = 0; i < 36; i++){
						if((i>=1&& i<=4)||(i>=7&&i<=10)|| (i>=31&& i<=34)||(i>=25&&i<=28))
							board.append('3'); // 9(X)
						else
							board.append('0');
						board.append(',');
					}
					
					final int[] Ghosts = new int[]{4, 4, 4, 4};   // 棋子數量
					// initial remain piece
					for(int i = 0; i < 4; i++){
						board.append(Ghosts[i]); // initial number
						board.append(',');
					}
					
					// initial gameStarted
					board.append(0);
					board.append(',');
					
					// initial step list length
					board.append(0);
					board.append(',');
					
					// initial step list
					// NO
					
					// initial eat list length
					board.append(0);
					board.append(',');
					
					// initial eat list
					// NO
					
					// initial time list length
					board.append(0);
					board.append(',');
					
					// initial time list
					// NO
					
					//After initial board="INITIAL,0,3,3,3,3,0,0,3,3,3,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,3,3,3,0,0,3,3,3,3,0,4,4,4,4,0,0,0,0,"
					//split "," length=1+36+4+1+1+1+1=45 , index: 0-44
					
					
				}else if(((GST_SettingFile)Setting).midBoard == GST_SettingFile.MIDBOARD.CUSTOM){
					String board_path = ((GST_SettingFile)Setting).boardPath;
					board.append(Paths.get(board_path).getFileName().toString());
					board.append(",");
					board.append(readMidBoard(board_path));
				}else if(((GST_SettingFile)Setting).midBoard == GST_SettingFile.MIDBOARD.YES){
					// if use disconnected midboard, then pass empty string to server
					return "";
				}
			}
//			else{//開局庫模式
//				String Board_Path = ((GST_SettingFile)Setting).boardPath;
//				if(Board_Path.startsWith(".")){
//					Board_Path = Paths.get(ProgramPath.getProgramPath(), Board_Path).toString();
//				}
//				File opening_folder = new File(Board_Path);
//				
//				// sort files
//				List<File> board_files = Arrays.asList(opening_folder.listFiles());
//				NameFileComparator NC = new NameFileComparator();
//				board_files = NC.sort(board_files);
//				Message.printInfo("File Order: %s", String.join(", ", board_files.stream().map((f) -> f.getName()).toArray(String[]::new)));
//				
//				if(((GST_SettingFile)Setting).setRedMode == GST_SettingFile.SETREDMODE.CUSTOM){
//					for (final File fileEntry : board_files) {
//				        if (fileEntry.isFile() && fileEntry.getName().endsWith(".txt")) {
//				        	board.append(fileEntry.getName());
//				        	board.append(",");
//				            board.append(readMidBoard(fileEntry.getAbsolutePath()));
//				        }
//				    }
//				}else if(((GST_SettingFile)Setting).setRedMode == GST_SettingFile.SETREDMODE.RANDOM){
//					List<String> list = new ArrayList<String>();
//					for (final File fileEntry : board_files) {
//				        if (fileEntry.isFile() && fileEntry.getName().endsWith(".txt")) {
//				        	list.add(fileEntry.getAbsolutePath());
//				        }
//				    }
//					Collections.shuffle(list);
//					
//					int random_size = (int)Math.round(Math.ceil((double)Setting.repeatTime / Setting.timePerOpening));
//					random_size = Math.min(list.size(), random_size);
//					
//					for (int i = 0; i < random_size; ++i){
//						board.append(Paths.get(list.get(i)).getFileName().toString());
//						board.append(",");
//						board.append(readMidBoard(list.get(i)));
//					}
//				}
//			}
		
		}catch(Exception e){
			Message.printError(ExceptionUtils.getStackTrace(e));
			Message.printError(e.getMessage());
			Message.printError("#CUSTOM BOARD PATH: %s", ((GST_SettingFile)Setting).boardPath);
			
			throw new Warning(Type.BOARD_FORMAT_ERROR, e.getMessage());
		}
		
		// check board length
		if(board.toString().length() <= 0){
			throw new Warning(Type.BOARD_FILE_EMPTY, "The board file is empty");
		}
		
		return board.substring(0, board.length() - 1).toString();
	}
	
	/**
	 * 取得加房參數。
	 */
	protected String getEnterRoomParameter() {
		return String.format("%s", "");
	}
	
	/**
	 * {@inheritDoc}
	 * 加房方將伺服器傳送過來的房間遊戲資訊記錄到 Setting 檔案中。
	 */
	@Override
	protected void setEnterRoomInfo(List<String> roomInfo, boolean isTransform) {
		// midboard
		if(isTransform){
			((GST_SettingFile)Setting).midBoard = GST_SettingFile.MIDBOARD.valueOf(roomInfo.get(4));
		}else{
			switch(Integer.parseInt(roomInfo.get(4))){
			case 0:
				((GST_SettingFile)Setting).midBoard = GST_SettingFile.MIDBOARD.NO;
				break;
			case 1:
				((GST_SettingFile)Setting).midBoard = GST_SettingFile.MIDBOARD.CUSTOM;
				break;
			case 2:
				((GST_SettingFile)Setting).midBoard = GST_SettingFile.MIDBOARD.YES;
				break;
			}
		}
		// repeat time
		Setting.repeatTime = Integer.parseInt(roomInfo.get(5));
		// first move
		if(isTransform){
			((GST_SettingFile)Setting).firstMove = GST_SettingFile.FIRSTMOVE.valueOf(roomInfo.get(6));
		}else{
			switch(Integer.parseInt(roomInfo.get(6))){
			case 1:
				((GST_SettingFile)Setting).firstMove = GST_SettingFile.FIRSTMOVE.YES;
				break;
			case 2:
				((GST_SettingFile)Setting).firstMove = GST_SettingFile.FIRSTMOVE.NO;
				break;
			}
		}
		// change first
		if(isTransform){
			((GST_SettingFile)Setting).changeFirstMove = GST_SettingFile.CHANGEFIRSTMOVE.valueOf(roomInfo.get(7));
		}else{
			switch(Integer.parseInt(roomInfo.get(7))){
			case 0:
				((GST_SettingFile)Setting).changeFirstMove = GST_SettingFile.CHANGEFIRSTMOVE.NO;
				break;
			case 1:
				((GST_SettingFile)Setting).changeFirstMove = GST_SettingFile.CHANGEFIRSTMOVE.YES;
				break;
			}
		}
		// time setting
		((GST_SettingFile)Setting).timeLimit = Integer.parseInt(roomInfo.get(8));
		// ply time
		((GST_SettingFile)Setting).onePlyTime = Integer.parseInt(roomInfo.get(9));
		// group ply
		((GST_SettingFile)Setting).plyShare = Integer.parseInt(roomInfo.get(10));
//		// longcatch
//		((GST_SettingFile)Setting).longCatch = Integer.parseInt(roomInfo.get(11));
//		// noeatflip
//		((GST_SettingFile)Setting).noEatFlip = Integer.parseInt(roomInfo.get(12));
	}
	
	/**
	 * {@inheritDoc}
	 * 回傳加房方的玩家類型。
	 * @return 加房方的玩家類型
	 */
	@Override
	protected int getPlayerKind() {
		return (((GST_SettingFile)Setting).connectMode == GST_SettingFile.CONNECTMODE.HUMAN ? 1 : 0);
	}
	
	/**
	 * {@inheritDoc}
	 * 關閉整個遊戲。
	 */
	@Override
	protected void closeGame(){
		// stop thread
		DOReadStepRunning.setFalse();
		DOReadResponseRunning.setFalse();
		
		if(!this.isCLI){((GST_Room)room).stopTimer();}
	}
	
	/**
	 * {@inheritDoc}
	 * 關閉 Search 程式。
	 */
	@Override
	protected void stopSearch(){
		if(connect != null){
			connect.stopSearch();
		}
	}
	
	/**
	 * {@inheritDoc}
	 * 取得遊戲動作指令的名稱，方便 Debug。
	 * @return 遊戲動作指令的名稱
	 */
	@Override
	protected String[] getActionName() {
		return GST_Action.GST_ActionName;
	}
	
	/**
	 * {@inheritDoc}
	 * 讓使用者選擇房間，依照是否是 CLI，做出相對應的動作。
	 * @param isCLI 是否是 CLI mode。
	 * @return 使用者選重的房間參數，用 List 型態包裝的各項欄位參數。
	 */
	@Override
	protected List<String> chooseRoomList(boolean isCLI, boolean isPlay, Integer specific_roomID) throws Warning {
		List<String> roomInfo = null;
		if(!this.isCLI){
			RoomList roomlist = new RoomList(room, Setting.gametype, new GST_GetRoomList(server, Setting.account));
			roomlist.setLocationRelativeTo((room.getState() == Frame.ICONIFIED ? null : room));
			roomlist.setModalityType(ModalityType.APPLICATION_MODAL); // to focus this window
			roomInfo = roomlist.getChooseRowInfo();

			return roomInfo;
		}else if(!this.isPlay){
			Scanner scanner = new Scanner(System.in);
			try{
				ArrayList<String> list;
				GetRoomList getRoomList = new GST_GetRoomList(server, Setting.account);
				
				do{
					// Get Room Count
					do{
						list = getRoomList.get(Setting.gametype, Setting.account);
						// re get room count
						if(list == null || list.size() <= 0){
							Message.printCLI("No available Room. Retry...");
							// delay
							try {
								TimeUnit.MILLISECONDS.sleep(2500);
							} catch (InterruptedException e) {}
						}
					}while(list == null || list.size() <= 0);
					
					// Get Room All List
					ArrayList<String> edit_list = new ArrayList<>();
					while(list.size() > 0){
						String[] data = list.remove(0).split(",");
						data = getRoomList.changeShowType(data);
						edit_list.add(String.join(",", data));
					}
					
					// calculate column max length 
					String[] column_names = getRoomList.getColumnNames();
					int[] max_length = new int[column_names.length];
					for(int i = 0; i < column_names.length; i++){
						int max = column_names[i].length();
						for(int j = 0; j < edit_list.size(); j++){
							max = Math.max(edit_list.get(j).split(",")[i].length(), max);
						}
						max_length[i] = max + 2;
					}
					
					// print all list
					// header
					Message.printCLI("");
					Message.printxCLI("| Index |");
					for(int i = 0; i < column_names.length; i++){
						Message.printxCLI(StringUtils.center(column_names[i], max_length[i]));
						Message.printxCLI("|");
					}
					Message.printCLI("");
					// row
					for(int i = 0; i < edit_list.size(); i++){
						Message.printxCLI("|%s|", StringUtils.center(String.valueOf(i + 1), 7));
						for(int j = 0; j < column_names.length; j++){
							Message.printxCLI(StringUtils.center(edit_list.get(i).split(",")[j], max_length[j]));
							Message.printxCLI("|");
						}
						Message.printCLI("");
					}
					Message.printCLI("");
					
					// get input
					do{
						int index;
						try{
							Message.printxCLI("Room Index (%d ~ %d) [-1: reload]: ", 1, edit_list.size());
							String input = scanner.nextLine();
							index = Integer.parseInt(input);
						}catch(NumberFormatException e){
							continue;
						}

						if(index <= edit_list.size() && index >= 1){
							List<String> select_room = Arrays.asList(edit_list.get(index - 1).split(","));
							Pair<Boolean, String> pair = getRoomList.checkLegel(select_room);
							if(pair.a){
								return select_room;
							}else{
								Message.printCLI("ERROR: %s", pair.b);
							}
						}else if(index == -1){
							break;
						}
					}while(true);
				}while(true);
			}catch(Warning w){
				Message.printCLI(w.getMessage());
				throw w;
			}
		}else{
			GetRoomList getRoomList = new GST_GetRoomList(server, Setting.account);
			// Get Room Count
			ArrayList<String> list;
			do{
				list = getRoomList.get(Setting.gametype, Setting.account);
				// re get room count
				if(list == null || list.size() <= 0){
					// delay
					try {
						TimeUnit.MILLISECONDS.sleep(1000);
					} catch (InterruptedException e) {}
				}
			}while(list == null || list.size() <= 0);

			for(int i = 0; i < list.size(); i++){
				String[] data = list.get(i).split(",");
				if(data[0].equals(String.valueOf(specific_roomID))){
					data = getRoomList.changeShowType(data);
					return Arrays.asList(data);
				}else{
					this.close(null);
				}
			}
			return null;
		}
	}
	
	/**
	 * {@inheritDoc}
	 * 取得此遊戲的比賽頁面顯示 Dialog。
	 * @param room 用來置中在 room 的參數。
	 * @return Contest 回傳比賽頁面顯示 Dialog。
	 */
	@Override
	protected Contest getContestDialog(Room room, Room_SettingFile roomSetting) {
		return new GST_Contest((GST_SettingFile)Setting, room, roomSetting);
	}
	
	/**
	 * {@inheritDoc}
	 * 取得此遊戲比賽結束後的顯示訊息，可能可以給比賽結果或分數統計。
	 * @return 遊戲比賽結束後的顯示訊息。
	 */
	@Override
	protected String getFinishInfo() {
		if(LastResult != null){
			return String.format("%dW / %dD / %dL", LastResult[0], LastResult[1], LastResult[2]);
		}else{
			return "0W / 0D / 0L";
		}
	}
	
	/**
	 * {@inheritDoc}
	 * 取得 Search 的相關資訊。通常狀況下只有使用 STDIO 的模式才能顯示程式名稱和版本。
	 * @return Search 的相關資訊字串。
	 */
	@Override
	protected String getSearchInfo(){
		GST_SettingFile tmp_s = (GST_SettingFile)Setting;
		
		StringBuilder sb = new StringBuilder();
		sb.append(String.format("Account: %s%n", tmp_s.account));
		sb.append(String.format("Mode: %s%n", tmp_s.connectMode.toString()));
		sb.append(String.format("Path: %s%n", (tmp_s.connectMode == GST_SettingFile.CONNECTMODE.HUMAN ? "" : tmp_s.searchPath)));
		sb.append(String.format("Arg: %s%n", tmp_s.arg));
		sb.append(String.format("Name: %s%n", Search_Name));
		sb.append(String.format("Version: %s%n", Search_Version));
		
		return sb.toString();
	}
	
	/**
	 * {@inheritDoc}
	 * 取得和 Search 的所有互動字串，提供使用者 Debug。
	 * @return 取得和 Search 的所有互動字串。
	 */
	@Override
	protected String getSearchInteractiveLog(){
		return connect.getInteractiveLog();
	}
}