package gst;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.exception.ExceptionUtils;

import client.GameRecordList;
import client.Message;

/**
 * 
 * 顯示暗棋遊戲紀錄的頁面，可以顯示各房間內的每場遊戲結果和資訊，也可以統計這房間內的遊戲結果。
 * 
 * @author ldslds449
 * @see client.GameRecordList
 * 
 */
public class GST_GameRecordList extends GameRecordList {//顯示遊戲紀錄的類別

	private String player1Name = null;
	private String player2Name = null;
	
	public class Round{
		String index;
		String player;
		String result;
		String type;
		String score;
		String time;
		String init_board;
	}
	
	/**
	 * 
	 * GST_GameRecordList 的 constructor。
	 * 
	 * @param recordFileName 紀錄檔案路徑，用來初始父類別的 constructor。
	 * 
	 */
	public GST_GameRecordList(String recordFileName) {
		super(recordFileName);
		this.setTitle("Room Statistics");
	}
	
	/**
	 * 
	 * {@inheritDoc}
	 * 父類別用來取得欄位名稱的 method。
	 * 請注意：順序是有差別的，顯示會依照此順序由左至右顯示。
	 * 
	 * @return 欄位名稱的陣列。
	 * 
	 */
	@Override
	protected String[] getColumnNames(){
		return new String[]{"Game", "Players", "Result", "Result Type", "Score", "Time", "Init Board"};
	}
	
	/**
	 * 
	 * {@inheritDoc}
	 * 設置 PieChart 分為幾類，需呼叫 addPieSeries 來增加類別。
	 * 
	 */
	@Override
	protected void setChartSeries(){
		// pie chart
		addPieSeries(player1Name);
		addPieSeries(player2Name);
		addPieSeries("Draw");
		// category chart
		addCategorySeries(player1Name);
		addCategorySeries(player2Name);
	}
	
	/**
	 * 
	 * {@inheritDoc}
	 * 讀取房間記錄檔 (檔名存在 {@link client.GameRecordList#recordFileName})，處理檔案之後，回傳每列資料各欄位需顯示的字串。
	 * 例如：List 內儲存各列資料，也就是在 table 中的 row。String[] 內儲存一個列裡面各欄位的資料。
	 * 請注意：List 和 String[] 內資料順序皆有順序性。
	 * 
	 * @return 每列資料各欄位需顯示的字串
	 */
	@Override
	protected List<String[]> readRecordFile(){
		List<Round> Records = new ArrayList<>();
		
		BufferedReader br = null;
		try {
			br = new BufferedReader(new FileReader(recordFileName));
			
			// read round
			String line;
			
			while((line = br.readLine()) != null){
				String[] data = line.split(" ");
				Round round = new Round();
				round.index = data[0].replace("_", "-");
				round.player = String.format("%s %s %s", data[1], data[2], data[3]);
				round.result = data[4];
				round.type = data[5];
				round.score = String.format("%s %s %s", data[6], data[7], data[8]);
				round.time = String.format("%s %s %s %s", data[9], data[10], data[11], data[12]);
				if(data.length > 13){
					round.init_board = data[13];
				}else{
					round.init_board = "";
				}
				
				Records.add(round);
			}
			
		}  catch (IOException e) {
			
			Message.printError(ExceptionUtils.getStackTrace(e));
		} finally {
			try {
				if(br != null){
					br.close();
				}
			} catch (IOException e) {
				Message.printError(ExceptionUtils.getStackTrace(e));
			}
		}
		
		List<String[]> lists = new ArrayList<>();
		
		// change to String[]
		for(int i = 0; i < Records.size(); i++){
			try{
				Round r = Records.get(i);
				String[] row_record = new String[7];
				
				row_record[0] = r.index;
				row_record[1] = r.player;
				row_record[2] = r.result;
				row_record[3] = r.type;
				row_record[4] = r.score;
				row_record[5] = r.time;
				row_record[6] = r.init_board;
				
				lists.add(row_record);
				
				// set player Names
				if(player1Name == null && player2Name == null){
					player1Name = r.player.split(" ")[0];
					player2Name = r.player.split(" ")[2];
				}
			}catch(Exception e){
				Message.printInfo("%d %s %s %s %s %s %s%s", 
						Records.get(i).index, 
						Records.get(i).player, 
						Records.get(i).result, 
						Records.get(i).type, 
						Records.get(i).score,
						Records.get(i).time, 
						Records.get(i).init_board, 
						System.getProperty("line.separator"));
			}
		}
		
		return lists;
	}
	
	/**
	 * 
	 * {@inheritDoc}
	 * 讀取房間記錄檔 (檔名存在 {@link client.GameRecordList#recordFileName})，讀取遊戲結果，設置 PieChart 數值。
	 * 請注意：設置 PieChart 數值請用 {@link client.GameRecordList#updatePieValue}。
	 * 。
	 */
	@Override
	protected void setChartValue(){
		int player1Win = 0;
		int player2Win = 0;
		int draw = 0;
		
		double player1Score = 0;
		double player2Score = 0;
		
		BufferedReader br = null;
		try {
			br = new BufferedReader(new FileReader(recordFileName));
			
			// read round
			String line;
			
			while((line = br.readLine()) != null){
				String[] data = line.split(" ");
				String result = data[4].replace("_wins", "");
				
				// result
				if(result.equals("Draw")){
					draw++;
				}else if(result.equals(player1Name)){
					player1Win++;
				}else if(result.equals(player2Name)){
					player2Win++;
				}
				
				Message.printInfo("1: %s", data[6]);
				Message.printInfo("2: %s", data[8]);
				
				// score
				if(data[1].equals(player1Name)){
					player1Score += new BigDecimal(data[6]).doubleValue();
					player2Score += new BigDecimal(data[8]).doubleValue();
				}else{
					player1Score += new BigDecimal(data[8]).doubleValue();
					player2Score += new BigDecimal(data[6]).doubleValue();
				}
			}
			
		}  catch (IOException e) {
			
			Message.printError(ExceptionUtils.getStackTrace(e));
		} finally {
			try {
				if(br != null){
					br.close();
				}
			} catch (IOException e) {
				Message.printError(ExceptionUtils.getStackTrace(e));
			}
		}
		
		// pie chart
		updatePieValue(player1Name, player1Win);
		updatePieValue(player2Name, player2Win);
		updatePieValue("Draw", draw);
		
		Message.printInfo("Player1Score: %.10f", player1Score);
		Message.printInfo("Player2Score: %.10f", player2Score);
		
		// category chart
		updateCategoryValue(player1Name, player1Score);
		updateCategoryValue(player2Name, player2Score);
	}
}
