package gst;

import java.util.ArrayList;
import java.util.List;

import client.GetRoomList;
import client.Message;
import client.Pair;
import client.Server;

/**
 * 
 * 繼承至 {@link client.GetRoomList}，負責暗棋房間列表顯示的功能。
 * 
 * @author ldslds449
 * @see client.GetRoomList
 */
public class GST_GetRoomList extends GetRoomList {
	
	/**
	 * 儲存我方玩家帳號名稱，因為 Server 不能讓同帳號玩家出現在同一房間內，故要做防範。
	 */
	private String MyName;
	
	/**
	 * 
	 * constructor。
	 * 
	 * @param server 與 Server 的 Socket 連線。
	 * @param account 我方玩家帳號名稱。
	 */
	public GST_GetRoomList(Server server, String account) {
		super(server);
		MyName = account;
	}
	
	/**
	 * {@inheritDoc}
	 * 將伺服器發送過來的房間列表參數做調整，因為伺服器會用代號來代表類別，例如：1 代表 YES。為了讓 User 更容易看懂，要將顯示的字串做更換。
	 * 
	 * @param array 一個房間的各項欄位參數。
	 * @return 更改過後的房間的各項欄位參數
	 */
	@Override
	public String[] changeShowType(String[] array){
		String[] array_clone = new String[array.length];
		// copy array
		for(int i = 0; i < array.length; i++){
			array_clone[i] = array[i];
		}
		
		array_clone[3] = (array[3].equals("0") ? "NO" : "YES");
		
		if(array[4].equals("0")){
			array_clone[4] = GST_SettingFile.MIDBOARD.NO.name();
		}else if(array[4].equals("2")){
			array_clone[4] = GST_SettingFile.MIDBOARD.YES.name();
		}else{
			array_clone[4] = GST_SettingFile.MIDBOARD.CUSTOM.name();
		}
		
		array_clone[6] = (array[6].equals("1") ? GST_SettingFile.FIRSTMOVE.YES.name() : GST_SettingFile.FIRSTMOVE.NO.name());
		
		array_clone[7] = (array[7].equals("0") ? GST_SettingFile.CHANGEFIRSTMOVE.NO.name() : GST_SettingFile.CHANGEFIRSTMOVE.YES.name());
		
		return array_clone;
	}
	
	/**
	 * {@inheritDoc}
	 * 為了讓 Client 取得房間列表的各欄位顯示名稱。
	 * 
	 * @return 房間列表的各欄位顯示名稱
	 */
	@Override
	public String[] getColumnNames() {
		return new String[]{"Room ID", "Room State", "Host", "isHuman", "MidBoard", "Round", "FirstMove", "ChangeFirst", "Time", "PlyTimeLimit", "GroupPly", "MaxPly"};
	}
	
	/**
	 * {@inheritDoc}
	 * 確認加房是否合法，暗棋這邊是用來檢測開房和加房是否為同一個帳號(伺服器不允許開房和加房為同一個帳號)。
	 * 
	 * @param roominfo 要加房的房間各欄位資訊。
	 * @return 是否合法。
	 */
	@Override
	public Pair<Boolean, String> checkLegel(List<String> roominfo) {
//		if(roominfo.get(2).equals(MyName)){
//			return Pair.<Boolean, String>makePair(false, "You can't use the same name as the host."); 
//		}else{
			// check by server
			return Pair.<Boolean, String>makePair(true, ""); 
//		}
	}
	
	/**
	 * {@inheritDoc}
	 * 取得主要的顯示資訊，房間ID 房間狀態 房主名稱 場數。
	 * 
	 * @param roomData 原始的房間資料。
	 * @return 精簡後的主要顯示資訊。
	 */
	@Override
	public ArrayList<String> chooseMainData(ArrayList<String> roomData) {
		ArrayList<String> main_roomData = new ArrayList<String>();
		for(String info : roomData){
			String[] split_info = info.split(",");
			StringBuilder result = new StringBuilder();
			result.append(split_info[0]); result.append(",");
			result.append(split_info[1]); result.append(",");
			result.append(split_info[2]); result.append(",");
			result.append(split_info[5]); main_roomData.add(result.toString());
		}
		return main_roomData;
	}
}
