package gst;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.math.BigDecimal;
import java.math.RoundingMode;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
import javax.swing.border.TitledBorder;

import client.Game;
import client.GameRecordList;
import client.LocalServer;
import client.LoginCallBack;
import client.Message;
import client.ProgramPath;
import client.Room;
import client.Room_SettingFile;
import client.Server;
import client.SettingFile;

/**
 * 房間頁面的各項設定。
 * @author ldslds449
 * @see client.Room
 */
public class GST_Room extends Room{
	
	private JLabel Name_Field;
	private JLabel Opponent_Field;
	private JLabel Host_Field;
	private JLabel MidBoard_Field;
	private JLabel Round_Field;
	private JLabel HostFirstMove_Field;
	private JLabel ChangeFirstMove_Field;
	private JLabel TimeLimit_Field;
	private JLabel PlyTimeLimit_Field;
	private JLabel GourpPly_Field;
	private JLabel LongCatch_Field;
	private JLabel MaxPly_Field;
	private JLabel PlayerType_Field;
	
	private JLabel MyTime_Field;
	private JLabel OppoTime_Field;
	
	private JLabel ErrorTime_Field;
	private JLabel Statistics_Field;
	
	private JPanel Name_Color;
	private JPanel Opponent_Color;
	private JLabel Opponent_Turn;
	private JLabel Name_Turn;
	
	private Timer MyTimer;
	private Timer OppoTimer;
	private long MyTime_startTime = 0;
	private long OppoTime_startTime = 0;
	
	/**
	 * constructor，初始化計時的 Timer。
	 * @param server 與 Server 連線的 Socket。
	 * @param setting 房間的遊戲設定。
	 * @param roomSetting 房間設定，像是盤面開啟方式。
	 * @param login 有實作{@link client.LoginCallBack}的設定頁面，用來在關閉視窗後恢復設定頁面用的。
	 * @param localserver 繼承至{@link client.LocalServer}的 class，用來在關閉視窗後關閉 localserver 用的。
	 */
	public GST_Room(Server server, SettingFile setting, Room_SettingFile roomSetting, LoginCallBack login,
			LocalServer localserver) {
		super(server, setting, roomSetting, login, localserver);
		
		// set Timer
		MyTimer = new Timer(100, new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				if(MyTime_startTime == 0){
					MyTime_startTime = System.currentTimeMillis();
				}else{
					long currentTime = System.currentTimeMillis();
					double diff = (double)(currentTime - MyTime_startTime) / 1000;
					
					String my = BigDecimal.valueOf(Double.parseDouble(MyTime_Field.getText()) - diff).setScale(3, BigDecimal.ROUND_HALF_UP).toString();
					
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							if(MyTimer.isRunning()){
								MyTime_Field.setText(my);
							}
						}
					});
					
					MyTime_startTime = currentTime;
				}
			}
		});
		MyTimer.setInitialDelay(0);
		
		OppoTimer = new Timer(100, new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				if(OppoTime_startTime == 0){
					OppoTime_startTime = System.currentTimeMillis();
				}else{
					long currentTime = System.currentTimeMillis();
					double diff = (double)(currentTime - OppoTime_startTime) / 1000;
					
					String oppo = BigDecimal.valueOf(Double.parseDouble(OppoTime_Field.getText()) - diff).setScale(3, BigDecimal.ROUND_HALF_UP).toString();
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							if(OppoTimer.isRunning()){
								OppoTime_Field.setText(oppo);
							}
						}
					});
					
					OppoTime_startTime = currentTime;
				}
			}
		});
		OppoTimer.setInitialDelay(0);
	}
	
	/**
	 * {@inheritDoc}
	 * Client 取得房間遊戲參數顯示的 Panel。
	 * @return 房間頁面中左半部的房間遊戲參數顯示 Panel。
	 */
	@Override
	protected JPanel getPanel(){
		JPanel panel_11 = new JPanel();
		panel_11.setMinimumSize(new Dimension(300, 600));
		panel_11.setPreferredSize(new Dimension(300, Short.MAX_VALUE));
		panel_11.setMaximumSize(new Dimension(300, 32767));
		panel_11.setLayout(new BoxLayout(panel_11, BoxLayout.Y_AXIS));
		
		JPanel panel_2 = new JPanel();
		panel_11.add(panel_2);
		panel_2.setBorder(new TitledBorder(null, "Room Info", TitledBorder.LEADING, TitledBorder.TOP, null, null));
		panel_2.setLayout(new GridLayout(12, 1, 0, 0));
		
		JPanel panel = new JPanel();
		panel_2.add(panel);
		GridBagLayout gbl_panel = new GridBagLayout();
		gbl_panel.columnWidths = new int[]{95, 120, 80, 0};
		gbl_panel.rowHeights = new int[]{23, 0};
		gbl_panel.columnWeights = new double[]{0.0, 0.0, 0.0, Double.MIN_VALUE};
		gbl_panel.rowWeights = new double[]{1.0, Double.MIN_VALUE};
		panel.setLayout(gbl_panel);
		
		JLabel Name = new JLabel("MyName: ");
		Name.setHorizontalAlignment(SwingConstants.RIGHT);
		Name.setOpaque(true);
		Name.setBackground(new Color(211, 211, 211));
		Name.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		GridBagConstraints gbc_Name = new GridBagConstraints();
		gbc_Name.fill = GridBagConstraints.BOTH;
		gbc_Name.insets = new Insets(0, 0, 0, 5);
		gbc_Name.gridx = 0;
		gbc_Name.gridy = 0;
		panel.add(Name, gbc_Name);
		
		Name_Field = new JLabel("", SwingConstants.CENTER);
		GridBagConstraints gbc_Name_Field = new GridBagConstraints();
		gbc_Name_Field.fill = GridBagConstraints.BOTH;
		gbc_Name_Field.insets = new Insets(0, 0, 0, 5);
		gbc_Name_Field.gridx = 1;
		gbc_Name_Field.gridy = 0;
		panel.add(Name_Field, gbc_Name_Field);
		Name_Field.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		
		JPanel panel_8 = new JPanel();
		GridBagConstraints gbc_panel_8 = new GridBagConstraints();
		gbc_panel_8.fill = GridBagConstraints.BOTH;
		gbc_panel_8.gridx = 2;
		gbc_panel_8.gridy = 0;
		panel.add(panel_8, gbc_panel_8);
		panel_8.setLayout(new BoxLayout(panel_8, BoxLayout.X_AXIS));
		
		Component horizontalStrut = Box.createHorizontalStrut(5);
		panel_8.add(horizontalStrut);
		
		Name_Color = new JPanel();
		panel_8.add(Name_Color);
		Name_Color.setPreferredSize(new Dimension(19, 19));
		Name_Color.setMinimumSize(new Dimension(19, 19));
		Name_Color.setMaximumSize(new Dimension(19, 19));
		
		Component horizontalGlue_1 = Box.createHorizontalGlue();
		panel_8.add(horizontalGlue_1);
		
		Name_Turn = new JLabel("");
		Name_Turn.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_8.add(Name_Turn);
		
		JPanel panel_4 = new JPanel();
		panel_2.add(panel_4);
		GridBagLayout gbl_panel_4 = new GridBagLayout();
		gbl_panel_4.columnWidths = new int[]{96, 120, 80, 0};
		gbl_panel_4.rowHeights = new int[]{28, 0};
		gbl_panel_4.columnWeights = new double[]{0.0, 0.0, 0.0, Double.MIN_VALUE};
		gbl_panel_4.rowWeights = new double[]{1.0, Double.MIN_VALUE};
		panel_4.setLayout(gbl_panel_4);
		
		JLabel Opponent = new JLabel("OpName: ");
		Opponent.setHorizontalAlignment(SwingConstants.RIGHT);
		Opponent.setBackground(new Color(211, 211, 211));
		Opponent.setOpaque(true);
		Opponent.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		GridBagConstraints gbc_Opponent = new GridBagConstraints();
		gbc_Opponent.fill = GridBagConstraints.BOTH;
		gbc_Opponent.insets = new Insets(0, 0, 0, 5);
		gbc_Opponent.gridx = 0;
		gbc_Opponent.gridy = 0;
		panel_4.add(Opponent, gbc_Opponent);
		
		Opponent_Field = new JLabel("", SwingConstants.CENTER);
		Opponent_Field.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		GridBagConstraints gbc_Opponent_Field = new GridBagConstraints();
		gbc_Opponent_Field.fill = GridBagConstraints.BOTH;
		gbc_Opponent_Field.insets = new Insets(0, 0, 0, 5);
		gbc_Opponent_Field.gridx = 1;
		gbc_Opponent_Field.gridy = 0;
		panel_4.add(Opponent_Field, gbc_Opponent_Field);
		
		JPanel panel_22 = new JPanel();
		GridBagConstraints gbc_panel_22 = new GridBagConstraints();
		gbc_panel_22.fill = GridBagConstraints.BOTH;
		gbc_panel_22.gridx = 2;
		gbc_panel_22.gridy = 0;
		panel_4.add(panel_22, gbc_panel_22);
		panel_22.setLayout(new BoxLayout(panel_22, BoxLayout.X_AXIS));
		
		Component horizontalStrut_1 = Box.createHorizontalStrut(5);
		panel_22.add(horizontalStrut_1);
		
		Opponent_Color = new JPanel();
		panel_22.add(Opponent_Color);
		Opponent_Color.setPreferredSize(new Dimension(19, 19));
		Opponent_Color.setMinimumSize(new Dimension(19, 19));
		Opponent_Color.setMaximumSize(new Dimension(19, 19));
		
		Component horizontalGlue = Box.createHorizontalGlue();
		panel_22.add(horizontalGlue);
		
		Opponent_Turn = new JLabel("");
		Opponent_Turn.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_22.add(Opponent_Turn);
		
		JPanel panel_14 = new JPanel();
		panel_2.add(panel_14);
		panel_14.setLayout(new GridLayout(0, 2, 0, 0));
		
		JLabel Host = new JLabel("Host: ");
		Host.setHorizontalAlignment(SwingConstants.RIGHT);
		Host.setBackground(new Color(211, 211, 211));
		Host.setOpaque(true);
		Host.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_14.add(Host);
		
		Host_Field = new JLabel("");
		Host_Field.setHorizontalAlignment(SwingConstants.CENTER);
		Host_Field.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_14.add(Host_Field);
		
		JPanel panel_15 = new JPanel();
		panel_2.add(panel_15);
		panel_15.setLayout(new GridLayout(0, 2, 0, 0));
		
		JLabel MidBoard = new JLabel("MidBoard: ");
		MidBoard.setHorizontalAlignment(SwingConstants.RIGHT);
		MidBoard.setBackground(new Color(211, 211, 211));
		MidBoard.setOpaque(true);
		MidBoard.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_15.add(MidBoard);
		
		MidBoard_Field = new JLabel("");
		MidBoard_Field.setHorizontalAlignment(SwingConstants.CENTER);
		MidBoard_Field.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_15.add(MidBoard_Field);
		
		JPanel panel_16 = new JPanel();
		panel_2.add(panel_16);
		panel_16.setLayout(new GridLayout(0, 2, 0, 0));
		
		JLabel Round = new JLabel("Round: ");
		Round.setHorizontalAlignment(SwingConstants.RIGHT);
		Round.setBackground(new Color(211, 211, 211));
		Round.setOpaque(true);
		Round.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_16.add(Round);
		
		Round_Field = new JLabel("");
		Round_Field.setHorizontalAlignment(SwingConstants.CENTER);
		Round_Field.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_16.add(Round_Field);
		
		JPanel panel_17 = new JPanel();
		panel_2.add(panel_17);
		panel_17.setLayout(new GridLayout(0, 2, 0, 0));
		
		JLabel HostFirstMove = new JLabel("HostFirstMove: ");
		HostFirstMove.setHorizontalAlignment(SwingConstants.RIGHT);
		HostFirstMove.setBackground(new Color(211, 211, 211));
		HostFirstMove.setOpaque(true);
		HostFirstMove.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_17.add(HostFirstMove);
		
		HostFirstMove_Field = new JLabel("");
		HostFirstMove_Field.setHorizontalAlignment(SwingConstants.CENTER);
		HostFirstMove_Field.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_17.add(HostFirstMove_Field);
		
		JPanel panel_18 = new JPanel();
		panel_2.add(panel_18);
		panel_18.setLayout(new GridLayout(0, 2, 0, 0));
		
		JLabel ChangeFirstMove = new JLabel("ChangeFirstMove: ");
		ChangeFirstMove.setHorizontalAlignment(SwingConstants.RIGHT);
		ChangeFirstMove.setBackground(new Color(211, 211, 211));
		ChangeFirstMove.setOpaque(true);
		ChangeFirstMove.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_18.add(ChangeFirstMove);
		
		ChangeFirstMove_Field = new JLabel("");
		ChangeFirstMove_Field.setHorizontalAlignment(SwingConstants.CENTER);
		ChangeFirstMove_Field.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_18.add(ChangeFirstMove_Field);
		
		JPanel panel_6 = new JPanel();
		panel_2.add(panel_6);
		panel_6.setLayout(new GridLayout(0, 2, 0, 0));
		
		JLabel TimeLimit = new JLabel("TimeLimit: ");
		TimeLimit.setHorizontalAlignment(SwingConstants.RIGHT);
		TimeLimit.setBackground(new Color(211, 211, 211));
		TimeLimit.setOpaque(true);
		TimeLimit.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_6.add(TimeLimit);
		
		TimeLimit_Field = new JLabel("", SwingConstants.CENTER);
		TimeLimit_Field.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_6.add(TimeLimit_Field);
		
		JPanel panel_19 = new JPanel();
		panel_2.add(panel_19);
		panel_19.setLayout(new GridLayout(0, 2, 0, 0));
		
		JLabel PlyTimeLimit = new JLabel("plyTimeLimit: ");
		PlyTimeLimit.setHorizontalAlignment(SwingConstants.RIGHT);
		PlyTimeLimit.setBackground(new Color(211, 211, 211));
		PlyTimeLimit.setOpaque(true);
		PlyTimeLimit.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_19.add(PlyTimeLimit);
		
		PlyTimeLimit_Field = new JLabel("");
		PlyTimeLimit_Field.setHorizontalAlignment(SwingConstants.CENTER);
		PlyTimeLimit_Field.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_19.add(PlyTimeLimit_Field);
		
		JPanel panel_20 = new JPanel();
		panel_2.add(panel_20);
		panel_20.setLayout(new GridLayout(0, 2, 0, 0));
		
		JLabel GourpPly = new JLabel("GroupPly: ");
		GourpPly.setHorizontalAlignment(SwingConstants.RIGHT);
		GourpPly.setBackground(new Color(211, 211, 211));
		GourpPly.setOpaque(true);
		GourpPly.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_20.add(GourpPly);
		
		GourpPly_Field = new JLabel("");
		GourpPly_Field.setHorizontalAlignment(SwingConstants.CENTER);
		GourpPly_Field.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_20.add(GourpPly_Field);
		
//		JPanel panel_5 = new JPanel();
//		panel_2.add(panel_5);
//		panel_5.setLayout(new GridLayout(0, 2, 0, 0));
		
//		JLabel LongCatch = new JLabel("LongCatch: ");
//		LongCatch.setHorizontalAlignment(SwingConstants.RIGHT);
//		LongCatch.setBackground(new Color(211, 211, 211));
//		LongCatch.setOpaque(true);
//		LongCatch.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
//		panel_5.add(LongCatch);
//		
//		LongCatch_Field = new JLabel("", SwingConstants.CENTER);
//		LongCatch_Field.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
//		panel_5.add(LongCatch_Field);
//		
		JPanel panel_21 = new JPanel();
		panel_2.add(panel_21);
		panel_21.setLayout(new GridLayout(0, 2, 0, 0));
		
		JLabel MaxPly = new JLabel("MaxPly: ");
		MaxPly.setHorizontalAlignment(SwingConstants.RIGHT);
		MaxPly.setBackground(new Color(211, 211, 211));
		MaxPly.setOpaque(true);
		MaxPly.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_21.add(MaxPly);
		
		MaxPly_Field = new JLabel("");
		MaxPly_Field.setHorizontalAlignment(SwingConstants.CENTER);
		MaxPly_Field.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_21.add(MaxPly_Field);
		
		JPanel panel_23 = new JPanel();
		panel_2.add(panel_23);
		panel_23.setLayout(new GridLayout(0, 2, 0, 0));
		
		JLabel PlayerType = new JLabel("PlayerType: ");
		PlayerType.setHorizontalAlignment(SwingConstants.RIGHT);
		PlayerType.setBackground(new Color(211, 211, 211));
		PlayerType.setOpaque(true);
		PlayerType.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_23.add(PlayerType);
		
		PlayerType_Field = new JLabel("");
		PlayerType_Field.setHorizontalAlignment(SwingConstants.CENTER);
		PlayerType_Field.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 14));
		panel_23.add(PlayerType_Field);
		
		JPanel panel_3 = new JPanel();
		panel_11.add(panel_3);
		panel_3.setBorder(new TitledBorder(null, "Game Info", TitledBorder.LEADING, TitledBorder.TOP, null, null));
		panel_3.setLayout(new GridLayout(0, 1, 0, 0));
		
		JPanel panel_7 = new JPanel();
		panel_3.add(panel_7);
		panel_7.setLayout(new GridLayout(0, 3, 0, 0));
		
		JLabel MyTime = new JLabel("MyTime: ");
		MyTime.setHorizontalAlignment(SwingConstants.RIGHT);
		MyTime.setBackground(new Color(211, 211, 211));
		MyTime.setOpaque(true);
		MyTime.setAlignmentX(Component.CENTER_ALIGNMENT);
		MyTime.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_7.add(MyTime);
		
		MyTime_Field = new JLabel("");
		MyTime_Field.setHorizontalAlignment(SwingConstants.CENTER);
		MyTime_Field.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		MyTime_Field.setMinimumSize(new Dimension(100, 0));
		MyTime_Field.setPreferredSize(new Dimension(100, 0));
		panel_7.add(MyTime_Field);
		
		JLabel MyTime_sec = new JLabel(" (sec)");
		MyTime_sec.setHorizontalAlignment(SwingConstants.CENTER);
		MyTime_sec.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_7.add(MyTime_sec);
		
		JPanel panel_1 = new JPanel();
		panel_3.add(panel_1);
		panel_1.setLayout(new GridLayout(0, 3, 0, 0));
		
		JLabel OppoTime = new JLabel("OpTime: ");
		OppoTime.setHorizontalAlignment(SwingConstants.RIGHT);
		OppoTime.setBackground(new Color(211, 211, 211));
		OppoTime.setOpaque(true);
		OppoTime.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_1.add(OppoTime);
		
		OppoTime_Field = new JLabel("");
		OppoTime_Field.setHorizontalAlignment(SwingConstants.CENTER);
		OppoTime_Field.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_1.add(OppoTime_Field);
		
		JLabel OppoTime_sec = new JLabel(" (sec)");
		OppoTime_sec.setHorizontalAlignment(SwingConstants.CENTER);
		OppoTime_sec.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_1.add(OppoTime_sec);
		
		JPanel panel_12 = new JPanel();
		panel_3.add(panel_12);
		panel_12.setLayout(new GridLayout(0, 3, 0, 0));
		
		JLabel ErrorTime = new JLabel("ErrorTime: ");
		ErrorTime.setHorizontalAlignment(SwingConstants.RIGHT);
		ErrorTime.setBackground(new Color(211, 211, 211));
		ErrorTime.setOpaque(true);
		ErrorTime.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_12.add(ErrorTime);
		
		ErrorTime_Field = new JLabel("");
		ErrorTime_Field.setHorizontalAlignment(SwingConstants.CENTER);
		ErrorTime_Field.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_12.add(ErrorTime_Field);
		
		JLabel ErrorTime_sec = new JLabel(" (sec)");
		ErrorTime_sec.setHorizontalAlignment(SwingConstants.CENTER);
		ErrorTime_sec.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_12.add(ErrorTime_sec);
		
		JPanel panel_13 = new JPanel();
		panel_3.add(panel_13);
		panel_13.setLayout(new GridLayout(0, 3, 0, 0));
		
		JLabel Statistics = new JLabel("Statistics: ");
		Statistics.setHorizontalAlignment(SwingConstants.RIGHT);
		Statistics.setBackground(new Color(211, 211, 211));
		Statistics.setOpaque(true);
		Statistics.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_13.add(Statistics);
		
		Statistics_Field = new JLabel("0/0/0");
		Statistics_Field.setHorizontalAlignment(SwingConstants.CENTER);
		Statistics_Field.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_13.add(Statistics_Field);
		
		JLabel Statistics_Label = new JLabel(" (W/D/L)");
		Statistics_Label.setHorizontalAlignment(SwingConstants.CENTER);
		Statistics_Label.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 16));
		panel_13.add(Statistics_Label);
		
		return panel_11;
	}
	
	/**
	 * {@inheritDoc}
	 * Client 用來取得此遊戲的 class。
	 * @return 此遊戲的 class(需繼承{@link client.Game})
	 */
	@Override
	protected Game getGame(Server server, SettingFile setting, Room_SettingFile roomSetting){
		return new GST_Game(server, (GST_SettingFile)setting, this, roomSetting, false, false, null);
	}
	
	/**
	 * 設定房間遊戲時間的顯示。
	 * @param mytime 我方時間。
	 * @param oppotime 敵方時間。
	 * @param errorTime 延遲時間。
	 * @param win 勝利場數。 
	 * @param draw 平手場數。
	 * @param lose 輸的場數。
	 */
	void setGameInfo(int mytime, int oppotime, int errorTime, int win, int draw, int lose) {
		
		if(errorTime < 0){
			errorTime = 0;
		}
		
		BigDecimal thousand = BigDecimal.valueOf(1000);
		BigDecimal my = BigDecimal.valueOf(mytime).divide(thousand, 3, RoundingMode.HALF_UP);
		BigDecimal oppo = BigDecimal.valueOf(oppotime).divide(thousand, 3, RoundingMode.HALF_UP);
		BigDecimal error = BigDecimal.valueOf(errorTime).divide(thousand, 3, RoundingMode.HALF_UP);
		
		SwingUtilities.invokeLater(new Runnable(){
			@Override
			public void run() {
				if(error.doubleValue() < 0.100){
					ErrorTime_Field.setForeground(Color.darkGray);
				}else if(error.doubleValue() < 0.200){
					ErrorTime_Field.setForeground(Color.orange);
				}else{
					ErrorTime_Field.setForeground(Color.red);
				}
				
				MyTime_Field.setText(my.toString());
				OppoTime_Field.setText(oppo.toString());
				ErrorTime_Field.setText(error.toString());
			}
		});
		
		GST_Room.this.setGameResult(win, draw, lose);
	}
	
	/**
	 * 設定房間遊戲參數的顯示。
	 * @param info 房間遊戲參數。
	 * @param oppoName 敵方名稱
	 * @param oppoPlayerType 敵方玩家類型。
	 */
	void setRoomInfo(SettingFile info, String oppoName, boolean oppoPlayerType) {
		SwingUtilities.invokeLater(new Runnable(){
			@Override
			public void run() {
				Name_Field.setText(info.account);
				Opponent_Field.setText(oppoName);
				
				Host_Field.setText((info.startMode == SettingFile.STARTMODE.OPEN ? info.account : oppoName));
				MidBoard_Field.setText(((GST_SettingFile)info).midBoard.name());
				Round_Field.setText(Integer.toString(info.repeatTime));
				HostFirstMove_Field.setText(((GST_SettingFile)info).firstMove.name());
				ChangeFirstMove_Field.setText(((GST_SettingFile)info).changeFirstMove.name());
				TimeLimit_Field.setText(Integer.toString(((GST_SettingFile)info).timeLimit));
				PlyTimeLimit_Field.setText(Integer.toString(((GST_SettingFile)info).onePlyTime));
				GourpPly_Field.setText(Integer.toString(((GST_SettingFile)info).plyShare));
				MaxPly_Field.setText(Integer.toString(((GST_SettingFile)info).MaxPly));
				
				String playerType_1 = (((GST_SettingFile)info).connectMode == GST_SettingFile.CONNECTMODE.HUMAN ? GST_SettingFile.CONNECTMODE.HUMAN.name() : "AI");
				String playerType_2 = (oppoPlayerType ? GST_SettingFile.CONNECTMODE.HUMAN.name() : "AI");
				PlayerType_Field.setText(String.format("%s vs %s", playerType_1, playerType_2));
			}
		});
	}
	
	/**
	 * 設定玩家執子顏色。
	 * @param mycolor 我方顏色(敵方顏色為相反)。
	 */
	void setPlayerColor(int mycolor) {
		SwingUtilities.invokeLater(new Runnable(){
			@Override
			public void run() {
				// 0: #ffa500 1:#5cc300
				if(mycolor == 0){
					Name_Color.setBackground(Color.decode("#ffa500"));
					Opponent_Color.setBackground(Color.decode("#5cc300"));
				}else{
					Name_Color.setBackground(Color.decode("#5cc300"));
					Opponent_Color.setBackground(Color.decode("#ffa500"));
				}
				Name_Color.repaint();
				Opponent_Color.repaint();
			}
		});
	}
	
	/**
	 * 設定輪到哪位玩家。
	 * @param turn 換誰(0: 我方 1:敵方)
	 */
	void setWhichTurn(int turn) {
		// 0: mine 1:your
		if(turn == 0){
			Name_Turn.setText("<===");
			Opponent_Turn.setText("");
			
			// stop Timer
			if(OppoTimer != null && OppoTimer.isRunning()){
				OppoTimer.stop();
			}
			// start Timer
			if(MyTimer != null){
				MyTime_startTime = 0;
				MyTimer.restart();
			}
		}else{
			Name_Turn.setText("");
			Opponent_Turn.setText("<===");
			
			// stop Timer
			if(MyTimer != null && MyTimer.isRunning()){
				MyTimer.stop();
			}
			// start Timer
			if(OppoTimer != null){
				OppoTime_startTime = 0;
				OppoTimer.restart();
			}
		}
	}
	
	/**
	 * 停止遊戲的計時器。
	 */
	void stopTimer() {
		// stop Timer
		if(OppoTimer != null && OppoTimer.isRunning()){
			OppoTimer.stop();
		}
		
		if(MyTimer != null && MyTimer.isRunning()){
			MyTimer.stop();
		}
	}
	
	/**
	 * 設定房間內遊戲勝負統計表。
	 * @param win 贏幾場。
	 * @param draw 平手幾場。
	 * @param lose 輸幾場。
	 */
	void setGameResult(int win, int draw, int lose) {		
		SwingUtilities.invokeLater(new Runnable(){
			@Override
			public void run() {
				Statistics_Field.setText(String.format("%d/%d/%d", win, draw, lose));
			}
		});
	}
	
	/**
	 * {@inheritDoc}
	 * 取得房間內遊戲記錄列表的 class(需繼承{@link client.GameRecordList})。
	 * @return 房間內遊戲記錄列表的 class
	 */
	@Override
	protected GameRecordList getGameRecordList(String path) {
		return new GST_GameRecordList(path);
	}
	
	/**
	 * {@inheritDoc}
	 * 取得遊戲歷史盤面的 HTML 檔案位置。
	 * @return 遊戲歷史盤面的 HTML 檔案位置
	 */
	@Override
	protected File getHistoryHTML() {
		return new File(String.format("%s%sWebServer%s%s%shistory.html", ProgramPath.getProgramPath(), File.separator, File.separator, GameInfo.getGameName(), File.separator));
	}
	
}