package gst;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Properties;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.exception.ExceptionUtils;

import client.DynActionView;
import client.Message;
import client.Room_SettingFile;
import client.SettingFile;
import io.bretty.console.view.AbstractView;
import io.bretty.console.view.ActionView;
import io.bretty.console.view.MenuView;
import io.bretty.console.view.Validator;
import io.bretty.console.view.ViewConfig;

/**
 * 管理整個遊戲的設定檔案，包含設定頁面的控制和儲存遊戲參數的 class。
 * @author ldslds449
 * @see client.SettingFile
 */
public class GST_SettingFile extends SettingFile{
	
	boolean useCLI;
	
	public enum MIDBOARD{NO, YES, CUSTOM}; // 0: no use midBoard 1: use midBoard 2: use custom midBoard
	public enum FIRSTMOVE{NO, YES}; // 0: opponent move first 1: user move first
	public enum CHANGEFIRSTMOVE{NO, YES}; // 0: keep the player firstMove 1: exchange the firstMove right
	public enum TIMERMODE{NONE, ABSOLUTE, BYOYMI, CANDIAN, CUSTOM}; // 0: none 1: absolute 2: byoymi 3: candian 4: custom
	public enum CONNECTMODE{TCP, STDIO, HUMAN}; // 0: ponder 1: MGTP 2: Human
//	public enum SETREDMODE{CUSTOM, RANDOM};
	
	public MIDBOARD midBoard;  // if use the midboard
	public FIRSTMOVE firstMove;  // if the player move first
	public CHANGEFIRSTMOVE changeFirstMove;  // if change the first move player
	public int timeLimit;  // limit the max time of a game
	public CONNECTMODE connectMode;  // the connect mode to search exe
	public int onePlyTime;  // limit the max time of one ply
	public int plyShare;  //
	public int MaxPly;  // limit the max MaxPly
	public TIMERMODE timerMode;  // the default time setting
	public String boardPath;  // the custom midBoard path
//	public SETREDMODE setRedMode;
//	public double winScore; // score of Win
//	public double drawScore; // score of Draw
//	public double loseScore; // score of Lose
	
	
	private JTextField account_Field;
	private JTextField password_Field;
	private JRadioButton roomType_1;
	private JRadioButton roomType_2;
	private JRadioButton startMode_1;
	private JRadioButton startMode_2;
	private JCheckBox localServer_CheckBox;
	private JRadioButton midBoard_1;
	private JRadioButton midBoard_2;
	private JRadioButton midBoard_3;
	private JRadioButton connectMode_1;
	private JRadioButton connectMode_2;
	private JRadioButton connectMode_3;
	private JTextField repeatTime_Field;
	private JRadioButton firstMove_1;
	private JRadioButton firstMove_2;
	private JRadioButton changeFirstMove_1;
	private JRadioButton changeFirstMove_2;
	private JComboBox<String> timerMode_ComboBox;
	private JTextField timeLimit_Field;
	private JTextField onePlyTime_Field;
	private JTextField plyShare_Field;
	private JTextField MaxPly_Field;
	private JTextField ip_Field;
	private JTextField boardPath_Field;
	private JButton chooseBoardFile;
	private JTextField searchPath_Field;
	private JButton chooseSearchFile;
	private JTextField arg_Field;
//	private JRadioButton setRedMode_1;
//	private JRadioButton setRedMode_2;
	
	/**
	 * GUI mode constructor，需要傳入 component 來設定各個 component 的初始值設定和取得 User 設定的值。
	 * @param account_Field 帳號欄位。
	 * @param password_Field 密碼欄位。
	 * @param roomType_1 一般模式。
	 * @param roomType_2 比賽模式。
	 * @param roomType_3 開局庫模式。
	 * @param startMode_1 開房。
	 * @param startMode_2 加房。
	 * @param localServer_CheckBox 是否用本地伺服器。
	 * @param midBoard_1 斷線中盤。
	 * @param midBoard_2 正常初始盤面。
	 * @param midBoard_3 自訂中盤。
	 * @param connectMode_1 ReadFile。
	 * @param connectMode_2 Ponder。
	 * @param connectMode_3 MGTP。
	 * @param connectMode_4 Human。
	 * @param repeatTime_Field 共要打幾局。
	 * @param firstMove_1 開房先手。
	 * @param firstMove_2 開房後手。
	 * @param changeFirstMove_1 結束一局後交換先後手。
	 * @param changeFirstMove_2 結束一局後不交換先後手。
	 * @param timerMode_ComboBox 計時模式選項欄。
	 * @param timeLimit_Field 遊戲限時幾秒。
	 * @param onePlyTime_Field 單手制時間。
	 * @param plyShare_Field 幾手共用。
	 * @param longCatch_Field 長捉次數。
	 * @param MaxPly_Field 無吃翻次數。
	 * @param ip_Field 伺服器 IP。
	 * @param boardPath_Field 盤面路徑。
	 * @param chooseBoardFile 選擇盤面路徑按鈕。
	 * @param searchPath_Field Search 路徑。
	 * @param chooseSearchFile 選擇 Search 路徑按鈕。
	 * @param setRedMode_1 循序。
	 * @param setRedMode_2 隨機。
	 * @param timePerOpening_Field 單盤次數。
	 * @param win_Field 勝場分數。
	 * @param draw_Field 和局分數。
	 * @param lose_Field 敗場分數。
	 * @param piece_Field 棋子分數。
	 * @param bonus_Field 加分分數。
	 * @param bonusLimit_Field 加分上限。
	 * @param scoreSetting 設定頁面按鈕。
	 * @param wdlScore_Label 顯示勝和負。
	 */
	public GST_SettingFile(
			JTextField account_Field,
			JTextField password_Field,
			JRadioButton roomType_1,
			JRadioButton roomType_2,
			JRadioButton startMode_1,
			JRadioButton startMode_2,
			JCheckBox localServer_CheckBox,
			JRadioButton midBoard_1,
			JRadioButton midBoard_2,
			JRadioButton midBoard_3,
			JRadioButton connectMode_1,
			JRadioButton connectMode_2,
			JRadioButton connectMode_3,
			JTextField repeatTime_Field,
			JRadioButton firstMove_1,
			JRadioButton firstMove_2,
			JRadioButton changeFirstMove_1,
			JRadioButton changeFirstMove_2,
			JComboBox<String> timerMode_ComboBox,
			JTextField timeLimit_Field,
			JTextField onePlyTime_Field,
			JTextField plyShare_Field,
			JTextField MaxPly_Field,
			JTextField ip_Field,
			JTextField boardPath_Field,
			JButton chooseBoardFile,
			JTextField searchPath_Field,
			JButton chooseSearchFile,
			JTextField arg_Field){
		super();
		
		fileName = String.format("%s%s%s.config", 
				this.fileDir, 
				File.separator,
				GameInfo.getGameName());
		gametype = GameInfo.getGameName();
		useCLI = false;
		
		this.account_Field = account_Field;
		this.password_Field = password_Field;
		this.roomType_1 = roomType_1;
		this.roomType_2 = roomType_2;
		this.startMode_1 = startMode_1;
		this.startMode_2 = startMode_2;
		this.localServer_CheckBox = localServer_CheckBox;
		this.midBoard_1 = midBoard_1;
		this.midBoard_2 = midBoard_2;
		this.midBoard_3 = midBoard_3;
		this.connectMode_1 = connectMode_1;
		this.connectMode_2 = connectMode_2;
		this.connectMode_3 = connectMode_3;
		this.repeatTime_Field = repeatTime_Field;
		this.firstMove_1 = firstMove_1;
		this.firstMove_2 = firstMove_2;
		this.changeFirstMove_1 = changeFirstMove_1;
		this.changeFirstMove_2 = changeFirstMove_2;
		this.timerMode_ComboBox = timerMode_ComboBox;
		this.timeLimit_Field = timeLimit_Field;
		this.onePlyTime_Field = onePlyTime_Field;
		this.plyShare_Field = plyShare_Field;
		this.MaxPly_Field = MaxPly_Field;
		this.ip_Field = ip_Field;
		this.boardPath_Field = boardPath_Field;
		this.chooseBoardFile = chooseBoardFile;
		this.searchPath_Field = searchPath_Field;
		this.chooseSearchFile = chooseSearchFile;
		this.arg_Field = arg_Field;
//		this.setRedMode_1 = setRedMode_1;
//		this.setRedMode_2 = setRedMode_2;
	}
	
	/**
	 * CLI mode constructor，因為 CLI mode 沒有使用者介面，故不用傳入 component。
	 */
	public GST_SettingFile(){
		// used for CLI mode
		super();
		
		fileName = String.format("%s%s%s.config", 
				this.fileDir, 
				File.separator,
				GameInfo.getGameName());
		gametype = GameInfo.getGameName();
		useCLI = true;
	}
	
	/**
	 * 讀取設定紀錄檔案。
	 */
	public void readSettingFile() throws IOException{
		// read initial file setting
		File f = new File(fileName);
		if(f.exists()) {
			Properties p = new Properties();
			try {
				p.load(new FileReader(fileName));
			} catch (IOException e) {
				Message.printError("#ERROR: READ Geister SETTING FILE");
				Message.printError("#FileName: %s", fileName);
				Message.printError(ExceptionUtils.getStackTrace(e));
				throw e;
			}
			
			// account
			account = p.getProperty("Account", "");
			// password
			password = p.getProperty("Password", "");
			// roomType
			roomType = ROOMTYPE.valueOf(p.getProperty("RoomType", ROOMTYPE.GENERAL.name()));
			// startMode
			startMode = STARTMODE.valueOf(p.getProperty("StartMode", STARTMODE.OPEN.name()));
			// localServer
			localServer = Boolean.valueOf(p.getProperty("LocalServer", "false"));
			// midBoard
			midBoard = MIDBOARD.valueOf(p.getProperty("MidBoard", MIDBOARD.NO.name()));
			// connectMode
			connectMode = CONNECTMODE.valueOf(p.getProperty("ConnectMode", CONNECTMODE.STDIO.name()));
			// repeatTime
			repeatTime = Integer.parseInt(p.getProperty("RepeatTime", "100"));
			// firstMove
			firstMove = FIRSTMOVE.valueOf(p.getProperty("FirstMove", FIRSTMOVE.YES.name()));
			// changeFirstMove
			changeFirstMove = CHANGEFIRSTMOVE.valueOf(p.getProperty("ChangeFirstMove", CHANGEFIRSTMOVE.YES.name()));
			// timerMode
			timerMode = TIMERMODE.valueOf(p.getProperty("TimerMode", TIMERMODE.ABSOLUTE.name()));
			// timeLimit
			timeLimit = Integer.parseInt(p.getProperty("TimeLimit", "900"));
			// onePlyTime
			onePlyTime = Integer.parseInt(p.getProperty("OnePlyTime", "0"));
			// plyShare
			plyShare = Integer.parseInt(p.getProperty("PlyShare", "0"));
			// MaxPly
			MaxPly = Integer.parseInt(p.getProperty("MaxPly", "200"));
			// ip
			ip = p.getProperty("IP", "");
			// boardPath
			boardPath = p.getProperty("BoardPath", "");
			// searchPath
			searchPath = p.getProperty("SearchPath", "");
			// arg
			arg = p.getProperty("Arg", "");
			// opening mode
//			setRedMode = SETREDMODE.valueOf(p.getProperty("setRedMode", SETREDMODE.CUSTOM.name()));
			
		}else{ // set default setting
			account = "";
			password = "";
			roomType = ROOMTYPE.GENERAL;
			startMode = STARTMODE.OPEN;
			localServer = false;
			midBoard = MIDBOARD.NO;
			connectMode = CONNECTMODE.STDIO;
			repeatTime = 100;
			firstMove = FIRSTMOVE.YES;
			changeFirstMove = CHANGEFIRSTMOVE.YES;
			timerMode = TIMERMODE.ABSOLUTE;
			timeLimit = 900;
			onePlyTime = 0;
			plyShare = 0;
			MaxPly = 200;
			ip = "120.126.151.213";
			boardPath = "";
			searchPath = "";
			arg = "";
//			setRedMode = SETREDMODE.CUSTOM;
		}
		// modify initial invalid setting
		if(localServer && midBoard == MIDBOARD.YES){
			midBoard = MIDBOARD.NO;
		}
		if(!localServer && roomType == ROOMTYPE.OPENING){
			localServer = true;
		}
		if(roomType == ROOMTYPE.OPENING && startMode == STARTMODE.ENTER){
			startMode = STARTMODE.OPEN;
		}
		if(useCLI){
			if(connectMode == CONNECTMODE.HUMAN){
				connectMode = CONNECTMODE.STDIO;
			}
		}
		switch(GST_SettingFile.this.timerMode){
		case ABSOLUTE:
			GST_SettingFile.this.timeLimit = 900;
			GST_SettingFile.this.onePlyTime = 0;
			GST_SettingFile.this.plyShare = 0;
			break;
		case BYOYMI:
			GST_SettingFile.this.timeLimit = 900;
			GST_SettingFile.this.onePlyTime = 5;
			GST_SettingFile.this.plyShare = 1;
			break;
		case CANDIAN:
			GST_SettingFile.this.timeLimit = 900;
			GST_SettingFile.this.onePlyTime = 30;
			GST_SettingFile.this.plyShare = 3;
			break;
		case CUSTOM:
			break;
		case NONE:
			GST_SettingFile.this.timeLimit = 0;
			GST_SettingFile.this.onePlyTime = 0;
			GST_SettingFile.this.plyShare = 0;
			break;
		}
	}
	
	/**
	 * 將遊戲參數寫入記錄設定檔案。
	 */
	public void writeSettingFile() throws IOException{
		Properties p = new Properties();
		
		p.setProperty("Account", account);
		p.setProperty("Password", password);
		p.setProperty("RoomType", roomType.name());
		p.setProperty("StartMode", startMode.name());
		p.setProperty("LocalServer", Boolean.toString(localServer));
		p.setProperty("MidBoard", midBoard.name());
		p.setProperty("ConnectMode", connectMode.name());
		p.setProperty("RepeatTime", Integer.toString(repeatTime));
		p.setProperty("FirstMove", firstMove.name());
		p.setProperty("ChangeFirstMove", changeFirstMove.name());
		p.setProperty("TimerMode", timerMode.name());
		p.setProperty("TimeLimit", Integer.toString(timeLimit));
		p.setProperty("OnePlyTime", Integer.toString(onePlyTime));
		p.setProperty("PlyShare", Integer.toString(plyShare));
		p.setProperty("MaxPly", Integer.toString(MaxPly));
		p.setProperty("IP", ip);
		p.setProperty("BoardPath", boardPath);
		p.setProperty("SearchPath", searchPath);
		p.setProperty("Arg", arg);
//		p.setProperty("setRedMode", setRedMode.name());
		
		try {
			p.store(new FileWriter(fileName), "USE FOR Geister INITIAL SETTING");
		} catch (IOException e) {
			Message.printError("#ERROR: WRITE Geister SETTING FILE");
			Message.printError("#FileName: %s", fileName);
			Message.printError(ExceptionUtils.getStackTrace(e));
			throw e;
		}
	}
	
	/**
	 * 將遊戲參數配置到 component 上面。
	 */
	public void getInitSetting() throws IOException{
		
		readSettingFile();
		// set swing value
		account_Field.setText(account);
		password_Field.setText(password);
		switch(roomType){
		case GENERAL:
			roomType_1.setSelected(true);
			roomType_2.setSelected(false);
			break;
		case CONTEST:
			roomType_1.setSelected(false);
			roomType_2.setSelected(true);
			break;
		}
		if(roomType == ROOMTYPE.GENERAL){
//			setRedMode_1.setEnabled(false);
//			setRedMode_2.setEnabled(false);
			switch(startMode){
			case OPEN:
				startMode_1.setSelected(true);
				startMode_2.setSelected(false);
				if(!localServer){
					midBoard_1.setEnabled(true);
				}else{
					midBoard_1.setEnabled(false);
				}
				midBoard_2.setEnabled(true);
				midBoard_3.setEnabled(true);
				repeatTime_Field.setEnabled(true);
				firstMove_1.setEnabled(true);
				firstMove_2.setEnabled(true);
				changeFirstMove_1.setEnabled(true);
				changeFirstMove_2.setEnabled(true);
				timerMode_ComboBox.setEnabled(true);
				if(timerMode_ComboBox.getSelectedIndex() == 4){
					timeLimit_Field.setEnabled(true);
					onePlyTime_Field.setEnabled(true);
					plyShare_Field.setEnabled(true);
				}
				MaxPly_Field.setEnabled(true);
				break;
			case ENTER:
				startMode_1.setSelected(false);
				startMode_2.setSelected(true);
				midBoard_1.setEnabled(false);
				midBoard_2.setEnabled(false);
				midBoard_3.setEnabled(false);
				repeatTime_Field.setEnabled(false);
				firstMove_1.setEnabled(false);
				firstMove_2.setEnabled(false);
				changeFirstMove_1.setEnabled(false);
				changeFirstMove_2.setEnabled(false);
				timerMode_ComboBox.setEnabled(false);
				timeLimit_Field.setEnabled(false);
				onePlyTime_Field.setEnabled(false);
				plyShare_Field.setEnabled(false);
				MaxPly_Field.setEnabled(false);
				break;
			}
			localServer_CheckBox.setSelected(localServer);
			if(localServer){
				ip_Field.setEnabled(false);
				midBoard_1.setEnabled(false);
				roomType_2.setEnabled(false);
			}else{
				ip_Field.setEnabled(true);
				roomType_2.setEnabled(true);
				if(startMode == STARTMODE.OPEN){
					midBoard_1.setEnabled(true);
				}else{
					midBoard_1.setEnabled(false);
				}
			}
			switch(midBoard){
			case NO:
				midBoard_1.setSelected(false);
				midBoard_2.setSelected(true);
				midBoard_3.setSelected(false);
				boardPath_Field.setEnabled(false);
				chooseBoardFile.setEnabled(false);
				break;
			case YES:
				if(!localServer){
					midBoard_1.setSelected(true);
					midBoard_2.setSelected(false);
				}else{
					midBoard_1.setSelected(false);
					midBoard_2.setSelected(true);
				}
				midBoard_3.setSelected(false);
				firstMove_1.setEnabled(false);
				firstMove_2.setEnabled(false);
				boardPath_Field.setEnabled(false);
				chooseBoardFile.setEnabled(false);
				break;
			case CUSTOM:
				midBoard_1.setSelected(false);
				midBoard_2.setSelected(false);
				midBoard_3.setSelected(true);
				if(startMode == STARTMODE.OPEN){
					boardPath_Field.setEnabled(true);
					chooseBoardFile.setEnabled(true);
				}else{
					boardPath_Field.setEnabled(false);
					chooseBoardFile.setEnabled(false);
				}
				break;
			}
			switch(connectMode){
			case TCP:
				connectMode_1.setSelected(true);
				connectMode_2.setSelected(false);
				connectMode_3.setSelected(false);
				
				searchPath_Field.setEnabled(true);
				chooseSearchFile.setEnabled(true);
				arg_Field.setEnabled(true);
				break;
			case STDIO:
				connectMode_1.setSelected(false);
				connectMode_2.setSelected(true);
				connectMode_3.setSelected(false);
				
				searchPath_Field.setEnabled(true);
				chooseSearchFile.setEnabled(true);
				arg_Field.setEnabled(true);
				break;
			case HUMAN:
				connectMode_1.setSelected(false);
				connectMode_2.setSelected(false);
				connectMode_3.setSelected(true);
				
//				setRedMode_1.setEnabled(true);
//				setRedMode_2.setEnabled(true);
				
				searchPath_Field.setEnabled(false);
				chooseSearchFile.setEnabled(false);
				arg_Field.setEnabled(false);
				break;
			}
			
			switch(firstMove){
			case NO:
				firstMove_1.setSelected(false);
				firstMove_2.setSelected(true);
				break;
			case YES:
				firstMove_1.setSelected(true);
				firstMove_2.setSelected(false);
				break;
			}
			switch(changeFirstMove){
			case NO:
				changeFirstMove_1.setSelected(false);
				changeFirstMove_2.setSelected(true);
				break;
			case YES:
				changeFirstMove_1.setSelected(true);
				changeFirstMove_2.setSelected(false);
				break;
			}
			switch(timerMode){
			case NONE:
				timerMode_ComboBox.setSelectedIndex(0);
				timeLimit_Field.setEnabled(false);
				onePlyTime_Field.setEnabled(false);
				plyShare_Field.setEnabled(false);
				break;
			case ABSOLUTE:
				timerMode_ComboBox.setSelectedIndex(1);
				timeLimit_Field.setEnabled(false);
				onePlyTime_Field.setEnabled(false);
				plyShare_Field.setEnabled(false);
				break;
			case BYOYMI:
				timerMode_ComboBox.setSelectedIndex(2);
				timeLimit_Field.setEnabled(false);
				onePlyTime_Field.setEnabled(false);
				plyShare_Field.setEnabled(false);
				break;
			case CANDIAN:
				timerMode_ComboBox.setSelectedIndex(3);
				timeLimit_Field.setEnabled(false);
				onePlyTime_Field.setEnabled(false);
				plyShare_Field.setEnabled(false);
				break;
			case CUSTOM:
				timerMode_ComboBox.setSelectedIndex(4);
				if(startMode == STARTMODE.OPEN){
					timeLimit_Field.setEnabled(true);
					onePlyTime_Field.setEnabled(true);
					plyShare_Field.setEnabled(true);
				}else{
					timeLimit_Field.setEnabled(false);
					onePlyTime_Field.setEnabled(false);
					plyShare_Field.setEnabled(false);
				}
				break;
			}
			
		}else if(roomType == ROOMTYPE.CONTEST){
			startMode_1.setEnabled(false);
			startMode_2.setEnabled(false);
			if(startMode == STARTMODE.OPEN){
				startMode_1.setSelected(true);
				startMode_2.setSelected(false);
			}else{
				startMode_1.setSelected(false);
				startMode_2.setSelected(true);
			}
			localServer_CheckBox.setSelected(false);
			localServer_CheckBox.setEnabled(false);
			if(!localServer) {
				roomType_2.setEnabled(true);
			}
			midBoard_1.setEnabled(false);
			midBoard_2.setEnabled(false);
			midBoard_3.setEnabled(false);
			switch(midBoard){
			case NO:
				midBoard_1.setSelected(false);
				midBoard_2.setSelected(true);
				midBoard_3.setSelected(false);
				break;
			case YES:
				midBoard_1.setSelected(true);
				midBoard_2.setSelected(false);
				midBoard_3.setSelected(false);
				break;
			case CUSTOM:
				midBoard_1.setSelected(false);
				midBoard_2.setSelected(false);
				midBoard_3.setSelected(true);
				break;
			}
			switch(connectMode){
			case TCP:
				connectMode_1.setSelected(true);
				connectMode_2.setSelected(false);
				connectMode_3.setSelected(false);
				
				searchPath_Field.setEnabled(true);
				chooseSearchFile.setEnabled(true);
				arg_Field.setEnabled(true);
				break;
			case STDIO:
				connectMode_1.setSelected(false);
				connectMode_2.setSelected(true);
				connectMode_3.setSelected(false);
				
				searchPath_Field.setEnabled(true);
				chooseSearchFile.setEnabled(true);
				arg_Field.setEnabled(true);
				break;
			case HUMAN:
				connectMode_1.setSelected(false);
				connectMode_2.setSelected(false);
				connectMode_3.setSelected(true);
				
				searchPath_Field.setEnabled(false);
				chooseSearchFile.setEnabled(false);
				arg_Field.setEnabled(false);
				break;
			}
			firstMove_1.setEnabled(false);
			firstMove_2.setEnabled(false);
			switch(firstMove){
			case NO:
				firstMove_1.setSelected(false);
				firstMove_2.setSelected(true);
				break;
			case YES:
				firstMove_1.setSelected(true);
				firstMove_2.setSelected(false);
				break;
			}
			changeFirstMove_1.setEnabled(false);
			changeFirstMove_2.setEnabled(false);
			switch(changeFirstMove){
			case NO:
				changeFirstMove_1.setSelected(false);
				changeFirstMove_2.setSelected(true);
				break;
			case YES:
				changeFirstMove_1.setSelected(true);
				changeFirstMove_2.setSelected(false);
				break;
			}
			ip_Field.setEnabled(true);
			timerMode_ComboBox.setEnabled(false);
			switch(timerMode){
			case NONE:
				timerMode_ComboBox.setSelectedIndex(0);
				break;
			case ABSOLUTE:
				timerMode_ComboBox.setSelectedIndex(1);
				break;
			case BYOYMI:
				timerMode_ComboBox.setSelectedIndex(2);
				break;
			case CANDIAN:
				timerMode_ComboBox.setSelectedIndex(3);
				break;
			case CUSTOM:
				timerMode_ComboBox.setSelectedIndex(4);
				break;
			}
			timeLimit_Field.setEnabled(false);
			onePlyTime_Field.setEnabled(false);
			plyShare_Field.setEnabled(false);
			repeatTime_Field.setEnabled(false);
			MaxPly_Field.setEnabled(false);
			boardPath_Field.setEnabled(false);
			chooseBoardFile.setEnabled(false);
//			setRedMode_1.setEnabled(false);
//			setRedMode_2.setEnabled(false);
		}
		
		repeatTime_Field.setText(Integer.toString(repeatTime));
		timeLimit_Field.setText(Integer.toString(timeLimit));
		onePlyTime_Field.setText(Integer.toString(onePlyTime));
		plyShare_Field.setText(Integer.toString(plyShare));
		MaxPly_Field.setText(Integer.toString(MaxPly));
		ip_Field.setText(ip);
		boardPath_Field.setText(boardPath);
		searchPath_Field.setText(searchPath);
		arg_Field.setText(arg);
	}
	
	/**
	 * 將 component 上面的資料讀取進來更新此 class 的紀錄。
	 * 
	 * @throws IOException 寫入檔案發生錯誤。
	 */
	public void setInitSetting() throws IOException{
		
		// account
		account = account_Field.getText();
		// password
		password = password_Field.getText();
		// roomType
		if(roomType_1.isSelected()){
			roomType = ROOMTYPE.GENERAL;
		}else if(roomType_2.isSelected()){
			roomType = ROOMTYPE.CONTEST;
		}
		// startMode
		if(startMode_1.isSelected()){
			startMode = STARTMODE.OPEN;
		}else if(startMode_2.isSelected()){
			startMode = STARTMODE.ENTER;
		}
		localServer = localServer_CheckBox.isSelected();
		// midBoard
		if(midBoard_1.isSelected()){
			midBoard = MIDBOARD.YES;
		}else if(midBoard_2.isSelected()){
			midBoard = MIDBOARD.NO;
		}else if(midBoard_3.isSelected()){
			midBoard = MIDBOARD.CUSTOM;
		}
		// connectMode
		if(connectMode_1.isSelected()){
			connectMode = CONNECTMODE.TCP;
		}else if(connectMode_2.isSelected()){
			connectMode = CONNECTMODE.STDIO;
		}else if(connectMode_3.isSelected()){
			connectMode = CONNECTMODE.HUMAN;
		}
		// repeatTime
		try{
			repeatTime = Integer.parseInt(repeatTime_Field.getText());
		}catch(Exception ignore){ /* if parseInt cause ERROR, then ignore this field */ }
		// firstMove
		if(firstMove_1.isSelected()){
			firstMove = FIRSTMOVE.YES;
		}else if(firstMove_2.isSelected()){
			firstMove = FIRSTMOVE.NO;
		}
		// changeFirstMove
		if(changeFirstMove_1.isSelected()){
			changeFirstMove = CHANGEFIRSTMOVE.YES;
		}else if(changeFirstMove_2.isSelected()){
			changeFirstMove = CHANGEFIRSTMOVE.NO;
		}
		// timerMode
		switch(timerMode_ComboBox.getSelectedIndex()){
		case 0:
			timerMode = TIMERMODE.NONE;
			break;
		case 1:
			timerMode = TIMERMODE.ABSOLUTE;
			break;
		case 2:
			timerMode = TIMERMODE.BYOYMI;
			break;
		case 3: 
			timerMode = TIMERMODE.CANDIAN;
			break;
		case 4:
			timerMode = TIMERMODE.CUSTOM;
			break;
		}
		// timeLimit
		try{
			timeLimit = Integer.parseInt(timeLimit_Field.getText());
		}catch(Exception ignore){ /* if parseInt cause ERROR, then ignore this field */ }
		// onePlyTime
		try{
			onePlyTime = Integer.parseInt(onePlyTime_Field.getText());
		}catch(Exception ignore){ /* if parseInt cause ERROR, then ignore this field */ }
		// plyShare
		try{
			plyShare = Integer.parseInt(plyShare_Field.getText());
		}catch(Exception ignore){ /* if parseInt cause ERROR, then ignore this field */ }
		// MaxPly
		try{
			MaxPly = Integer.parseInt(MaxPly_Field.getText());
		}catch(Exception ignore){ /* if parseInt cause ERROR, then ignore this field */ }
		// ip
		ip = ip_Field.getText();
		// boardPath
		boardPath = boardPath_Field.getText();
		// searchPath
		searchPath = searchPath_Field.getText();
		// arg
		arg = arg_Field.getText();
		// setRedMode
//		if(setRedMode_1.isSelected()){
//			setRedMode = SETREDMODE.CUSTOM;
//		}else if(setRedMode_2.isSelected()){
//			setRedMode = SETREDMODE.RANDOM;
//		}
		
		writeSettingFile();
	}
	
	/**
	 * {@inheritDoc}
	 * CLI mode 才會用到的，顯示目前各項遊戲參數。
	 */
	@Override
	public void show() {
		Message.printCLI("Account: %s", account);
		Message.printCLI("Password: %s", password);
		Message.printCLI("RoomType: %s", roomType);
		Message.printCLI("Start Mode: %s", startMode.name());
		Message.printCLI("LocalServer: %s", (localServer ? "Yes" : "No" ));
		Message.printCLI("MidBoard: %s", midBoard.name());
		Message.printCLI("Connect Mode: %s", connectMode.name());
		Message.printCLI("Repeat Times: %d", repeatTime);
		Message.printCLI("Host First Move: %s", firstMove.name());
		Message.printCLI("Change First Move: %s", changeFirstMove.name());
		Message.printCLI("Server IP: %s", ip);
		Message.printCLI("Timer Mode: %s", timerMode.name());
		Message.printCLI("Time Limit: %d", timeLimit);
		Message.printCLI("One Ply Time: %d", onePlyTime);
		Message.printCLI("Ply Share: %d", plyShare);
		Message.printCLI("Max Plies: %d", MaxPly);
		Message.printCLI("Board Path: %s", boardPath);
		Message.printCLI("Search Path: %s", searchPath);
		Message.printCLI("Search Arg: %s", arg);
//		Message.printCLI("Opening Mode: %s", setRedMode.name());
	}
	
	/**
	 * {@inheritDoc}
	 * Client 用來取得 CLI mode 時的 menu。
	 * 可以參考 https://github.com/nathanielove/Java-Console-View。
	 * @param roomSetting 房間設定，用來選擇 Human mode 時，直接更改盤面設定。
	 * @return 每個選項的 AbstractView 用 List 裝起來。
	 */
	@Override
	public List<AbstractView> getMenuItems(Room_SettingFile roomSetting) {
		List<AbstractView> items = new ArrayList<>();
		
		ViewConfig viewConfig = new ViewConfig.Builder()
			 	.setActionSuccessfulMessage("=== Successful ===")
			 	.build();
		
		// Account
		items.add(new ActionView("*** Account ***", 
				String.format("Account [%s]", GST_SettingFile.this.account), 
				viewConfig){
			@Override
			public void executeCustomAction() {
				// get input
				String account = this.prompt("Please input new value: ", String.class);
				// update value
				GST_SettingFile.this.account = account;
				// store value
				try {
					writeSettingFile();
				} catch (IOException e) {
					Message.printError(ExceptionUtils.getStackTrace(e));
				}
				this.actionSuccessful();
			}
			
			@Override
			public void onBack(){
				// change name
				this.setNameInParentMenu(String.format("Account [%s]", account));
			}
		});
		
		// Password
		items.add(new ActionView("*** Password ***", 
				String.format("Password [%s]", GST_SettingFile.this.password), 
				viewConfig){
			@Override
			public void executeCustomAction() {
				// get input
				String password = this.prompt("Please input new value: ", String.class);
				// update value
				GST_SettingFile.this.password = password;
				// store value
				try {
					writeSettingFile();
				} catch (IOException e) {
					Message.printError(ExceptionUtils.getStackTrace(e));
				}
				this.actionSuccessful();
			}
			
			@Override
			public void onBack(){
				// change name
				this.setNameInParentMenu(String.format("Password [%s]", password));
			}
		});
		
		// Room Type
		MenuView room_type_view = new MenuView("*** Room Type ***", 
				String.format("Room Type [%s]", StringUtils.capitalize(GST_SettingFile.this.roomType.name().toLowerCase())), 
				viewConfig){
			@Override
			public void onBack(){
				// change name
				this.setNameInParentMenu(String.format("Room Type [%s]", StringUtils.capitalize(GST_SettingFile.this.roomType.name().toLowerCase())));
			}
		};
		items.add(room_type_view);
		for(ROOMTYPE type : ROOMTYPE.values()){
			if((GST_SettingFile.this.startMode == STARTMODE.ENTER ||
					!GST_SettingFile.this.localServer) && 
					type == ROOMTYPE.OPENING){ continue; }
			room_type_view.addMenuItem(new DynActionView("", StringUtils.capitalize(type.name().toLowerCase()), viewConfig){
				@Override
				public void action() {
					// update value
					GST_SettingFile.this.roomType = type;
					// change setting
					if(type == ROOMTYPE.CONTEST){
						GST_SettingFile.this.localServer = false;
					}
					// store value
					try {
						writeSettingFile();
					} catch (IOException e) {
						Message.printError(ExceptionUtils.getStackTrace(e));
					}
					this.actionSuccessful();
				}
			});
		}
		
		// Start Mode (only for general room type)
		if(GST_SettingFile.this.roomType == ROOMTYPE.GENERAL ||
				GST_SettingFile.this.roomType == ROOMTYPE.OPENING){
			MenuView start_mode_view = new MenuView("*** Start Mode ***", 
					String.format("Start Mode [%s]", StringUtils.capitalize(GST_SettingFile.this.startMode.name().toLowerCase())), 
					viewConfig){
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(String.format("Start Mode [%s]", StringUtils.capitalize(GST_SettingFile.this.startMode.name().toLowerCase())));
				}
			};
			items.add(start_mode_view);
			
			for(STARTMODE type : STARTMODE.values()){
				if(type == STARTMODE.ENTER && roomType == ROOMTYPE.OPENING){ continue; }
				start_mode_view.addMenuItem(new DynActionView("", StringUtils.capitalize(type.name().toLowerCase()), viewConfig){
					@Override
					public void action() {
						// update value
						GST_SettingFile.this.startMode = type;
						// store value
						try {
							writeSettingFile();
						} catch (IOException e) {
							Message.printError(ExceptionUtils.getStackTrace(e));
						}
						this.actionSuccessful();
					}
				});
			}
		}
		
		// LocalServer (only for general room type)
		if(GST_SettingFile.this.roomType == ROOMTYPE.GENERAL ||
				GST_SettingFile.this.roomType == ROOMTYPE.OPENING){
			MenuView localserver_view = new MenuView("*** LocalServer ***", 
					String.format("LocalServer [%s]", (GST_SettingFile.this.localServer ? "Yes" : "No")), 
					viewConfig){
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(String.format("LocalServer [%s]", (GST_SettingFile.this.localServer ? "Yes" : "No")));
				}
			};
			items.add(localserver_view);
			
			localserver_view.addMenuItem(new DynActionView("", "Yes", viewConfig){
				@Override
				public void action() {
					// update value
					GST_SettingFile.this.localServer = true;
					// change setting
					if(GST_SettingFile.this.midBoard == MIDBOARD.YES){
						GST_SettingFile.this.midBoard = MIDBOARD.NO;
					}
					// store value
					try {
						writeSettingFile();
					} catch (IOException e) {
						Message.printError(ExceptionUtils.getStackTrace(e));
					}
					this.actionSuccessful();
				}
			});
			if(roomType != ROOMTYPE.OPENING){ 
				localserver_view.addMenuItem(new DynActionView("", "No", viewConfig){
					@Override
					public void action() {
						// update value
						GST_SettingFile.this.localServer = false;
						// store value
						try {
							writeSettingFile();
						} catch (IOException e) {
							Message.printError(ExceptionUtils.getStackTrace(e));
						}
						this.actionSuccessful();
					}
				});
			}
		}
		
		// MidBoard (only for open room and general room type)
		if(GST_SettingFile.this.startMode == STARTMODE.OPEN &&
				GST_SettingFile.this.roomType == ROOMTYPE.GENERAL){
			MenuView midboard_view = new MenuView("*** MidBoard ***", 
					String.format("MidBoard [%s]", StringUtils.capitalize(GST_SettingFile.this.midBoard.name().toLowerCase())), 
					viewConfig){
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(StringUtils.capitalize(GST_SettingFile.this.midBoard.name().toLowerCase()));
				}
			};
			items.add(midboard_view);
			
			// MidBoard will not be allowed during using localServer
			MIDBOARD[] local_version_midboard = new MIDBOARD[]{MIDBOARD.NO, MIDBOARD.CUSTOM};
			for(MIDBOARD type : (GST_SettingFile.this.localServer ? local_version_midboard : MIDBOARD.values())){
				midboard_view.addMenuItem(new DynActionView("", StringUtils.capitalize(type.name().toLowerCase()), viewConfig){
					@Override
					public void action() {
						// update value
						GST_SettingFile.this.midBoard = type;
						// store value
						try {
							writeSettingFile();
						} catch (IOException e) {
							Message.printError(ExceptionUtils.getStackTrace(e));
						}
						this.actionSuccessful();
					}
				});
			}
		}
		
		// Connect Mode
		MenuView connect_mode_view = new MenuView("*** Connect Mode ***", 
				String.format("Connect Mode [%s]", StringUtils.capitalize(GST_SettingFile.this.connectMode.name().toLowerCase())), 
				viewConfig){
			@Override
			public void onBack(){
				// change name
				this.setNameInParentMenu(String.format("Connect Mode [%s]", StringUtils.capitalize(GST_SettingFile.this.connectMode.name().toLowerCase())));
			}
		};
		items.add(connect_mode_view);
		
		for(CONNECTMODE type : GST_SettingFile.CONNECTMODE.values()){
			connect_mode_view.addMenuItem(new DynActionView("", StringUtils.capitalize(type.name().toLowerCase()), viewConfig){
				@Override
				public void action() {
					// update value
					GST_SettingFile.this.connectMode = type;
					// modify the board open type
					if(type == GST_SettingFile.CONNECTMODE.HUMAN){
						roomSetting.boardOpenType = Room_SettingFile.BoardOpenType.REMOTE;
					}
					// store value
					try {
						roomSetting.writeSettingFile();
						writeSettingFile();
					} catch (IOException e) {
						Message.printError(ExceptionUtils.getStackTrace(e));
					}
					this.actionSuccessful();
				}
			});
		}
		
		// Repeat Time (only for open room and general room type)
		if(GST_SettingFile.this.startMode == STARTMODE.OPEN && 
				(GST_SettingFile.this.roomType == ROOMTYPE.GENERAL ||
				GST_SettingFile.this.roomType == ROOMTYPE.OPENING)){
			items.add(new ActionView("*** Repeat Times ***", 
					String.format("Repeat Times [%d]", GST_SettingFile.this.repeatTime), 
					viewConfig){
				@Override
				public void executeCustomAction() {
					// get input
					Integer repeatTime = this.prompt("Please input new value: ", Integer.class);
					// update value
					GST_SettingFile.this.repeatTime = repeatTime;
					// store value
					try {
						writeSettingFile();
					} catch (IOException e) {
						Message.printError(ExceptionUtils.getStackTrace(e));
					}
					this.actionSuccessful();
				}
				
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(String.format("Repeat Times [%d]", GST_SettingFile.this.repeatTime));
				}
			});
		}
		
		// Host First Move (only for open room and general room type)
		if(GST_SettingFile.this.startMode == STARTMODE.OPEN && 
				((GST_SettingFile.this.roomType == ROOMTYPE.GENERAL &&
				GST_SettingFile.this.midBoard != MIDBOARD.YES) ||
				(GST_SettingFile.this.roomType == ROOMTYPE.OPENING))){
			MenuView host_first_move_view = new MenuView("*** Host First Move ***", 
					String.format("Host First Move [%s]", StringUtils.capitalize(GST_SettingFile.this.firstMove.name().toLowerCase())), 
					viewConfig){
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(StringUtils.capitalize(GST_SettingFile.this.firstMove.name().toLowerCase()));
				}
			};
			items.add(host_first_move_view);
			
			for(FIRSTMOVE type : FIRSTMOVE.values()){
				host_first_move_view.addMenuItem(new DynActionView("", StringUtils.capitalize(type.name().toLowerCase()), viewConfig){
					@Override
					public void action() {
						// update value
						GST_SettingFile.this.firstMove = type;
						// store value
						try {
							writeSettingFile();
						} catch (IOException e) {
							Message.printError(ExceptionUtils.getStackTrace(e));
						}
						this.actionSuccessful();
					}
				});
			}
		}
		
		// Change First Move (only for open room and general room type)
		if(GST_SettingFile.this.startMode == STARTMODE.OPEN && 
				(GST_SettingFile.this.roomType == ROOMTYPE.GENERAL ||
				GST_SettingFile.this.roomType == ROOMTYPE.OPENING)){
			MenuView change_first_move_view = new MenuView("*** Change First Move ***", 
					String.format("Change First Move [%s]", StringUtils.capitalize(GST_SettingFile.this.changeFirstMove.name().toLowerCase())), 
					viewConfig){
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(String.format("Change First Move [%s]", StringUtils.capitalize(GST_SettingFile.this.changeFirstMove.name().toLowerCase())));
				}
			};
			items.add(change_first_move_view);
			
			for(CHANGEFIRSTMOVE type : CHANGEFIRSTMOVE.values()){
				change_first_move_view.addMenuItem(new DynActionView("", StringUtils.capitalize(type.name().toLowerCase()), viewConfig){
					@Override
					public void action() {
						// update value
						GST_SettingFile.this.changeFirstMove = type;
						// store value
						try {
							writeSettingFile();
						} catch (IOException e) {
							Message.printError(ExceptionUtils.getStackTrace(e));
						}
						this.actionSuccessful();
					}
				});
			}
		}
		
		// Server IP (only for online mode)
		if(!GST_SettingFile.this.localServer){
			items.add(new ActionView("*** Server IP ***", 
					String.format("Server IP [%s]", GST_SettingFile.this.ip), 
					viewConfig){
				@Override
				public void executeCustomAction() {
					Validator<String> bidValidator = new Validator<String>(){
						@Override
						public boolean isValid(String input) {
							// judge if it's a valid IP address (IPv4)
							return input.matches("^((?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?))*$");
						}
					};
					// get input
					String ip = this.prompt("Please input new value: ", String.class, bidValidator);
					// update value
					GST_SettingFile.this.ip = ip;
					// store value
					try {
						writeSettingFile();
					} catch (IOException e) {
						Message.printError(ExceptionUtils.getStackTrace(e));
					}
					this.actionSuccessful();
				}
				
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(String.format("Server IP [%s]", GST_SettingFile.this.ip));
				}
			});
		}
		
		// Timer Mode (only for open room and general room type)
		if(GST_SettingFile.this.startMode == STARTMODE.OPEN && 
				(GST_SettingFile.this.roomType == ROOMTYPE.GENERAL ||
				GST_SettingFile.this.roomType == ROOMTYPE.OPENING)){
			MenuView timer_mode_view = new MenuView("*** Timer Mode ***", 
					String.format("Timer Mode [%s (%d/%d/%d)]", 
							StringUtils.capitalize(GST_SettingFile.this.timerMode.name().toLowerCase()),
							GST_SettingFile.this.timeLimit,
							GST_SettingFile.this.onePlyTime,
							GST_SettingFile.this.plyShare), 
					viewConfig){
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(
							String.format("Timer Mode [%s (%d/%d/%d)]", 
									StringUtils.capitalize(GST_SettingFile.this.timerMode.name().toLowerCase()),
									GST_SettingFile.this.timeLimit,
									GST_SettingFile.this.onePlyTime,
									GST_SettingFile.this.plyShare));
				}
			};
			items.add(timer_mode_view);
			
			for(TIMERMODE type : TIMERMODE.values()){
				timer_mode_view.addMenuItem(new DynActionView("", StringUtils.capitalize(type.name().toLowerCase()), viewConfig){
					@Override
					public void action() {
						// update value
						GST_SettingFile.this.timerMode = type;
						// change setting
						switch(GST_SettingFile.this.timerMode){
						case ABSOLUTE:
							GST_SettingFile.this.timeLimit = 900;
							GST_SettingFile.this.onePlyTime = 0;
							GST_SettingFile.this.plyShare = 0;
							break;
						case BYOYMI:
							GST_SettingFile.this.timeLimit = 900;
							GST_SettingFile.this.onePlyTime = 5;
							GST_SettingFile.this.plyShare = 1;
							break;
						case CANDIAN:
							GST_SettingFile.this.timeLimit = 900;
							GST_SettingFile.this.onePlyTime = 30;
							GST_SettingFile.this.plyShare = 3;
							break;
						case CUSTOM:
							break;
						case NONE:
							GST_SettingFile.this.timeLimit = 0;
							GST_SettingFile.this.onePlyTime = 0;
							GST_SettingFile.this.plyShare = 0;
							break;
						}
						// store value
						try {
							writeSettingFile();
						} catch (IOException e) {
							Message.printError(ExceptionUtils.getStackTrace(e));
						}
						this.actionSuccessful();
					}
				});
			}
		}
		
		// Time Limit (only for open room, use custom timer mode and general room type)
		if(GST_SettingFile.this.startMode == STARTMODE.OPEN &&
				GST_SettingFile.this.timerMode == TIMERMODE.CUSTOM && 
				(GST_SettingFile.this.roomType == ROOMTYPE.GENERAL ||
				GST_SettingFile.this.roomType == ROOMTYPE.OPENING)){
			items.add(new ActionView("*** Time Limit ***", 
					String.format("Time Limit [%d]", GST_SettingFile.this.timeLimit), 
					viewConfig){
				@Override
				public void executeCustomAction() {
					// get input
					Integer timelimit = this.prompt("Please input new value: ", Integer.class);
					// update value
					GST_SettingFile.this.timeLimit = timelimit;
					// store value
					try {
						writeSettingFile();
					} catch (IOException e) {
						Message.printError(ExceptionUtils.getStackTrace(e));
					}
					this.actionSuccessful();
				}
				
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(String.format("Time Limit [%d]", GST_SettingFile.this.timeLimit));
				}
			});
		}
		
		// One Ply Time (only for open room, use custom timer mode and general room type)
		if(GST_SettingFile.this.startMode == STARTMODE.OPEN &&
				GST_SettingFile.this.timerMode == TIMERMODE.CUSTOM && 
				(GST_SettingFile.this.roomType == ROOMTYPE.GENERAL ||
				GST_SettingFile.this.roomType == ROOMTYPE.OPENING)){
			items.add(new ActionView("*** One Ply Time ***", 
					String.format("One Ply Time [%d]", GST_SettingFile.this.onePlyTime), 
					viewConfig){
				@Override
				public void executeCustomAction() {
					// get input
					Integer oneplytime = this.prompt("Please input new value: ", Integer.class);
					// update value
					GST_SettingFile.this.onePlyTime = oneplytime;
					// store value
					try {
						writeSettingFile();
					} catch (IOException e) {
						Message.printError(ExceptionUtils.getStackTrace(e));
					}
					this.actionSuccessful();
				}
				
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(String.format("One Ply Time [%d]", GST_SettingFile.this.onePlyTime));
				}
			});
		}
		
		// Ply Share (only for open room, use custom timer mode and general room type)
		if(GST_SettingFile.this.startMode == STARTMODE.OPEN &&
				GST_SettingFile.this.timerMode == TIMERMODE.CUSTOM && 
				(GST_SettingFile.this.roomType == ROOMTYPE.GENERAL ||
				GST_SettingFile.this.roomType == ROOMTYPE.OPENING)){
			items.add(new ActionView("*** Ply Share ***", 
					String.format("Ply Share [%d]", GST_SettingFile.this.plyShare), 
					viewConfig){
				@Override
				public void executeCustomAction() {
					// get input
					Integer plyshare = this.prompt("Please input new value: ", Integer.class);
					// update value
					GST_SettingFile.this.plyShare = plyshare;
					// store value
					try {
						writeSettingFile();
					} catch (IOException e) {
						Message.printError(ExceptionUtils.getStackTrace(e));
					}
					this.actionSuccessful();
				}
				
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(String.format("Ply Share [%d]", GST_SettingFile.this.plyShare));
				}
			});
		}
		
		// Long Catch (only for open room and general room type)
//		if(GST_SettingFile.this.startMode == STARTMODE.OPEN && 
//				(GST_SettingFile.this.roomType == ROOMTYPE.GENERAL ||
//				GST_SettingFile.this.roomType == ROOMTYPE.OPENING)){
//			items.add(new ActionView("*** Long Catch ***", 
//					String.format("Long Catch [%d]", GST_SettingFile.this.longCatch), 
//					viewConfig){
//				@Override
//				public void executeCustomAction() {
//					// get input
//					Integer longcatch = this.prompt("Please input new value: ", Integer.class);
//					// update value
//					GST_SettingFile.this.longCatch = longcatch;
//					// store value
//					try {
//						writeSettingFile();
//					} catch (IOException e) {
//						Message.printError(ExceptionUtils.getStackTrace(e));
//					}
//					this.actionSuccessful();
//				}
//				
//				@Override
//				public void onBack(){
//					// change name
//					this.setNameInParentMenu(String.format("Long Catch [%d]", GST_SettingFile.this.longCatch));
//				}
//			});
//		}
		
		// Max Plies (only for open room and general room type)
		if(GST_SettingFile.this.startMode == STARTMODE.OPEN && 
				(GST_SettingFile.this.roomType == ROOMTYPE.GENERAL ||
				GST_SettingFile.this.roomType == ROOMTYPE.OPENING)){
			items.add(new ActionView("*** Max Plies ***", 
					String.format("Max Plies [%d]", GST_SettingFile.this.MaxPly), 
					viewConfig){
				@Override
				public void executeCustomAction() {
					// get input
					Integer maxPly = this.prompt("Please input new value: ", Integer.class);
					// update value
					GST_SettingFile.this.MaxPly = maxPly;
					// store value
					try {
						writeSettingFile();
					} catch (IOException e) {
						Message.printError(ExceptionUtils.getStackTrace(e));
					}
					this.actionSuccessful();
				}
				
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(String.format("Max Plies [%d]", GST_SettingFile.this.MaxPly));
				}
			});
		}
		
		// Board Path (only for open room, use custom midBoard mode and general room type)
		if(GST_SettingFile.this.startMode == STARTMODE.OPEN &&
				GST_SettingFile.this.midBoard == MIDBOARD.CUSTOM && 
				GST_SettingFile.this.roomType == ROOMTYPE.GENERAL || 
				GST_SettingFile.this.roomType == ROOMTYPE.OPENING){
			items.add(new ActionView("*** Board Path ***", 
					String.format("Board Path [%s]", GST_SettingFile.this.boardPath), 
					viewConfig){
				@Override
				public void executeCustomAction() {
					// get input
					String baordpath = this.prompt("Please input new value: ", String.class);
					// update value
					GST_SettingFile.this.boardPath = baordpath;
					// store value
					try {
						writeSettingFile();
					} catch (IOException e) {
						Message.printError(ExceptionUtils.getStackTrace(e));
					}
					this.actionSuccessful();
				}
				
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(String.format("Board Path [%s]", GST_SettingFile.this.boardPath));
				}
			});
		}
		
		// Search Path (use except Human mode)
		if(GST_SettingFile.this.connectMode != GST_SettingFile.CONNECTMODE.HUMAN){
			items.add(new ActionView("*** Search Path ***", 
					String.format("Search Path [%s]", GST_SettingFile.this.searchPath), 
					viewConfig){
				@Override
				public void executeCustomAction() {
					// get input
					String searchpath = this.prompt("Please input new value: ", String.class);
					// update value
					GST_SettingFile.this.searchPath = searchpath;
					// store value
					try {
						writeSettingFile();
					} catch (IOException e) {
						Message.printError(ExceptionUtils.getStackTrace(e));
					}
					this.actionSuccessful();
				}
				
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(String.format("Search Path [%s]", GST_SettingFile.this.searchPath));
				}
			});
		}
		
		// Search Arg (use except Human mode)
		if(GST_SettingFile.this.connectMode != GST_SettingFile.CONNECTMODE.HUMAN){
			items.add(new ActionView("*** Search Arg ***", 
					String.format("Search Arg [%s]", GST_SettingFile.this.arg), 
					viewConfig){
				@Override
				public void executeCustomAction() {
					// get input
					String arg = this.prompt("Please input new value: ", String.class);
					// update value
					GST_SettingFile.this.arg = arg;
					// store value
					try {
						writeSettingFile();
					} catch (IOException e) {
						Message.printError(ExceptionUtils.getStackTrace(e));
					}
					this.actionSuccessful();
				}
				
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(String.format("Search Arg [%s]", GST_SettingFile.this.arg));
				}
			});
		}
		/*
		// opening mode
		if(GST_SettingFile.this.roomType == SettingFile.ROOMTYPE.OPENING){
			MenuView opening_mode_view = new MenuView("*** Opening Mode ***", 
					String.format("Opening Mode [%s]", StringUtils.capitalize(GST_SettingFile.this.setRedMode.name().toLowerCase())), 
					viewConfig){
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(String.format("Opening Mode [%s]", StringUtils.capitalize(GST_SettingFile.this.setRedMode.name().toLowerCase())));
				}
			};
			items.add(opening_mode_view);
			for(SETREDMODE type : SETREDMODE.values()){
				opening_mode_view.addMenuItem(new DynActionView("", StringUtils.capitalize(type.name().toLowerCase()), viewConfig){
					@Override
					public void action() {
						// update value
						GST_SettingFile.this.setRedMode = type;
						// store value
						try {
							writeSettingFile();
						} catch (IOException e) {
							Message.printError(ExceptionUtils.getStackTrace(e));
						}
						this.actionSuccessful();
					}
				});
			}
		}
		
		// timePerOpening
		if(GST_SettingFile.this.roomType == SettingFile.ROOMTYPE.OPENING){
			items.add(new ActionView("*** Time Per Opening ***", 
					String.format("Time Per Opening [%d]", GST_SettingFile.this.timePerOpening), 
					viewConfig){
				@Override
				public void executeCustomAction() {
					// get input
					Integer time = this.prompt("Please input new value: ", Integer.class);
					// update value
					GST_SettingFile.this.timePerOpening = time;
					// store value
					try {
						writeSettingFile();
					} catch (IOException e) {
						Message.printError(ExceptionUtils.getStackTrace(e));
					}
					this.actionSuccessful();
				}
				
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(String.format("Time Per Opening [%d]", GST_SettingFile.this.timePerOpening));
				}
			});
		}
		
		// win score
		if(GST_SettingFile.this.startMode == STARTMODE.OPEN && GST_SettingFile.this.roomType != ROOMTYPE.CONTEST){
			items.add(new ActionView("*** Win Score ***", 
					String.format("Win Score [%f]", GST_SettingFile.this.winScore), 
					viewConfig){
				@Override
				public void executeCustomAction() {
					// get input
					Double value = this.prompt("Please input new value: ", Double.class);
					// update value
					GST_SettingFile.this.winScore = value;
					// store value
					try {
						writeSettingFile();
					} catch (IOException e) {
						Message.printError(ExceptionUtils.getStackTrace(e));
					}
					this.actionSuccessful();
				}
				
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(String.format("Win Score [%f]", GST_SettingFile.this.winScore));
				}
			});
		}
		
		// draw score
		if(GST_SettingFile.this.startMode == STARTMODE.OPEN && GST_SettingFile.this.roomType != ROOMTYPE.CONTEST){
			items.add(new ActionView("*** Draw Score ***", 
					String.format("Draw Score [%f]", GST_SettingFile.this.drawScore), 
					viewConfig){
				@Override
				public void executeCustomAction() {
					// get input
					Double value = this.prompt("Please input new value: ", Double.class);
					// update value
					GST_SettingFile.this.drawScore = value;
					// store value
					try {
						writeSettingFile();
					} catch (IOException e) {
						Message.printError(ExceptionUtils.getStackTrace(e));
					}
					this.actionSuccessful();
				}
				
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(String.format("Draw Score [%f]", GST_SettingFile.this.drawScore));
				}
			});
		}
				
		// lose score
		if(GST_SettingFile.this.startMode == STARTMODE.OPEN && GST_SettingFile.this.roomType != ROOMTYPE.CONTEST){
			items.add(new ActionView("*** Lose Score ***", 
					String.format("Lose Score [%f]", GST_SettingFile.this.loseScore), 
					viewConfig){
				@Override
				public void executeCustomAction() {
					// get input
					Double value = this.prompt("Please input new value: ", Double.class);
					// update value
					GST_SettingFile.this.loseScore = value;
					// store value
					try {
						writeSettingFile();
					} catch (IOException e) {
						Message.printError(ExceptionUtils.getStackTrace(e));
					}
					this.actionSuccessful();
				}
				
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(String.format("Lose Score [%f]", GST_SettingFile.this.loseScore));
				}
			});
		}
		
		// piece score
		if(GST_SettingFile.this.startMode == STARTMODE.OPEN && GST_SettingFile.this.roomType != ROOMTYPE.CONTEST){
			MenuView piece_score_view = new MenuView("*** Piece Score ***", 
					String.format("Piece Score [%s]", StringUtils.join(GST_SettingFile.this.pieceScore, ',')), 
					viewConfig){
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(String.format("Piece Score [%s]", StringUtils.join(GST_SettingFile.this.pieceScore, ',')));
				}
			};
			items.add(piece_score_view);
			final String piece_name[] = {"K", "G", "M", "R", "N", "C", "P", "k", "g", "m", "r", "n", "c", "p"};
			for(int i = 0; i < piece_name.length; ++i){
				final int i_final = i;
				piece_score_view.addMenuItem(new DynActionView(piece_name[i], piece_name[i], viewConfig){
					@Override
					public void action() {
						// get input
						Double value = this.prompt("Please input new value: ", Double.class);
						// update value
						GST_SettingFile.this.pieceScore[i_final] = value;
						// store value
						try {
							writeSettingFile();
						} catch (IOException e) {
							Message.printError(ExceptionUtils.getStackTrace(e));
						}
						this.actionSuccessful();
					}
				});
			}
		}
		
		// bonus
		if(GST_SettingFile.this.startMode == STARTMODE.OPEN && GST_SettingFile.this.roomType != ROOMTYPE.CONTEST){
			items.add(new ActionView("*** Bonus Score ***", 
					String.format("Bonus Score [%f]", GST_SettingFile.this.bonus), 
					viewConfig){
				@Override
				public void executeCustomAction() {
					// get input
					Double value = this.prompt("Please input new value: ", Double.class);
					// update value
					GST_SettingFile.this.bonus = value;
					// store value
					try {
						writeSettingFile();
					} catch (IOException e) {
						Message.printError(ExceptionUtils.getStackTrace(e));
					}
					this.actionSuccessful();
				}
				
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(String.format("Bonus Score [%f]", GST_SettingFile.this.bonus));
				}
			});
		}
		
		// bonus limit
		if(GST_SettingFile.this.startMode == STARTMODE.OPEN && GST_SettingFile.this.roomType != ROOMTYPE.CONTEST){
			items.add(new ActionView("*** Bonus Limit ***", 
					String.format("Bonus Limit [%f]", GST_SettingFile.this.bonusLimit), 
					viewConfig){
				@Override
				public void executeCustomAction() {
					// get input
					Double value = this.prompt("Please input new value: ", Double.class);
					// update value
					GST_SettingFile.this.bonusLimit = value;
					// store value
					try {
						writeSettingFile();
					} catch (IOException e) {
						Message.printError(ExceptionUtils.getStackTrace(e));
					}
					this.actionSuccessful();
				}
				
				@Override
				public void onBack(){
					// change name
					this.setNameInParentMenu(String.format("Bonus Limit [%f]", GST_SettingFile.this.bonusLimit));
				}
			});
		}
		*/
		return items;
	}
	
	/**
	 * {@inheritDoc}
	 * Client 用來取得 Contest CLI mode 時的 menu。
	 * 可以參考 https://github.com/nathanielove/Java-Console-View。
	 * @return 每個選項的 AbstractView 用 List 裝起來。
	 */
	@Override
	public List<AbstractView> getContestMenuItems(Room_SettingFile roomSetting) {
		List<AbstractView> items = new ArrayList<>();
		
		ViewConfig viewConfig = new ViewConfig.Builder()
			 	.setActionSuccessfulMessage("=== Successful ===")
			 	.build();
		// Connect Mode
		MenuView connect_mode_view = new MenuView("*** Connect Mode ***", 
				String.format("Connect Mode [%s]", StringUtils.capitalize(GST_SettingFile.this.connectMode.name().toLowerCase())), 
				viewConfig){
			@Override
			public void onBack(){
				// change name
				this.setNameInParentMenu(String.format("Connect Mode [%s]", StringUtils.capitalize(GST_SettingFile.this.connectMode.name().toLowerCase())));
			}
		};
		items.add(connect_mode_view);
		
		for(CONNECTMODE type : GST_SettingFile.CONNECTMODE.values()){
			connect_mode_view.addMenuItem(new DynActionView("", StringUtils.capitalize(type.name().toLowerCase()), viewConfig){
				@Override
				public void action() {
					// update value
					GST_SettingFile.this.connectMode = type;
					// modify the board open type
					if(type == GST_SettingFile.CONNECTMODE.HUMAN){
						roomSetting.boardOpenType = Room_SettingFile.BoardOpenType.REMOTE;
					}
					// store value
					try {
						roomSetting.writeSettingFile();
						writeSettingFile();
					} catch (IOException e) {
						Message.printError(ExceptionUtils.getStackTrace(e));
					}
					this.actionSuccessful();
				}
			});
		}
		
		// Search Path
		items.add(new ActionView("*** Search Path ***", 
				String.format("Search Path [%s]", GST_SettingFile.this.searchPath), 
				viewConfig){
			@Override
			public void executeCustomAction() {
				// get input
				String searchpath = this.prompt("Please input new value: ", String.class);
				// update value
				GST_SettingFile.this.searchPath = searchpath;
				// store value
				try {
					writeSettingFile();
				} catch (IOException e) {
					Message.printError(ExceptionUtils.getStackTrace(e));
				}
				this.actionSuccessful();
			}
			
			@Override
			public void onBack(){
				// change name
				this.setNameInParentMenu(String.format("Search Path [%s]", GST_SettingFile.this.searchPath));
			}
		});
		
		// Search Arg
		items.add(new ActionView("*** Search Arg ***", 
				String.format("Search Arg [%s]", GST_SettingFile.this.arg), 
				viewConfig){
			@Override
			public void executeCustomAction() {
				// get input
				String arg = this.prompt("Please input new value: ", String.class);
				// update value
				GST_SettingFile.this.arg = arg;
				// store value
				try {
					writeSettingFile();
				} catch (IOException e) {
					Message.printError(ExceptionUtils.getStackTrace(e));
				}
				this.actionSuccessful();
			}
			
			@Override
			public void onBack(){
				// change name
				this.setNameInParentMenu(String.format("Search Arg [%s]", GST_SettingFile.this.arg));
			}
		});
		
		return items;
	}
}
