package gst;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.nio.file.Paths;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.ButtonGroup;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;
import javax.swing.filechooser.FileNameExtensionFilter;

import client.HintTextField;
import client.ProgramPath;
import client.Room_SettingFile;
import client.SettingFile;
import client.UI;
import client.Room_SettingFile.BoardOpenType;
import net.miginfocom.swing.MigLayout;

/**
 * 遊戲設定頁面的配置。
 * @author ldslds449
 * @see client.UI
 */
public class GST_UI implements UI {
	
	private final ButtonGroup RoomType_Gp = new ButtonGroup();
	private final ButtonGroup StartMode_Gp = new ButtonGroup();
	private final ButtonGroup MidBoard_Gp = new ButtonGroup();
	private final ButtonGroup FirstMove_Gp = new ButtonGroup();
	private final ButtonGroup ChangeFirstMove_Gp = new ButtonGroup();
	private final ButtonGroup ConnectMode_Gp = new ButtonGroup();
//	private final ButtonGroup SetRedMode_Gp = new ButtonGroup();
	
	private JTextField Password_TextField;
	private JTextField RepeatTime_TextField;
	private JTextField TimeLimit_TextField;
	private JTextField OnePlyTime_TextField;
	private JTextField PlyShare_TextField;
	private JTextField MaxPly_TextField;
	private JTextField IP_TextField;
	private JTextField BoardPath_TextField;
	private JTextField AIPath_TextField;
	private JTextField Account_TextField;
	
	private JPanel GST_Panel;
	private JRadioButton RoomType_1;
	private JRadioButton RoomType_2;
	private JRadioButton StartMode_1;
	private JRadioButton StartMode_2;
	private JCheckBox LocalServer_CheckBox;
	
	private JFileChooser fileChooser = new JFileChooser();
	private SettingFile setting;
	
	private JComboBox<String> TimerMode_ComboBox;
	private JTextField AIArg_TextField;
	
	private ResourceBundle bundle;
	private JLabel Account, TimerMode, Password, TimeLimit, RoomType, OnePlyTime, StartMode;
	private JLabel MidBoard, RepeatTime, ConnectMode, FirstMove, MaxPly, ChangeFirstMove;
	private JLabel BoardPath, AIPath, IP, PlyShare, AIArg;
	private JButton BoardPath_Button, AIPath_button;
	private JRadioButton MidBoard_1, MidBoard_2, MidBoard_3;
	private JRadioButton ConnectMode_1, ConnectMode_2, ConnectMode_3;
	private JRadioButton FirstMove_1, FirstMove_2;
	private JRadioButton ChangeFirstMove_1, ChangeFirstMove_2;
	/**
	 * constructor，設定所有的欄位、按鈕等。
	 */
	public GST_UI(){
		GST_Panel = new JPanel();
		GST_Panel.setSize(new Dimension(920, 550));
		GST_Panel.setLayout(new MigLayout("", "[110.00][80.00][80.00][110.00][70.00,grow][15.00][120.00][grow][50.00][70.00]", "[40.00,grow][40.00,grow][40.00,grow][40.00,grow][40.00,grow][40.00,grow][40.00,grow][40.00,grow][40.00,grow][40.00,grow]"));
		
		// 載入默認語言 (英文)
        bundle = ResourceBundle.getBundle("geister", Locale.ENGLISH);
		
		Account = new JLabel(bundle.getString("label.account"));
		GST_Panel.add(Account, "cell 0 0,alignx right");
		Account.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		
		Account_TextField = new JTextField();
		GST_Panel.add(Account_TextField, "cell 1 0 2 1,grow");
		Account_TextField.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		Account_TextField.setColumns(10);
		
		TimerMode = new JLabel(bundle.getString("label.timerMode"));
		TimerMode.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(TimerMode, "cell 6 0,alignx right,growy");
		
		TimerMode_ComboBox = new JComboBox<String>();
		TimerMode_ComboBox.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(TimerMode_ComboBox, "cell 7 0,grow");
		
		TimerMode_ComboBox.addItem("None");
		TimerMode_ComboBox.addItem("Absolute");
		TimerMode_ComboBox.addItem("Byoymi");
		TimerMode_ComboBox.addItem("Candian");
		TimerMode_ComboBox.addItem("Custom");
		TimerMode_ComboBox.addItemListener(new ItemListener(){
			@Override
		    public void itemStateChanged(ItemEvent event) {
		       if (event.getStateChange() == ItemEvent.SELECTED) {
		          String item = (String)event.getItem();
		          switch(item){
		          case "None":
		        	  TimeLimit_TextField.setText("0");
		        	  OnePlyTime_TextField.setText("0");
		        	  PlyShare_TextField.setText("0");
		        	  TimeLimit_TextField.setEnabled(false);
		        	  OnePlyTime_TextField.setEnabled(false);
		        	  PlyShare_TextField.setEnabled(false);
		        	  break;
		          case "Absolute":
		        	  TimeLimit_TextField.setText("900");
		        	  OnePlyTime_TextField.setText("0");
		        	  PlyShare_TextField.setText("0");
		        	  TimeLimit_TextField.setEnabled(false);
		        	  OnePlyTime_TextField.setEnabled(false);
		        	  PlyShare_TextField.setEnabled(false);
		        	  break;
		          case "Byoymi":
		        	  TimeLimit_TextField.setText("900");
		        	  OnePlyTime_TextField.setText("5");
		        	  PlyShare_TextField.setText("1");
		        	  TimeLimit_TextField.setEnabled(false);
		        	  OnePlyTime_TextField.setEnabled(false);
		        	  PlyShare_TextField.setEnabled(false);
		        	  break;
		          case "Candian":
		        	  TimeLimit_TextField.setText("900");
		        	  OnePlyTime_TextField.setText("30");
		        	  PlyShare_TextField.setText("3");
		        	  TimeLimit_TextField.setEnabled(false);
		        	  OnePlyTime_TextField.setEnabled(false);
		        	  PlyShare_TextField.setEnabled(false);
		        	  break;
		          case "Custom":
		        	  TimeLimit_TextField.setEnabled(true);
		        	  OnePlyTime_TextField.setEnabled(true);
		        	  PlyShare_TextField.setEnabled(true);
		        	  break;
		          }
		       }
		    }
		});
		
		Password = new JLabel(bundle.getString("label.password"));
		Password.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(Password, "cell 0 1,alignx right,growy");
		
		Password_TextField = new JTextField();
		Password_TextField.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		Password_TextField.setColumns(10);
		GST_Panel.add(Password_TextField, "cell 1 1 2 1,grow");
		
		TimeLimit = new JLabel(bundle.getString("label.timeLimit"));
		TimeLimit.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(TimeLimit, "cell 6 1,alignx right,growy");
		
		TimeLimit_TextField = new JTextField();
		TimeLimit_TextField.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		TimeLimit_TextField.setEnabled(false);
		TimeLimit_TextField.setColumns(10);
		GST_Panel.add(TimeLimit_TextField, "cell 7 1,grow");
		
		JLabel sec = new JLabel("(sec)");
		sec.setHorizontalAlignment(SwingConstants.LEFT);
		sec.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(sec, "cell 8 1,alignx center,growy");
		
		//房間類別：
		RoomType = new JLabel(bundle.getString("label.roomType"));
		RoomType.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(RoomType, "cell 0 2,alignx right,growy");
		
		//一般
		RoomType_1 = new JRadioButton(bundle.getString("radioButton.roomType1"));
		RoomType_1.setSelected(true);
		RoomType_1.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(RoomType_1, "cell 1 2,alignx left,growy");
		RoomType_Gp.add(RoomType_1);
		
		//比賽
		RoomType_2 = new JRadioButton(bundle.getString("radioButton.roomType2"));
		RoomType_2.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(RoomType_2, "cell 2 2,alignx left,growy");
		RoomType_Gp.add(RoomType_2);
		
		
		OnePlyTime = new JLabel(bundle.getString("label.timePerMove"));
		OnePlyTime.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(OnePlyTime, "cell 6 2,alignx right,growy");
		
		OnePlyTime_TextField = new JTextField();
		OnePlyTime_TextField.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		OnePlyTime_TextField.setColumns(10);
		GST_Panel.add(OnePlyTime_TextField, "cell 7 2,grow");
		
		//啟動模式
		StartMode = new JLabel(bundle.getString("label.startMode"));
		StartMode.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(StartMode, "cell 0 3,alignx right,growy");
		
		//開房
		StartMode_1 = new JRadioButton(bundle.getString("radioButton.startMode1"));
		StartMode_1.setSelected(true);
		StartMode_1.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(StartMode_1, "cell 1 3,alignx left,growy");
		StartMode_Gp.add(StartMode_1);
		
		//加房
		StartMode_2 = new JRadioButton(bundle.getString("radioButton.startMode2"));
		StartMode_2.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(StartMode_2, "cell 2 3,alignx left,growy");
		StartMode_Gp.add(StartMode_2);
		
		LocalServer_CheckBox = new JCheckBox(bundle.getString("checkBox.localServer"));
		LocalServer_CheckBox.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(LocalServer_CheckBox, "cell 3 3,alignx left,growy");
		
		LocalServer_CheckBox.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				if(LocalServer_CheckBox.isSelected()){
					IP_TextField.setEnabled(false);
					
					if(MidBoard_1.isSelected()){
						MidBoard_2.setSelected(true);
					}
					MidBoard_1.setEnabled(false);
					RoomType_2.setEnabled(false);
				}else{
					if(StartMode_1.isSelected()){
						MidBoard_1.setEnabled(true);
					}
					IP_TextField.setEnabled(true);
					RoomType_2.setEnabled(true);
				}
			}
		});
		
		MidBoard = new JLabel(bundle.getString("label.midBoard"));
		MidBoard.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(MidBoard, "cell 0 4,alignx right,growy");
		
		MidBoard_1 = new JRadioButton(bundle.getString("radioButton.midBoard1"));
		MidBoard_1.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(MidBoard_1, "cell 1 4,alignx left,growy");
		MidBoard_Gp.add(MidBoard_1);
		
		MidBoard_2 = new JRadioButton(bundle.getString("radioButton.midBoard2"));
		MidBoard_2.setSelected(true);
		MidBoard_2.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(MidBoard_2, "cell 2 4,alignx left,growy");
		MidBoard_Gp.add(MidBoard_2);
		
		MidBoard_3 = new JRadioButton(bundle.getString("radioButton.midBoard3"));
		MidBoard_3.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(MidBoard_3, "cell 3 4,alignx left,growy");
		MidBoard_Gp.add(MidBoard_3);
		
		RepeatTime = new JLabel(bundle.getString("label.repeatTime"));
		RepeatTime.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(RepeatTime, "cell 6 4,alignx right,growy");
		
		RepeatTime_TextField = new JTextField();
		RepeatTime_TextField.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		RepeatTime_TextField.setColumns(10);
		GST_Panel.add(RepeatTime_TextField, "cell 7 4,grow");
		
		ConnectMode = new JLabel(bundle.getString("label.connectMode"));
		ConnectMode.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(ConnectMode, "cell 0 5,alignx right,growy");
		
		ConnectMode_1 = new JRadioButton(bundle.getString("radioButton.connectMode1"));
		ConnectMode_1.setSelected(true);
		ConnectMode_1.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(ConnectMode_1, "cell 1 5,alignx left,growy");
		ConnectMode_Gp.add(ConnectMode_1);
			
		ConnectMode_2 = new JRadioButton(bundle.getString("radioButton.connectMode2"));
		ConnectMode_2.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(ConnectMode_2, "cell 2 5,alignx left,growy");
		ConnectMode_Gp.add(ConnectMode_2);
		
		ConnectMode_3 = new JRadioButton(bundle.getString("radioButton.connectMode3"));
		ConnectMode_3.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(ConnectMode_3, "cell 3 5,alignx left,growy");
		ConnectMode_Gp.add(ConnectMode_3);
		
		FirstMove = new JLabel(bundle.getString("label.firstMove"));
		FirstMove.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(FirstMove, "cell 0 6,alignx right,growy");
		
		FirstMove_1 = new JRadioButton(bundle.getString("radioButton.firstMove1"));
		FirstMove_1.setSelected(true);
		FirstMove_1.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(FirstMove_1, "cell 1 6,alignx left,growy");
		FirstMove_Gp.add(FirstMove_1);
		
		FirstMove_2 = new JRadioButton(bundle.getString("radioButton.firstMove2"));
		FirstMove_2.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(FirstMove_2, "cell 2 6,alignx left,growy");
		FirstMove_Gp.add(FirstMove_2);
		
		MaxPly = new JLabel(bundle.getString("label.maxPly"));
		MaxPly.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(MaxPly, "cell 6 5,alignx right,growy");
		
		MaxPly_TextField = new JTextField();
		MaxPly_TextField.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		MaxPly_TextField.setColumns(10);
		GST_Panel.add(MaxPly_TextField, "cell 7 5,grow");
		
		ChangeFirstMove = new JLabel(bundle.getString("label.changeFirstMove"));
		ChangeFirstMove.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(ChangeFirstMove, "cell 0 7,alignx right,growy");
		
		ChangeFirstMove_1 = new JRadioButton(bundle.getString("radioButton.changeFirstMove1"));
		ChangeFirstMove_1.setSelected(true);
		ChangeFirstMove_1.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(ChangeFirstMove_1, "cell 1 7,alignx left,growy");
		ChangeFirstMove_Gp.add(ChangeFirstMove_1);
		
		ChangeFirstMove_2 = new JRadioButton(bundle.getString("radioButton.changeFirstMove2"));
		ChangeFirstMove_2.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(ChangeFirstMove_2, "cell 2 7,alignx left,growy");
		ChangeFirstMove_Gp.add(ChangeFirstMove_2);

		
		BoardPath = new JLabel(bundle.getString("label.boardPath"));
		BoardPath.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(BoardPath, "cell 6 7,alignx right,growy");
		
		BoardPath_TextField = new JTextField();
		BoardPath_TextField.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		BoardPath_TextField.setColumns(10);
		GST_Panel.add(BoardPath_TextField, "cell 7 7 2 1,grow");
		
		BoardPath_Button = new JButton(bundle.getString("button.boardPath"));
		BoardPath_Button.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 15));
		GST_Panel.add(BoardPath_Button, "cell 9 7,grow");
		
		BoardPath_Button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				File f = new File(ProgramPath.changePathToAbsolute(BoardPath_TextField.getText()));
				if(f.isFile()){
					fileChooser.setCurrentDirectory(f.getAbsoluteFile().getParentFile());
				}else if(f.isDirectory()){
					fileChooser.setCurrentDirectory(f.getAbsoluteFile());
				}else{
					fileChooser.setCurrentDirectory(new File(String.format("%s%sBoard%s%s", ProgramPath.getProgramPath(), File.separator, File.separator, GameInfo.getGameName())));
				}
				fileChooser.resetChoosableFileFilters();
				
				FileNameExtensionFilter txt = new FileNameExtensionFilter("TXT FILES", "txt");
				fileChooser.addChoosableFileFilter(txt);
				fileChooser.setFileFilter(txt);
				fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
				
				int selectValue = fileChooser.showDialog(GST_Panel, bundle.getString("dialog.select"));
				
				if(selectValue == JFileChooser.APPROVE_OPTION){
					File fl = fileChooser.getSelectedFile();
					BoardPath_TextField.setText(fl.getAbsolutePath());
				}
			}
		});

		AIPath = new JLabel(bundle.getString("label.aiPath"));
		AIPath.setHorizontalAlignment(SwingConstants.RIGHT);
		AIPath.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(AIPath, "cell 6 8,alignx right,growy");
		
		AIPath_TextField = new JTextField();
		AIPath_TextField.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		AIPath_TextField.setColumns(10);
		GST_Panel.add(AIPath_TextField, "cell 7 8 2 1,grow");
		
		AIPath_button = new JButton(bundle.getString("button.aiPath"));
		GST_Panel.add(AIPath_button, "cell 9 8,grow");
		AIPath_button.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 15));
		
		AIPath_button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				File f = new File(ProgramPath.changePathToAbsolute(AIPath_TextField.getText()));
				if(f.isFile()){
					fileChooser.setCurrentDirectory(f.getAbsoluteFile().getParentFile());
				}else{
					fileChooser.setCurrentDirectory(new File(String.format("%s%sSearch", ProgramPath.getProgramPath(), File.separator)));
				}
				fileChooser.resetChoosableFileFilters();
				fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
				int selectValue = fileChooser.showDialog(GST_Panel, bundle.getString("dialog.select"));//
				
				if(selectValue == JFileChooser.APPROVE_OPTION){
					File fl = fileChooser.getSelectedFile();
					AIPath_TextField.setText(fl.getAbsolutePath());
				}
			}
		});
		
		IP = new JLabel(bundle.getString("label.ip"));
		IP.setHorizontalAlignment(SwingConstants.RIGHT);
		IP.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(IP, "cell 0 9,alignx right,growy");
		
		IP_TextField = new JTextField();
		IP_TextField.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		IP_TextField.setColumns(10);
		GST_Panel.add(IP_TextField, "cell 1 9 3 1,grow");
		
//		JLabel label_48 = new JLabel("");
//		GST_Panel.add(label_48, "cell 8 8");
		
		PlyShare = new JLabel(bundle.getString("label.plyShare"));
		PlyShare.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		GST_Panel.add(PlyShare, "cell 6 3,alignx right,growy");
		
		PlyShare_TextField = new JTextField();
		PlyShare_TextField.setFont(new Font("Microsoft JhengHei", Font.PLAIN, 17));
		PlyShare_TextField.setColumns(10);
		GST_Panel.add(PlyShare_TextField, "cell 7 3,grow");
		
		AIArg = new JLabel(bundle.getString("label.aiArg"));
		AIArg.setHorizontalAlignment(SwingConstants.RIGHT);
		AIArg.setFont(new Font("微軟正黑體", Font.PLAIN, 17));
		GST_Panel.add(AIArg, "cell 6 9,alignx right,growy");
		
		AIArg_TextField = new HintTextField("( Empty )");
		AIArg_TextField.setFont(new Font("微軟正黑體", Font.PLAIN, 17));
		AIArg_TextField.setColumns(10);
		GST_Panel.add(AIArg_TextField, "cell 7 9 3 1,grow");
		
		RoomType_1.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				StartMode_1.setEnabled(true);
				StartMode_2.setEnabled(true);
				LocalServer_CheckBox.setEnabled(true);
				IP_TextField.setEnabled(!LocalServer_CheckBox.isSelected());
				
				if(StartMode_1.isSelected()){
					MidBoard_1.setEnabled(!LocalServer_CheckBox.isSelected());
					MidBoard_2.setEnabled(true);
					MidBoard_3.setEnabled(true);
					FirstMove_1.setEnabled(true);
					FirstMove_2.setEnabled(true);
					ChangeFirstMove_1.setEnabled(true);
					ChangeFirstMove_2.setEnabled(true);
					TimerMode_ComboBox.setEnabled(true);
					if(TimerMode_ComboBox.getSelectedIndex() == 4){
						TimeLimit_TextField.setEnabled(true);
						OnePlyTime_TextField.setEnabled(true);
						PlyShare_TextField.setEnabled(true);
					}
					RepeatTime_TextField.setEnabled(true);
					MaxPly_TextField.setEnabled(true);
					if(MidBoard_3.isSelected()){
						BoardPath_TextField.setEnabled(true);
						BoardPath_Button.setEnabled(true);
					}else{
						BoardPath_TextField.setEnabled(false);
						BoardPath_Button.setEnabled(false);
					}
				}else if(StartMode_2.isSelected()){
					MidBoard_1.setEnabled(false);
					MidBoard_2.setEnabled(false);
					MidBoard_3.setEnabled(false);
					FirstMove_1.setEnabled(false);
					FirstMove_2.setEnabled(false);
					ChangeFirstMove_1.setEnabled(false);
					ChangeFirstMove_2.setEnabled(false);
					TimerMode_ComboBox.setEnabled(false);
					TimeLimit_TextField.setEnabled(false);
					OnePlyTime_TextField.setEnabled(false);
					PlyShare_TextField.setEnabled(false);
					RepeatTime_TextField.setEnabled(false);
					MaxPly_TextField.setEnabled(false);
					BoardPath_TextField.setEnabled(false);
					BoardPath_Button.setEnabled(false);
				}
				AIPath_TextField.setEnabled(!ConnectMode_3.isSelected());
				AIPath_button.setEnabled(!ConnectMode_3.isSelected());
				AIArg_TextField.setEnabled(!ConnectMode_3.isSelected());
//				SetRedMode_1.setEnabled(ConnectMode_3.isSelected());
//				SetRedMode_2.setEnabled(ConnectMode_3.isSelected());
			}
		});
		
		RoomType_2.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				StartMode_1.setEnabled(false);
				StartMode_2.setEnabled(false);
				LocalServer_CheckBox.setSelected(false);
				LocalServer_CheckBox.setEnabled(false);
				MidBoard_1.setEnabled(false);
				MidBoard_2.setEnabled(false);
				MidBoard_3.setEnabled(false);
				FirstMove_1.setEnabled(false);
				FirstMove_2.setEnabled(false);
				ChangeFirstMove_1.setEnabled(false);
				ChangeFirstMove_2.setEnabled(false);
				IP_TextField.setEnabled(true);
				TimerMode_ComboBox.setEnabled(false);
				TimeLimit_TextField.setEnabled(false);
				OnePlyTime_TextField.setEnabled(false);
				PlyShare_TextField.setEnabled(false);
				RepeatTime_TextField.setEnabled(false);
				MaxPly_TextField.setEnabled(false);
				BoardPath_TextField.setEnabled(false);
				BoardPath_Button.setEnabled(false);
				AIPath_TextField.setEnabled(!ConnectMode_3.isSelected());
				AIPath_button.setEnabled(!ConnectMode_3.isSelected());
				AIArg_TextField.setEnabled(!ConnectMode_3.isSelected());
//				SetRedMode_1.setEnabled(ConnectMode_3.isSelected());
//				SetRedMode_2.setEnabled(ConnectMode_3.isSelected());
			}
		});

		StartMode_1.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				if(!LocalServer_CheckBox.isSelected()){
					MidBoard_1.setEnabled(true);
				}else{
					MidBoard_1.setEnabled(false);
				}
				MidBoard_2.setEnabled(true);
				MidBoard_3.setEnabled(true);
				RepeatTime_TextField.setEnabled(true);
				FirstMove_1.setEnabled(true);
				FirstMove_2.setEnabled(true);
				ChangeFirstMove_1.setEnabled(true);
				ChangeFirstMove_2.setEnabled(true);
				TimerMode_ComboBox.setEnabled(true);
				if(TimerMode_ComboBox.getSelectedIndex() == 4){
					TimeLimit_TextField.setEnabled(true);
					OnePlyTime_TextField.setEnabled(true);
					PlyShare_TextField.setEnabled(true);
				}
				if(MidBoard_3.isSelected()) {
					BoardPath_TextField.setEnabled(true);
					BoardPath_Button.setEnabled(true);
				}
				MaxPly_TextField.setEnabled(true);
			}
		});
		
		StartMode_2.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				MidBoard_1.setEnabled(false);
				MidBoard_2.setEnabled(false);
				MidBoard_3.setEnabled(false);
				RepeatTime_TextField.setEnabled(false);
				FirstMove_1.setEnabled(false);
				FirstMove_2.setEnabled(false);
				ChangeFirstMove_1.setEnabled(false);
				ChangeFirstMove_2.setEnabled(false);
				TimerMode_ComboBox.setEnabled(false);
				TimeLimit_TextField.setEnabled(false);
				OnePlyTime_TextField.setEnabled(false);
				PlyShare_TextField.setEnabled(false);
				if(MidBoard_3.isSelected()) {
					BoardPath_TextField.setEnabled(false);
					BoardPath_Button.setEnabled(false);
				}
				MaxPly_TextField.setEnabled(false);
//				SetRedMode_1.setEnabled(ConnectMode_3.isSelected());
//				SetRedMode_2.setEnabled(ConnectMode_3.isSelected());
			}
		});
		

		MidBoard_1.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				BoardPath_TextField.setEnabled(false);
				BoardPath_Button.setEnabled(false);
				FirstMove_1.setEnabled(false);
				FirstMove_2.setEnabled(false);
			}
		});
		
		MidBoard_2.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				BoardPath_TextField.setEnabled(false);
				BoardPath_Button.setEnabled(false);
				FirstMove_1.setEnabled(true);
				FirstMove_2.setEnabled(true);
			}
		});
		
		MidBoard_3.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				BoardPath_TextField.setEnabled(true);
				BoardPath_Button.setEnabled(true);
				FirstMove_1.setEnabled(true);
				FirstMove_2.setEnabled(true);
			}
		});

		ConnectMode_1.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				AIPath_TextField.setEnabled(true);
				AIPath_button.setEnabled(true);
				AIArg_TextField.setEnabled(true);
//				SetRedMode_1.setEnabled(false);
//				SetRedMode_2.setEnabled(false);
			}
		});
		
		ConnectMode_2.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				AIPath_TextField.setEnabled(true);
				AIPath_button.setEnabled(true);
				AIArg_TextField.setEnabled(true);
//				SetRedMode_1.setEnabled(false);
//				SetRedMode_2.setEnabled(false);
			}
		});
		
		ConnectMode_3.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				AIPath_TextField.setEnabled(false);
				AIPath_button.setEnabled(false);
				AIArg_TextField.setEnabled(false);
//				SetRedMode_1.setEnabled(true);
//				SetRedMode_2.setEnabled(true);
			}
		});
		
		
		// set fileChooser hot key
		InputMap im = fileChooser.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, InputEvent.ALT_MASK), "parentFolder");
		ActionMap am = fileChooser.getActionMap();
		am.put("parentFolder", new AbstractAction(){
			@Override
			public void actionPerformed(ActionEvent e) {
				fileChooser.changeToParentDirectory();
			}
		});
		
		// set setting file
		setting = new GST_SettingFile(
				Account_TextField, 
				Password_TextField, 
				RoomType_1, 
				RoomType_2,
				StartMode_1, 
				StartMode_2, 
				LocalServer_CheckBox, 
				MidBoard_1, 
				MidBoard_2, 
				MidBoard_3, 
				ConnectMode_1, 
				ConnectMode_2, 
				ConnectMode_3,  
				RepeatTime_TextField, 
				FirstMove_1, 
				FirstMove_2, 
				ChangeFirstMove_1, 
				ChangeFirstMove_2, 
				TimerMode_ComboBox, 
				TimeLimit_TextField, 
				OnePlyTime_TextField, 
				PlyShare_TextField,  
				MaxPly_TextField, 
				IP_TextField, 
				BoardPath_TextField, 
				BoardPath_Button, 
				AIPath_TextField, 
				AIPath_button,
				AIArg_TextField
				);
	}
	
	/**
	 * {@inheritDoc}
	 * Client 用來取得設定頁面的 JPanel。
	 * @return 配置好 component 的 JPanel
	 */
	@Override
	public JPanel getUIPanel() {
		return GST_Panel;
	}
	
	/**
	 * {@inheritDoc}
	 * 當使用者按下連線按鈕的時候，需要檢查輸入的值是否合法或者都完成了。
	 * @return 回傳錯誤需要顯示的字串，如果沒有錯誤回傳空字串("")
	 */
	@Override
	public String checkAllDone(Room_SettingFile room_setting) {
		if(Account_TextField.getText().length() <= 0){
			return "Account Field is Empty";
		}
		
		if(Password_TextField.getText().length() <= 0){
			return "Password Field is Empty";
		}
		
		if(RoomType_1.isSelected()){
			
			
			if(!StartMode_1.isSelected() && !StartMode_2.isSelected()){
				return "Please select the Start Mode";
			}
			
			if(StartMode_1.isSelected()){
				if(!MidBoard_1.isSelected() && !MidBoard_2.isSelected() && !MidBoard_3.isSelected()){
					return "Please select the MidBoard Mode";
				}
				
				if(RepeatTime_TextField.getText().length() <= 0){
					return "RepeatTime Field is Empty";
				}
				if(!RepeatTime_TextField.getText().matches("\\d+")){
					return "The Input of RepeatTime Field is not a number";
				}
				if(RepeatTime_TextField.getText().equals("0")){
					return "RepeatTime must larger than zero";
				}
				if(Long.parseLong(RepeatTime_TextField.getText()) > Long.valueOf(Integer.MAX_VALUE)){
					return "Value of RepeatTime Overflow";
				}
				
				if(TimeLimit_TextField.getText().length() <= 0){
					return "TimeLimit Field is Empty";
				}
				if(!TimeLimit_TextField.getText().matches("\\d+")){
					return "The Input of TimeLimit Field is not a number";
				}
				if(Long.parseLong(TimeLimit_TextField.getText()) > Long.valueOf(Integer.MAX_VALUE)){
					return "Value of TimeLimit Overflow";
				}
				
				if(OnePlyTime_TextField.getText().length() <= 0){
					return "OnePlyTime Field is Empty";
				}
				if(!OnePlyTime_TextField.getText().matches("\\d+")){
					return "The Input of OnePlyTime Field is not a number";
				}
				if(Long.parseLong(OnePlyTime_TextField.getText()) > Long.valueOf(Integer.MAX_VALUE)){
					return "Value of OnePlyTime Overflow";
				}
				
				if(PlyShare_TextField.getText().length() <= 0){
					return "PlyShare Field is Empty";
				}
				if(!PlyShare_TextField.getText().matches("\\d+")){
					return "The Input of PlyShare Field is not a number";
				}
				if(Long.parseLong(PlyShare_TextField.getText()) > Long.valueOf(Integer.MAX_VALUE)){
					return "Value of PlyShare Overflow";
				}
				
				if(MaxPly_TextField.getText().length() <= 0){
					return "MaxPly Field is Empty";
				}
				if(!MaxPly_TextField.getText().matches("\\d+")){
					return "The Input of MaxPly Field is not a number";
				}
				if(MaxPly_TextField.getText().equals("0")){
					return "MaxPly must larger than zero";
				}
				if(Long.parseLong(MaxPly_TextField.getText()) > Long.valueOf(Integer.MAX_VALUE)){
					return "Value of MaxPly Overflow";
				}
				
				if(!LocalServer_CheckBox.isSelected()){
					if(IP_TextField.getText().length() <= 0){
						return "IP Field is Empty";
					}
				}
			}
			
			if(MidBoard_3.isSelected() && StartMode_1.isSelected()){
				if(BoardPath_TextField.getText().length() <= 0){
					return "BoardPath Field is Empty";
				}
				
				if(!Paths.get(ProgramPath.changePathToAbsolute(BoardPath_TextField.getText())).toFile().isFile()){
					return "The Input of BoardPath Field is not a Path";
				}else{
					if(!Paths.get(ProgramPath.changePathToAbsolute(BoardPath_TextField.getText())).toFile().canRead()){
						return "The Input of BoardPath Field can not be read";
					}
				}
			}
			
			
			if(!ConnectMode_3.isSelected()){
				if(AIPath_TextField.getText().length() <= 0){
					return "AIPath Field is Empty";
				}
				if(!Paths.get(ProgramPath.changePathToAbsolute(AIPath_TextField.getText())).toFile().isFile()){
					return "The Input of AIPath Field is not a Path";
				}else{
					if(!Paths.get(ProgramPath.changePathToAbsolute(AIPath_TextField.getText())).toFile().canExecute()){
						return "Search Permission denied";
					}
				}
			}
			
			if(room_setting.boardOpenType == BoardOpenType.CLOSE &&
					ConnectMode_3.isSelected()){
				return "In order to use HUMAN mode, Please change the browser mode.";
			}
		}else if(RoomType_2.isSelected()){
			if(IP_TextField.getText().length() <= 0){
				return "IP Field is Empty";
			}
			
			if(!ConnectMode_3.isSelected()){
				if(AIPath_TextField.getText().length() <= 0){
					return "AIPath Field is Empty";
				}
				if(!Paths.get(ProgramPath.changePathToAbsolute(AIPath_TextField.getText())).toFile().isFile()){
					return "The Input of AIPath Field is not a Path";
				}else{
					if(!Paths.get(ProgramPath.changePathToAbsolute(AIPath_TextField.getText())).toFile().canExecute()){
						return "Search Permission denied";
					}
				}
			}
		}
		// no problem
		return "";
	}
	
	/**
	 * {@inheritDoc}
	 * 回傳此遊戲設定的 class。
	 * @return 此遊戲設定的 class
	 */
	@Override
	public SettingFile getSetting() {
		return setting;
	}

	/**
	 * {@inheritDoc}
	 * 回傳按下 Tab 之後 Component 需要 focus 的順序。
	 * 請注意：此 Array 具有順序性。
	 * @return Component 的 Array。
	 */
	@Override
	public Component[] getTabOrder(JButton Connect_Button) {
		return new Component[]{Account_TextField, Password_TextField, RepeatTime_TextField, IP_TextField, TimerMode_ComboBox, MaxPly_TextField, BoardPath_TextField, AIPath_TextField, Connect_Button};
	}
	
	@Override
	public void switchLanguage(Locale locale) {
		bundle = ResourceBundle.getBundle("geister", locale);
		Account.setText(bundle.getString("label.account"));
		TimerMode.setText(bundle.getString("label.timerMode"));
	    Password.setText(bundle.getString("label.password"));
	    TimeLimit.setText(bundle.getString("label.timeLimit"));
	    RoomType.setText(bundle.getString("label.roomType"));
	    RoomType_1.setText(bundle.getString("radioButton.roomType1"));
	    RoomType_2.setText(bundle.getString("radioButton.roomType2"));
	    OnePlyTime.setText(bundle.getString("label.timePerMove"));
	    StartMode.setText(bundle.getString("label.startMode"));
	    StartMode_1.setText(bundle.getString("radioButton.startMode1"));
	    StartMode_2.setText(bundle.getString("radioButton.startMode2"));
	    LocalServer_CheckBox.setText(bundle.getString("checkBox.localServer"));
	    MidBoard.setText(bundle.getString("label.midBoard"));
	    RepeatTime.setText(bundle.getString("label.repeatTime"));
	    ConnectMode.setText(bundle.getString("label.connectMode"));
	    FirstMove.setText(bundle.getString("label.firstMove"));
	    MaxPly.setText(bundle.getString("label.maxPly"));
	    ChangeFirstMove.setText(bundle.getString("label.changeFirstMove"));
	    MidBoard_1.setText(bundle.getString("radioButton.midBoard1"));
	    MidBoard_2.setText(bundle.getString("radioButton.midBoard2"));
	    MidBoard_3.setText(bundle.getString("radioButton.midBoard3"));
	    ConnectMode_1.setText(bundle.getString("radioButton.connectMode1"));
	    ConnectMode_2.setText(bundle.getString("radioButton.connectMode2"));
	    ConnectMode_3.setText(bundle.getString("radioButton.connectMode3"));
	    FirstMove_1.setText(bundle.getString("radioButton.firstMove1"));
	    FirstMove_2.setText(bundle.getString("radioButton.firstMove2"));
	    ChangeFirstMove_1.setText(bundle.getString("radioButton.changeFirstMove1"));
	    ChangeFirstMove_2.setText(bundle.getString("radioButton.changeFirstMove2"));
	    BoardPath.setText(bundle.getString("label.boardPath"));
	    AIPath.setText(bundle.getString("label.aiPath"));
	    IP.setText(bundle.getString("label.ip"));
	    PlyShare.setText(bundle.getString("label.plyShare"));
	    AIArg.setText(bundle.getString("label.aiArg"));
	    BoardPath_Button.setText(bundle.getString("button.boardPath"));
	    AIPath_button.setText(bundle.getString("button.aiPath"));
	}
}
