package gst;

import java.util.ArrayList;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

import client.Packet;
import client.SettingFile;
import client.Warning;

/**
 * 用來建立 Search 的 template，用來實作不同的連接模式的。
 * @author ldslds449
 * 
 */
public abstract class SearchConnect {
	
	protected long STDOUT_READ_DELAY = 0;
	protected long STDERR_READ_DELAY = 0;
	
	protected long STDOUT_READ_DELAY_INC = 50;
	
	protected ArrayList<Integer> stdout_stamp = new ArrayList<Integer>();
	protected ArrayList<Integer> stderr_stamp = new ArrayList<Integer>();

	protected ReadWriteLock stdout_lock = new ReentrantReadWriteLock();
	protected ReadWriteLock stderr_lock = new ReentrantReadWriteLock();
	
	protected StringBuilder connect_Info = new StringBuilder();
	protected StringBuilder error_Info = new StringBuilder();
	
	protected boolean finish = false;
	protected static int stdThreadPriority = 7;
	protected static int errThreadPriority = 7;
	
	final char pieceKind[] = {'-', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'X', 'a', 'b', 'c', 'd', 'e', 'f','g' ,'h'};
	
	abstract protected Packet askMove(String ghostCodeInfo) throws Warning;
	
	abstract protected Packet askSetRed() throws Warning;
	
	abstract protected void sendMove(String data, int capturePieceColor, int noEatFlip, int step_time, int total_spent_time, int time, int oppoTime, boolean isLast) throws Warning;
	
	abstract protected void sendMessage(String data, int capturePieceColor, boolean isMoveAck) throws Warning;
	
	abstract protected void setGameInfo(SettingFile setting, String midboard, int myTime, int[] player_times, Integer[] step_time, Integer[] total_spent_time) throws Warning;
	
	abstract protected void gameStart(int playerNum, boolean firstMove) throws Warning;
	
	abstract protected void stopSearch();
	
	abstract protected String getInteractiveLog();
	
	/**
	 * 
	 * 用來取得目前程式輸出的所有標準輸出訊息。
	 * 
	 * @return 目前程式輸出所有標準輸出訊息。
	 */
	String getConnectInfo(){
		return connect_Info.toString();
	}
	
	/**
	 * 
	 * 用來取得目前程式輸出的所有錯誤輸出訊息。
	 * 
	 * @return 目前程式輸出所有錯誤輸出訊息。
	 */
	String getErrorInfo(){
		return error_Info.toString();
	}
	
	/**
	 * 
	 * 取得標示每個走步所標準輸出的內容行數戳記。
	 * 
	 * @return log 檔案中用來大約標示每個走步所標準輸出的內容行數戳記。
	 */
	protected String getStdoutStamps(){
		stdout_lock.readLock().lock();
		
		StringBuilder sb = new StringBuilder();
		sb.append("# ");
		synchronized(stdout_stamp){
			for(int index : stdout_stamp){
				sb.append(index);
				sb.append(' ');
			}
		}
		sb.append("#");
		
		stdout_lock.readLock().unlock();
		return sb.toString();
	}
	
	/**
	 * 
	 * 取得標示每個走步所錯誤輸出的內容行數戳記。
	 * 
	 * @return log 檔案中用來大約標示每個走步所錯誤輸出的內容行數戳記。
	 */
	protected String getStderrStamps(){
		stderr_lock.readLock().lock();
		
		StringBuilder sb = new StringBuilder();
		sb.append("# ");
		synchronized(stderr_stamp){
			for(int index : stderr_stamp){
				sb.append(index);
				sb.append(' ');
			}
		}
		sb.append("#");
		
		stderr_lock.readLock().unlock();
		return sb.toString();
	}
}
