// languages.js
const resources = {
    zh: {
        translation: {
            //html
            "switch": "語言切換",
            "view_history": "觀看歷史盤面：",
            "player_first": "先手：",
            "player_second": "後手：",
            "opponent_disagree": "對手不同意",
            "opponent_draw": "對手求和",
            "opponent_draw_ask": "對方提和，請問你要同意嗎?",
            "agree": "同意",
            "disagree": "不同意",
            "cannot_use": "不能使用此功能",
            "draw": "提和",
            "resign": "認負",
            "set_finish": "擺盤完成",
            "draw_ask": "請問你確定要提和嗎?",
            "yes": "是",
            "no": "否",
            "resign_ask": "請問你確定要認負嗎?",
            "abort": "中止",
            "abort_ask": "請問你確定要中止嗎?",
            "set_point": "設置點數",
            "set_point_to": "設置點數為:",
            "submit": "送出",
            "clear": "清除",
            "input_error": "輸入錯誤，請重新輸入",
            //board.js
            "step": "走步",
            "endgame": "殘局",
            "hideP2": " 隱藏對手棋子",
            "hideP1": " 隱藏自己棋子",
            "user_stdout": "使用者標準輸出自動換行:",
            "user_stderr": "使用者標準錯誤輸出自動換行:",
        },
    },
    en: {
        translation: {
            "switch": "Switch Language",
            "view_history": "History Board:",
            "player_first": "First Player:",
            "player_second": "Second Player:",
            "opponent_disagree": "Opponent Disagree",
            "opponent_draw": "Opponent Draw",
            "opponent_draw_ask": "Opponent ask draw, do you agree?",
            "agree": "Agree",
            "disagree": "Disagree",
            "cannot_use": "Cannot use this function",
            "draw": "Draw",
            "resign": "Resign",
            "set_finish": "Set Finish",
            "draw_ask": "Do you want to draw?",
            "yes": "Yes",
            "no": "No",
            "resign_ask": "Do you want to resign?",
            "abort": "Abort",
            "abort_ask": "Do you want to abort?",
            "set_point": "Set Point",
            "set_point_to": "Set Point to:",
            "submit": "Submit",
            "clear": "Clear",
            "input_error": "Input Error, please input again",
            //board.js
            "step": "Step",
            "endgame": "Endgame",
            "hideP2": " Hide enemy",
            "hideP1": " Hide myself",
            "user_stdout": "User standard output word wrap:",
            "user_stderr": "User standard error output word wrap:",

        }
    },
    jp: {
        translation: {
            "switch": "言語選択",
            "view_history": "履歴を表示：",
            "player_first": "先手：",
            "player_second": "後手：",
            "opponent_disagree": "相手が否定しました",
            "opponent_draw": "相手の引き分け",
            "opponent_draw_ask": "相手が引き分けを申し出たが、同意しますか？",
            "agree": "同意",
            "disagree": "否定",
            "cannot_use": "この機能は使用できません",
            "draw": "引き分け",
            "resign": "降参",
            "set_finish": "配置完了",
            "draw_ask": "引き分けを申し出をしますか？",
            "yes": "はい",
            "no": "いいえ",
            "resign_ask": "本当に降参しますか？",
            "abort": "中止",
            "abort_ask": "本当に中止しますか？",
            "set_point": "ポイント設定",
            "set_point_to": "ポイント設定値:",
            "submit": "提交",
            "clear": "消除",
            "input_error": "入力エラー、再入力してください",
            //board.js
            "step": "指し手",
            "endgame": "終盤",
            "hideP2": "相手の駒を隠す",
            "hideP1": "自分の駒を隠す",
            "user_stdout": "標準出力自動改行:",
            "user_stderr": "標準エラー出力自動改行:",

        }
    }
    // 如需更多語言，繼續添加
};

async function initializeI18next() {
    // 從本地儲存中讀取已儲存的語言設定
    const savedLanguage = sessionStorage.getItem('selectedLanguage') || 'en';
    await i18next.init({
        lng: savedLanguage, // 預設語言
        debug: true, // 啟用除錯模式
        resources: resources, // 語言資源
    });
}

i18next.on('languageChanged', function (lng) {
    console.log('Language changed to ' + lng);
    // 將選擇的語言儲存在本地儲存中
    sessionStorage.setItem('selectedLanguage', lng);

    document.querySelectorAll('.i18n').forEach(element => {
        const key = element.getAttribute('i18next_key');
        element.innerHTML = i18next.t(key);
    });


    console.log("finish");
})
function bindLanguageSelector() {
    const languageSelector = document.getElementById('language-selector');
    if (languageSelector) {
        languageSelector.addEventListener('change', function () {
            i18next.changeLanguage(this.value);
        });
    }
}

document.addEventListener('DOMContentLoaded', function () {
    initializeI18next().then(() => {
        bindLanguageSelector();
    });
});